/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       EncField.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Encoding of a NodeField object
-------------------------------------------------------------------------*/
#include "defs.h"
#include "NodeFld.h"
#include "Quantize.h"
#include "Token.h"
#include "BifsEnc.h"

/*-----------------------------------------------------------------
Encode an SFInt
-----------------------------------------------------------------*/
BOOL SF<int> :: Encode (BIFSEncoder *pBifs,  Quantizer *pQuant)
{
	/*-----------------------------------------------------------------
	Look at the Quantizer to see if min-max values applied
	-----------------------------------------------------------------*/
	m_value = nextToken.GetInt ();
	pQuant -> EncodeInt (pBifs, m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFBool
-----------------------------------------------------------------*/
BOOL SF<Bool> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value = nextToken.GetBool ();
	pBifs -> EncodeInt (m_value, 1, "value");
	return TRUE;
}

/*-----------------------------------------------------------------
Encode and SFFloat
-----------------------------------------------------------------*/
BOOL SF<float> :: Encode (BIFSEncoder *pBifs,  Quantizer * /*pQuant*/)
{
	m_value = nextToken.GetReal ();
	pBifs -> EncodeFloat (m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFTime
-----------------------------------------------------------------*/
BOOL SF<Time> :: Encode (BIFSEncoder *pBifs,  Quantizer * /*pQuant*/)
{
	m_value = nextToken.GetReal ();
	pBifs -> EncodeDouble (m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFVec3f
-----------------------------------------------------------------*/
BOOL SF<Vec3f> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value.x = nextToken.GetReal ();
	m_value.y = nextToken.GetReal ();
	m_value.z = nextToken.GetReal ();
	pBifs -> EncodeFloat (m_value.x);
	pBifs -> EncodeFloat (m_value.y);
	pBifs -> EncodeFloat (m_value.z);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFVec2f
-----------------------------------------------------------------*/
BOOL SF<Vec2f> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value.x = nextToken.GetReal ();
	m_value.y = nextToken.GetReal ();
	pBifs -> EncodeFloat (m_value.x );
	pBifs -> EncodeFloat (m_value.y );
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFRotation
-----------------------------------------------------------------*/
BOOL SF<Rotation> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value.x = nextToken.GetReal ();
	m_value.y = nextToken.GetReal ();
	m_value.z = nextToken.GetReal ();
	m_value.theta = nextToken.GetReal ();
	pBifs -> EncodeFloat (m_value.x);
	pBifs -> EncodeFloat (m_value.y);
	pBifs -> EncodeFloat (m_value.z);
	pBifs -> EncodeFloat (m_value.theta);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFRotation2D
-----------------------------------------------------------------*/
BOOL SF<Rotation2D> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value.x = nextToken.GetReal ();
	m_value.y = nextToken.GetReal ();
	m_value.theta = nextToken.GetReal ();
	pBifs -> EncodeFloat (m_value.x);
	pBifs -> EncodeFloat (m_value.y);
	pBifs -> EncodeFloat (m_value.theta);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFColor
-----------------------------------------------------------------*/
BOOL SF<Color3f> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value.r = nextToken.GetReal ();
	m_value.g = nextToken.GetReal ();
	m_value.b = nextToken.GetReal ();
	pBifs -> EncodeFloat (m_value.r);
	pBifs -> EncodeFloat (m_value.g);
	pBifs -> EncodeFloat (m_value.b);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFObjectID
-----------------------------------------------------------------*/
BOOL SFObjectID :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value = nextToken.GetInt ();
	pBifs -> EncodeInt (m_value, 10, "value");
	return TRUE;
}

/*-----------------------------------------------------------------
Encode SFString
-----------------------------------------------------------------*/
BOOL SF<ZString> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	char szBuffer [10];

	m_value = nextToken.GetString ();
	nextToken.Scan ();
	pBifs -> EncodeInt (m_value.GetLength (), 8, "value.size");
	for (int i = 0; i < m_value.GetLength (); ++i)
		pBifs -> EncodeInt (
			m_value [i],
			8,
			(ZString ("value [") += itoa (i, szBuffer, 10)) += ']'
		);
	return TRUE;
}

/*-----------------------------------------------------------------
A dummy function
-----------------------------------------------------------------*/
BOOL SF<MediaObject *> :: Encode (BIFSEncoder *, Quantizer *)
{
	return FALSE;
}

/*-----------------------------------------------------------------
Encode a generic node
-----------------------------------------------------------------*/
BOOL SFGenericNode :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	/*-----------------------------------------------------------------
	Check if "USE" case
	-----------------------------------------------------------------*/
	BOOL bIsReused = nextToken == TID_USE;
	pBifs -> EncodeInt (bIsReused, 1, "isReused");

	if (bIsReused)
	{	nextToken.Scan ();
		if (nextToken == TID_NAME)
		{	int nNodeID = nextToken.GetDefinedName ();
			if (nNodeID >= 0)
				pBifs -> EncodeInt (nNodeID, 10, "nodeID");
			else
				nextToken.Error ("Name not found");
			nextToken.Scan ();
			return TRUE;
		}
		else
		{	nextToken.Error ("Unexpected symbol");
			nextToken.Scan ();
			return FALSE;
		}
	}

	/*-----------------------------------------------------------------
	Check if "DEF" is used
	-----------------------------------------------------------------*/
	BOOL bIsUpdateable = nextToken == TID_DEF;
	int nID = -1;

	if (bIsUpdateable)
	{	nextToken.Scan ();
		if (nextToken == TID_NAME)
		{	nID = nextToken.GetDefinedName ();
			if (nID < 0)
				nID = nextToken.AddPredefinedName ();
			else
			{	nextToken.Error ("Name already defined");
				nID = -1;
			}
			nextToken.Scan ();
		}
		else
		{	nextToken.Error ("Unexpected symbol");
			nextToken.Scan ();
		}
	}
				
	/*-----------------------------------------------------------------
	Determine the node type, instantiate the object, and let it
	Encode itself
	-----------------------------------------------------------------*/
	if (nextToken != TID_NAME)
	{	nextToken.Error ("Unexpected symbol");
		nextToken.Scan ();
		return FALSE;
	}

	int nNodeType;
	m_pNode = CreateNode (nextToken.GetName (), nNodeType);
	if (!m_pNode)
		nextToken.Error ("Unrecognized node type");
	if (nNodeType < 0)
		nextToken.Error ("Unexpected node type");

	nextToken.Scan ();
	if (!m_pNode)
		return FALSE;

	pBifs -> EncodeInt (
		nNodeType,
		BitFieldWidth (GetNodeTableSize ()),
		"nodeType"
	);
	pBifs -> EncodeInt (bIsUpdateable, 1, "isUpdateable");
	if (bIsUpdateable)
	{	pBifs -> EncodeInt (nID, 10, "nodeID");
		if (nID >= 0)
			nextToken.SetNodeID (nID, m_pNode -> GetNodeCode ());
	}
	
	BOOL bResult = m_pNode -> Encode (pBifs);
	delete m_pNode;
	return bResult;
}

/*-----------------------------------------------------------------
Create a node whose name is szNodeName. To do this try to instantiate
all nodes, and for each try if this is the required one.
Return the found nodeType in rNodeType.
-----------------------------------------------------------------*/
MediaObject *SFGenericNode :: CreateNode (
	LPCSTR szNodeName,
	int &rNodeType
)
{
	for (int nCode = 0; nCode < GetNodeTableSize (); nCode ++)
		if (m_bifsFactory [nCode] != NULL)
		{	MediaObject *pNode = m_bifsFactory [nCode] -> CreateNode ();
			if (!strcmp (pNode -> GetName (), szNodeName))
			{	rNodeType = nCode;
				return pNode;
			}
		}
	return NULL;
}

/*-----------------------------------------------------------------
Encode an MF field
-----------------------------------------------------------------*/
BOOL MField :: Encode (BIFSEncoder *pBifs, Quantizer *pQuant)
{
	pBifs -> EncodeInt (1, 1, "isListDescription");

	if (nextToken != TID_LEFT_BRACKET)
	{	nextToken.Error ("Unexpected symbol");
		nextToken.Scan ();
		return FALSE;
	}
	nextToken.Scan ();

	do
	{
		EncodeElement (pBifs, pQuant);
		if (nextToken == TID_COMMA)
			nextToken.Scan ();
		pBifs -> EncodeInt (
			nextToken != TID_RIGHT_BRACKET,
			1,
			"moreFields"
		);
	} while (nextToken != TID_RIGHT_BRACKET);
	nextToken.Scan ();	
	return TRUE;
}

/*-----------------------------------------------------------------
The default implementation of EncodeElement can be called only by 
error
-----------------------------------------------------------------*/
BOOL NodeField :: EncodeElement (BIFSEncoder *, Quantizer *)
{
	nextToken.Error ("Not an MF field");
	return FALSE;
}