#if !defined(_Token_h)	// Sentry, use file only if it's not already included.
#define _Token_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       Token.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of Token, the class that handles text parsing.
-------------------------------------------------------------------------*/
#include "ztl.h"

typedef enum tagTokenCodes
{
	TID_UNKNOWN,
	TID_LEFT_BRACE,
	TID_RIGHT_BRACE,
	TID_LEFT_BRACKET,
	TID_RIGHT_BRACKET,
	TID_COMMA,
	TID_PERIOD,
	TID_STRING,
	TID_USE,
	TID_DEF,
	TID_FALSE,
	TID_TRUE,
	TID_ROUTE,
	TID_TO,
	TID_AT,
	TID_INSERT,
	TID_REPLACE,
	TID_DELETE,
	TID_APPEND,
	TID_LAST,
	TID_SCENE,
	TID_BY,
	TID_NUMBER_OF_RESERVED,
	TID_NAME = TID_NUMBER_OF_RESERVED,
	TID_NUMBER,
	TID_INTEGER,
	TID_EOF
} TOKEN_CODE;

class MediaObject;

class Token
{
	TOKEN_CODE m_code;
	ZString m_string;
	int m_cNext, m_nInputOffset, m_nInputLength;
	FILE *m_fInput, *m_fList;
	char m_szLine [300];
	static LPCSTR m_reserved [];

	struct DefinedNode
	{
		ZString m_name;
		int m_nCode;
		BOOL operator == (const DefinedNode &) const {return FALSE;}
	};

	ZArray<DefinedNode> m_defined;
	ZArray<ZString> m_routes;

public:
	void Init (FILE *fInput, FILE *fList);
	operator int () {return m_code;}
	int GetInt ();
	BOOL GetBool ();
	double GetReal ();
	LPCSTR GetName () {return m_string;}
	LPCSTR GetString ();
	void Scan ();
	int GetDefinedName () const;
	int GetDefinedRoute () const;
	void Error (LPCSTR szDesc);
	int AddPredefinedName ();
	int AddPredefinedRoute ();
	void SetNodeID (int nIndex, int nNodeCode)
	{
		m_defined [nIndex].m_nCode = nNodeCode;
	}
	int GetNodeID (int nIndex) const {return m_defined [nIndex].m_nCode;}
private:
	int GetChar ();
};

class ExceptionEndOfFile {};

#ifndef MAIN
extern Token nextToken;
#endif	// MAIN

#endif                                      // _Token_h sentry.
