//
// This code has been taken from that written by Mike Coleman and
// presented at Stockholm.  The changes have been to remove
// references to the OS class, add the code which writes the
// audio data format into the media stream buffer, and remove the
// Run() method, using the base class's method.
//
// Matthew Leditschke
// CSELT
//

#include "defs.h"
#include "AACDecoder.h"
#include <assert.h>
#include <limits.h>

#include "Caps.h"
#include "Stream.h"

#include "Executiv.h"
#include "Presenter-CSELT.h"


AACDecoder::AACDecoder( Executive *pExecutive )
  : Decoder(pExecutive), m_targetBitrate(64000), m_targetSampleRate(44100)
{
	m_firstDecode = TRUE;
	m_PUlen = 1024 * sizeof(short);
	m_timeStampIncrement = 1.0e6 * (1024.0 / m_targetSampleRate);
	m_timeStamp = 0.0;
	const char* path;
	path = "aacTool.dll";

	m_pluginHandle = ::LoadLibrary( path );
	assert(m_pluginHandle != 0);

	m_decodeFrame     = (long(*)(void*, void*, int, void*, long))  ::GetProcAddress( (HMODULE)m_pluginHandle, "decodeFrame" );
	m_initInstance    = (int (*)(void**)) ::GetProcAddress( (HMODULE)m_pluginHandle, "initInstance" );
	m_releaseInstance = (int (*)(void*))  ::GetProcAddress( (HMODULE)m_pluginHandle, "releaseInstance" );

	int res = (*m_initInstance)( &m_pluginInstanceData );
}

AACDecoder::~AACDecoder()
{
	if ( m_releaseInstance != 0 )
		(*m_releaseInstance)( m_pluginInstanceData );
}

int AACDecoder::GetOptimalInputSize()
{
	return (m_targetBitrate * 1024) / m_targetSampleRate;
}

int AACDecoder::GetOptimalOutputSize()
{
	return m_PUlen + sizeof(DecoderAudioFormat);
}

int AACDecoder::GetRecommendedInputBufferSize()
{
	return 2 * GetOptimalInputSize();
}

int AACDecoder::GetRecommendedOutputBufferSize()
{
	return 480 * GetOptimalOutputSize();
}

FORMAT AACDecoder::SetFormat( Capabilities *pCaps )
{
	int nFormats = pCaps->GetFormatCapabilityLength();
	FORMAT ret = FORMAT_PCM;

	for ( int i = 0; i < nFormats; i++ )
		if (pCaps->GetFormatCapability(i) == FORMAT_PCM )
		{
			ret = FORMAT_PCM;
			break;
		}
	return ret;
}		

//#define SILENCE_PLEASE

BOOL AACDecoder::Decode( LPBYTE AUptr, int AUlen, LPBYTE PUptr, int &PUlen )
{
	if ( PUlen < m_PUlen )
	{
		PUlen = m_PUlen;
		return FALSE;
	}

	int headerSize = 0;
	if (m_firstDecode)
	{
		m_firstDecode = FALSE;

		DecoderAudioFormat* pFmt = (DecoderAudioFormat*)PUptr;
		pFmt->samplesPerSecond = 44100;
		pFmt->bitsPerSample	   = 16;
		pFmt->numChannels	   = 1;
		headerSize = sizeof(DecoderAudioFormat);
	}

#ifdef SILENCE_PLEASE
	::memset( PUptr + headerSize, m_PUlen, 0 );
	PUlen = m_PUlen + headerSize;
#else
	PUlen = (*m_decodeFrame)( m_pluginInstanceData, AUptr, AUlen, PUptr + headerSize, PUlen - headerSize);
	if ( PUlen == 0 )
	{
		::memset( PUptr + headerSize, m_PUlen, 0 );
		PUlen = m_PUlen + headerSize;
	}
	else
		PUlen += headerSize;
#endif
	return PUlen > headerSize;
}
