///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of tlis software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Appearance2DProxy.cpp
//
// Authors: Antonio Arena, CSELT, arena@kim.cselt.it
//			Franco Casalino, CSELT, Franco.Casalino@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Appearance2D node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Appearance2DProxy.h"
#include "Material2DProxy.h"
#include "Properties2DProxy.h"


// =====================================================================

//
// PresenterCSELT::InitAppearance2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitAppearance2D(Appearance2D* pObj)
{
	pObj->SetPresenterData(new Appearance2DProxy(pObj));
}


// =====================================================================

//
// Appearance2DProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void Appearance2DProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "Appearance2D {\n");
	const char* indentStr = IndentPrefix(depth + 1);

	fprintf(outfile, "%smaterial ", indentStr);
	if (GetMaterial2D() == NULL)
		fprintf(outfile, "NULL\n");
	else
		((Material2DProxy*)(GetMaterial2D()->GetPresenterData()))->Print(outfile, depth + 1);

	fprintf(outfile, "%sproperties ", indentStr);
	if (GetProperties2D() == NULL)
		fprintf(outfile, "NULL\n");
	else
		((Properties2DProxy*)(GetProperties2D()->GetPresenterData()))->Print(outfile, depth + 1);

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* Appearance2DProxy::Print */


// =====================================================================
