///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: AudRend-CSELT.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of an abstract base class which performs audio
//   rendering.  Actual implementations of the audio renderer will
//   derive from this class.
//
// Created: August, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "AudRend-CSELT.h"

// Include the details of the Windows 95/NT specific audio renderers
#ifdef WIN32
#include "AudRend-CSELT-DS.h"
#endif

// Include the details of the SGI specific audio renderers
#if sgi
#include "AudRend-CSELT-Sgi.h"
#endif

// Include the details of the sun specific audio renderers
#if sun
#endif


// ============================================================================

//
// AudioRenderer::Create
//
// Return a pointer to an audio renderer.  Use this to construct
// an audio render appropriate for the current platform.  A pointer
// to the presenter, and a platform specific parameter are given.
//

AudioRenderer* AudioRenderer::Create(Presenter* pPresenter,
									 void* pInfo)
{
	AudioRenderer* pAudRend;

// Construct a Windows 95/NT specific audio renderer
#ifdef WIN32
	pAudRend = new AudioRendererDS(pPresenter, (HWND)pInfo);
#endif

// Construct an SGI specific audio renderer
#if sgi
	pAudRend = new AudioRendererSgi(pPresenter);
#endif

// Construct a sun specific audio renderer
#if sun
#endif

	pAudRend->m_pPresenter = pPresenter;

	return pAudRend;
} /* AudioRenderer::Create */


// ============================================================================
