///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  ColorProxy.cpp
//
// Author: Antonio Arena, CSELT, arena@kim.cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Color node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "ColorProxy.h"
#include "trace.h"
#include <assert.h>

// =====================================================================

//
// PresenterCSELT::InitColor
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitColor(Color* pObj)
{
	pObj->SetPresenterData(new ColorProxy(pObj));
}


// ======================================================================

//
// ColorProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void ColorProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "Circle {\n");
	const char* indentStr = IndentPrefix(depth + 1);

	fprintf(outfile, "%scolor [", indentStr);
	for (int i = 0; i < GetColor()->color.GetLength(); i++)
		fprintf(outfile, " %.4f %.4f %.4f",
				(float)GetColor()->color[i].r,
				(float)GetColor()->color[i].g,
				(float)GetColor()->color[i].b);
	fprintf(outfile, " ]\n");

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* ColorProxy::Print */

// ======================================================================
