///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: Copy.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of functions common to the copying of both
//   RGBA and YUVA images to the offscreen RGB surfaces in the
//   visual renderer.
//
// Created: April, 1997
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "VisRend.h"
#include "Copy.h"

#define MIN3(x,y,z) (((x) < (y)) ? min((x),(z)) : min((y),(z)))


// ========================================================================

//
// SetUpCopyParameters
//
// This function sets up the copy parameters for each of the image
// copying functions.  If TRUE is returned, no copying needs to be
// performed.
//

BOOL SetUpCopyParameters(CopyParameters& cpParams,
						 SrcImage const& srcImage,
						 SurfaceDesc const& surfaceDesc)
{
	// The pitch must be at least the width
	assert (srcImage.pitch >= srcImage.width);

	// Determine the region of the source frame which needs to be copied.
	//  - take into account offset and dimensions of destination surface.
	cpParams.srcStartX = (srcImage.left < 0) ? (-1 * srcImage.left) : 0;
	cpParams.srcStartY = (srcImage.top < 0)  ? (-1 * srcImage.top)  : 0;
	cpParams.copyWidth  = MIN3(srcImage.width - cpParams.srcStartX,
							   surfaceDesc.dwWidth - srcImage.left,
							   surfaceDesc.dwWidth);
	cpParams.copyHeight = MIN3(srcImage.height - cpParams.srcStartY,
							   surfaceDesc.dwHeight - srcImage.top,
							   surfaceDesc.dwHeight);

	// Determine the top-left point in the destination frame
	cpParams.destStartX = (srcImage.left < 0) ? 0 : srcImage.left;
	cpParams.destStartY = (srcImage.top < 0)  ? 0 : srcImage.top;

	// Currently, the copy width must be even
//	assert ((cpParams.copyWidth % 2) == 0);

	// Only proceed if the image will actually appear
	if ((cpParams.srcStartX >= srcImage.width) ||
		(cpParams.srcStartY >= srcImage.height) ||
		(srcImage.left >= (int)surfaceDesc.dwWidth) ||
		(srcImage.top >= (int)surfaceDesc.dwHeight))
		return TRUE;
	else
		return FALSE;
} /* SetUpCopyParameters */


// ========================================================================
