///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: Dai.cpp
//
// Authors: Nuno Delgado, CSELT
//
// Description:
//
///////////////////////////////////////////////////////////////////////////


// Prevent inclusion of winsock.h in windows.h
#ifndef _WINSOCKAPI_
#define _WINSOCKAPI_
#endif

#include "winsock2.h"


#include "defs.h"
#include "Dai.h"
#include "DaiSession.h"
#include "DmifPeer.h"
#include "TransMuxChannel.h"
#include "Remote.h"
#include "fmt.h"




CRITICAL_SECTION ReceiveNetInfoCriticalSection;


/*
/***************************************************************************************

	AcceptDmifThreadFunction

	Thread function that is called by the server side of DMIF to accept a connection
	from a client and to establish the Dmif control channel with the client

***************************************************************************************/
BOOL AcceptDmifThreadFunction(IN LPVOID ParamPtr)
{
	struct sockaddr address;
	int addr_size = sizeof(struct sockaddr);
	AcceptDmifThreadStruct* StructPtr;
	Dai* pDaiObject;
	CDmifPeer* pDmifPeer;

	// Lets cast this paramater
	StructPtr = (AcceptDmifThreadStruct*)ParamPtr;
	pDaiObject = StructPtr->pDai;

	// Create the DmifPeer object
	pDmifPeer = (CDmifPeer*) new CDmifPeer;

	// Accept the connection from the remote peer
	pDmifPeer-> m_hDmifCtrlSock = WSAAccept(StructPtr->sock, &address, &addr_size, 
		                                                               NULL, NULL);
	if((pDmifPeer-> m_hDmifCtrlSock) == INVALID_SOCKET)
	{
		pDaiObject->IdentifyError("AcceptDmifThreadFunction-WSAAccept: ",
			                                            WSAGetLastError());
		delete pDmifPeer;
	}

	closesocket(StructPtr->sock);  // Free the Temp Socket

	
	// Create the session
	DaiSession* pDaiSessionObj;
	unsigned int NewSession;
	int Response;

	pDaiObject->DA_SessionCreate(NewSession,Response);
	pDaiObject->FindSessionInList(NewSession, &pDaiSessionObj);
	pDmifPeer->FillDaiAndDaiSessionInfo(pDaiObject,pDaiSessionObj);


	// I must receive the ClientDescriptor info before going further
	Conversion_Union conv;
	DWORD BytesReceived, BytesSent;
	unsigned int ClientDesc;
	char aux[CONTROL_MSG_SIZE];

	// Lets received the Client Descriptor
	if(!pDaiObject->MyRecv(pDmifPeer-> m_hDmifCtrlSock, aux, CONTROL_MSG_SIZE, BytesReceived))
		return FALSE;

	memcpy(conv.data,aux,4);
	ClientDesc = (unsigned int)conv.int_number;

	struct sockaddr_in* addr_aux;
	addr_aux = (sockaddr_in*)(&address);


	//Lets continue to fill the Peer member variables and add it to the linked list
	strcpy(pDmifPeer -> m_pDmifPeerAddress,inet_ntoa(addr_aux->sin_addr));
	pDmifPeer -> m_ServiceId = StructPtr->ServiceId;
	pDaiSessionObj->AddPeerToList(pDmifPeer);

	// Call the callback function to inform the Dmif server of the session attach
	pDaiObject->DA_SessionAttachIndication(NewSession,inet_ntoa(addr_aux->sin_addr), 
		                                  StructPtr->ServiceId,ClientDesc);


	// This packet does not contain anything. Is just to synchronize the client code that
	// have to wait for the conclusion of DA_SessionAttachIndication (at the server side) before
	// being able to issue DA_ChannelAddRequest at the client side
	if(!pDaiObject->MySend(pDmifPeer-> m_hDmifCtrlSock, aux, CONTROL_MSG_SIZE, BytesSent))
		return FALSE;


	if(!pDmifPeer->AssociateSocketWithEvent())
		return FALSE;

	// Lets create the Thread that will wait for the event
	DWORD ThreadId;
	ChannelNegotiationThreadStruct* pStruct;

	pStruct = (ChannelNegotiationThreadStruct*) new ChannelNegotiationThreadStruct;
	pStruct -> pDai = pDaiObject;
	pStruct -> pSession = pDaiSessionObj;
	pStruct -> pDmifPeer = pDmifPeer;

	// Lets create the thread
	if(CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)ChannelNegotiationThreadFunction,
		                                                   pStruct,0,&ThreadId) == NULL)
		MessageBox(GetActiveWindow(),"Error on ChannelNegotiationThreadFunction thread creation","",MB_OK);


	// delete the structure that was passed to the thread
	delete StructPtr;

	return TRUE;
}




/***************************************************************************************

	ChannelEstablishmentThreadFunction

	This thread function is called by the server side to establish the data channels
	with the client

***************************************************************************************/
BOOL ChannelEstablishmentThreadFunction(IN LPVOID ParamPtr)
{

	AuxiliarThreadStruct* pThreadStruct;
	
	pThreadStruct = (AuxiliarThreadStruct*) ParamPtr;

	// I will create an array of  ( (NumOfElements/2) x 6)    var: Table
	// The first column contain the QoS of each stream. The second column contain the
	// TransMuxAssociationTag of the transmux that will transport this Elementary Stream. 
	// The third column contains the FlexMux channel number of the FlexMux that 
	// transports the stream. The fourth column, contains the channelAssociationTag 
	// attributed to each elementary stream. The fifth column will have the port numbers 
	// at the server
	// Finaly, the sixth column will have the value of the socket handle

	int (*Table)[6] = new int[(((pThreadStruct->pData)->NumOfElements)/2)][6];


	// Fill the QoS column
	for(int i=0; i < ((pThreadStruct->pData)->NumOfElements)/2; i++)
		Table[i][0] = *((pThreadStruct->pData)->pBuffer + (i*2));
	

	// Fill the rest of the table
	(pThreadStruct->pSession)->FillTableOfTags(Table,
		                                 ((pThreadStruct->pData)->NumOfElements)/2);
	
	
	//Establish the connections
	(pThreadStruct->pDai)->EstablishConnectionsServerSide(Table, 
	                      ((pThreadStruct->pData)->NumOfElements)/2, 
		                  pThreadStruct->pSession,pThreadStruct->pDmifPeer);
	

	// Send the FMC descriptors to the client side
	(pThreadStruct->pDai)->SendFmcDescriptors(Table,((pThreadStruct->pData)->NumOfElements)/2,
											pThreadStruct->pSession,pThreadStruct->pDmifPeer);



	// ALTERATION TO GIVE TO THE DMIF SERVER THE SOCKET HANDLES AND FLEXMUX NUMBERS
	// Send back the ChannelAssociationTags + FlexMuxChannels + SocketHandle
	DATA* pData;

	pData = (DATA*) new DATA;
	pData->NumOfElements = (((pThreadStruct->pData)->NumOfElements)/2*3);
	pData->pBuffer = new int[pData->NumOfElements];
	
	for(int j=0; j<(pData->NumOfElements)/3; j++)
	{
		*(pData->pBuffer + 3*j) = Table[j][3];
		*(pData->pBuffer + (3*j+1)) = Table[j][2];
		*(pData->pBuffer + (3*j+2)) = Table[j][5];
	}

	(pThreadStruct->pDai)->DA_ChannelSetupConfirmation(pData);

	
	delete pData->pBuffer;
	delete pData;

	// delete the Table
	delete [] Table;
		
	// delete the structure that was passed to the thread
	delete (pThreadStruct->pData)->pBuffer;
	delete pThreadStruct->pData;
	delete pThreadStruct;

	return TRUE;
}



/***************************************************************************************

	ChannelNegotiationThreadFunction

	This thread is "attached" at both ends of the DMIF control channel, in the server
	and in the client, to receive/send information through this control channel

***************************************************************************************/
BOOL ChannelNegotiationThreadFunction(IN LPVOID ParamPtr)
{
	ChannelNegotiationThreadStruct* pThreadStruct;
	DWORD WaitStatus;
	WSANETWORKEVENTS NetworkEvents;
	int Result;
	BOOL Forever = TRUE;
	CDmifPeer* pPeer;
	
	pThreadStruct = (ChannelNegotiationThreadStruct*) ParamPtr;
	pPeer = pThreadStruct -> pDmifPeer;

	do
	{
		WaitStatus = WSAWaitForMultipleEvents(pPeer->m_NumEvents,
			                                  pPeer->m_EventArray,
											  FALSE,
											  WSA_INFINITE,TRUE);
		switch(WaitStatus)
		{
		case WSA_WAIT_EVENT_0:   // Something occured on the socket
			Result = WSAEnumNetworkEvents(pPeer->m_hDmifCtrlSock, 
                                         pPeer->m_EventArray[0], &NetworkEvents);
			if (Result == SOCKET_ERROR) 
			{
				(pThreadStruct->pDai)->IdentifyError("WSAEnumNetworkEvents1",WSAGetLastError());
				Forever = FALSE;
			}   
			else
				if(	NetworkEvents.lNetworkEvents  & FD_READ)
					(pThreadStruct->pDai)->RcvMsgInDmifChannel(pThreadStruct->pDmifPeer, 
						                                         pThreadStruct->pSession);
			break;

		case (WSA_WAIT_EVENT_0 + 1):    // Its the control event to terminate thread
			Forever = FALSE;
			break;

		case WSA_WAIT_FAILED:
			//(pThreadStruct->pDai)->IdentifyError("WSAWaitForMultipleEvents",WSAGetLastError());
			Forever = FALSE;
			break;

		}  // End of the switch

	}while(Forever);

	delete pThreadStruct;  // Delete the structure that was passed to the Thread

	return TRUE;
}


/***************************************************************************************

	ExchangeDataThreadFunction

	Thread function called by the DMIF client for receiving data in all DATA channels.
	Each time a new DATA channel is created, the firts event of the event array is 
	signaled to refresh the event array and include the new created channel

***************************************************************************************/
BOOL ExchangeDataThreadFunction(IN LPVOID ParamPtr)
{


	WSAEVENT* pEventArray;
	ExchangeDataThreadStruct* StructPtr;
	DWORD WaitStatus;
	BOOL Forever = TRUE;
	int NumSockEvents; 
	int j;
	
	
	// Lets cast this paramater
	StructPtr = (ExchangeDataThreadStruct*)ParamPtr;

	// Determine the number of events associated with sockets
	NumSockEvents = ((StructPtr->pSession)->m_WsaEventArray).GetLength();

	// Create the Event Array to support all socket events plus the end event
	// and the refresh table event
	pEventArray = (WSAEVENT*) new WSAEVENT[NumSockEvents+2];
	pEventArray[0] = (StructPtr->pSession)->m_hRefreshTableEvent;
	pEventArray[1] = (StructPtr->pSession)->m_hEndThreadEvent;

	for(j = 0; j < NumSockEvents; j++)
		pEventArray[j+2] = (StructPtr->pSession)->m_WsaEventArray[j];


	InitializeCriticalSection(&ReceiveNetInfoCriticalSection);

	do
	{
		WaitStatus = WSAWaitForMultipleEvents(NumSockEvents+2,pEventArray,
			                                      FALSE,WSA_INFINITE,TRUE);


		switch(WaitStatus)
		{
			case WSA_WAIT_FAILED:
				(StructPtr->pDai)->IdentifyError("WSAWaitForMultipleEvents ",
					                                       WSAGetLastError());
				Forever = FALSE;
				break;

			case WSA_WAIT_EVENT_0:    // The refresh table event was signaled.

				// We have to reload the table of events
				NumSockEvents = ((StructPtr->pSession)->m_WsaEventArray).GetLength();
				delete pEventArray;

				// Create the New Event Array to support all socket events 
				//plus the control event
				pEventArray = (WSAEVENT*) new WSAEVENT[NumSockEvents+1];

				// Reload the new socket events
				pEventArray[0] = (StructPtr->pSession)->m_hRefreshTableEvent;
				pEventArray[1] = (StructPtr->pSession)->m_hEndThreadEvent;
				for(j = 0; j < NumSockEvents; j++)
					pEventArray[j+2] = (StructPtr->pSession)->m_WsaEventArray[j];
			
				break;

			case (WSA_WAIT_EVENT_0 + 1):
					Forever = FALSE;
					break;  // breaks the switch and ends the thread
				
			default:
				if ( (WaitStatus > (WSA_WAIT_EVENT_0+1)) &&
				     (WaitStatus  <= (WSA_WAIT_EVENT_0 + NumSockEvents +1)))
				{
					EnterCriticalSection(&ReceiveNetInfoCriticalSection);
					(StructPtr->pDai)->HandleEventsOnDataSocket(pEventArray[WaitStatus]);
					LeaveCriticalSection(&ReceiveNetInfoCriticalSection);
				}
				else
					MessageBox(GetActiveWindow(),"WSAWaitForMultipleEvents Unexpected event",
					                                                      "DmifLayer",MB_OK);
				break;
			
		}
	}while(Forever);

	// delete the structure that was passed to the thread
	delete StructPtr;
	DeleteCriticalSection(&ReceiveNetInfoCriticalSection);

	return TRUE;
}




/***************************************************************************************

	WaitForServiceClientThreadFunction

	This thread function waits for a client to connect. When a client contacts the 
	NetDaemon of the server, the NetDaemon signals this UDP socket and the DMIF
	control channel is created

***************************************************************************************/
BOOL WaitForServiceClientThreadFunction(IN LPVOID ParamPtr)
{

	ServiceClientThreadStruct* StructPtr;
	Conversion_Union conversion;
	WSABUF WsaBuf;
	char BufAux[CONTROL_MSG_SIZE];
	DWORD BytesReceived, BytesSent;
	DWORD Flags = 0;
	DWORD ThreadId;
	struct sockaddr PeerAddr;
	int PeerAddrLength = sizeof(struct sockaddr);
	Dai* pDaiLayer;
	unsigned short PortUsed;
	BOOL Forever = TRUE;
	DWORD WaitStatus;
	WSAEVENT EventArray[2];
	WSANETWORKEVENTS NetworkEvents;
	int Result;

	WsaBuf.buf = BufAux;
	WsaBuf.len = CONTROL_MSG_SIZE;

	// Lets cast this paramater
	StructPtr = (ServiceClientThreadStruct*)ParamPtr;
	pDaiLayer = StructPtr->pDai;

	// Create the event for the socket and the event array
	EventArray[0] = (WSAEVENT)CreateEvent(NULL, FALSE, FALSE, NULL);
	EventArray[1] = (StructPtr->hControlEndEvent);


	// Create the Udp socket
	if(!pDaiLayer->CreateUdpServiceSocket(StructPtr->ServicePort))
		goto end;
	
	// Put the sockets in asynchronous mode
	if(WSAEventSelect(pDaiLayer->m_hUdpServiceSocket,
		               (WSAEVENT)EventArray[0],FD_READ) == SOCKET_ERROR)
	{
		pDaiLayer->IdentifyError("ConnectTransChannelToServer - WSAEventSelect",
		                                                         WSAGetLastError());
		goto end;
	}

	do
	{

		WaitStatus = WSAWaitForMultipleEvents(2,EventArray,FALSE,WSA_INFINITE,TRUE);
		switch(WaitStatus)
		{
			case WSA_WAIT_EVENT_0:   // Something occured on the socket
			
				Result = WSAEnumNetworkEvents(pDaiLayer->m_hUdpServiceSocket, 
												EventArray[0], &NetworkEvents);
				if (Result == SOCKET_ERROR) 
				{
					pDaiLayer->IdentifyError("WSAEnumNetworkEvents6",WSAGetLastError());
					goto end;
				}   

				if(	NetworkEvents.lNetworkEvents  & FD_READ)
				{
					if(WSARecvFrom(pDaiLayer->m_hUdpServiceSocket,&WsaBuf,1,
					 &BytesReceived,&Flags,&PeerAddr,&PeerAddrLength,NULL,NULL) == SOCKET_ERROR)
					{
						pDaiLayer->IdentifyError("WaitForServiceClientThreadFunction - WSARecvFrom ",
		                                                              WSAGetLastError());
						goto end;
					}
			
					// Lets retrieve the DMIF control port number
					memcpy(conversion.data,WsaBuf.buf,2);
			
					// Create the structure
					AcceptDmifThreadStruct* pAcceptStruct;
					pAcceptStruct = new AcceptDmifThreadStruct;
					pAcceptStruct->pDai = pDaiLayer;
					pAcceptStruct->ServiceId = StructPtr -> ServiceId;

					// Lets try to create the Tcp listen socket	
					if(!pDaiLayer->CreateAndBindTcpSocket(pAcceptStruct->sock, 
					                                   conversion.ushort_number, PortUsed))
					{

						conversion.int_number = MSG_ERROR;
						memcpy(WsaBuf.buf,conversion.data,4);
						if(WSASendTo(pDaiLayer->m_hUdpServiceSocket,&WsaBuf,1,&BytesSent,0,
						            &PeerAddr,PeerAddrLength,NULL,NULL) == SOCKET_ERROR)
						{
							pDaiLayer->IdentifyError("WaitForServiceClientThreadFunction-WSASendTo ",
			                                                              WSAGetLastError());
	
							closesocket(pAcceptStruct->sock);  // Free the Temp Socket
							delete pAcceptStruct;
							continue;
						}
					}

					// Put the socket listening
					if(listen(pAcceptStruct->sock,SOMAXCONN) == SOCKET_ERROR)
					{
						pDaiLayer->IdentifyError("WaitForServiceClientThreadFunction-Listening: ",
					                                                    WSAGetLastError());
						closesocket(pAcceptStruct->sock);  // Free the Temp Socket
						delete pAcceptStruct;
						continue;
					}
	
					// The socket was successfully created. Lets send back its port number and
					//lets put it listening
					conversion.int_number = MSG_OK;
					memcpy(WsaBuf.buf,conversion.data,4);
					conversion.ushort_number = PortUsed;
					memcpy((WsaBuf.buf)+4,conversion.data,2);
					if(WSASendTo(pDaiLayer->m_hUdpServiceSocket,&WsaBuf,1,&BytesSent,0,&PeerAddr,
						                        PeerAddrLength,NULL,NULL) == SOCKET_ERROR)
					{
						pDaiLayer->IdentifyError("WaitForServiceClientThreadFunction - WSASendTo ",
			                                                              WSAGetLastError());
	
						closesocket(pAcceptStruct->sock);  
						delete pAcceptStruct;
						continue;
					}
	
					if(CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)AcceptDmifThreadFunction,
					                                       pAcceptStruct,0,&ThreadId) == NULL)
						MessageBox(GetActiveWindow(),
					             "Error on AcceptDmifThreadFunction thread creation","",MB_OK);
				}	
				break;
	
			case (WSA_WAIT_EVENT_0 + 1):    // Its the control event to terminate thread
				Forever = FALSE;
				break;

			case WSA_WAIT_FAILED:
				pDaiLayer->IdentifyError("WSAWaitForMultipleEvents",WSAGetLastError());
				break;

		}  // End of the switch

	}while(Forever);


end:
	delete StructPtr;
	CloseHandle((HANDLE)EventArray[0]);
	return TRUE;
}


/****************************************************************************************

****************************************************************************************/
DLLExport Dai::Dai()
{
	// Put the pointer for installed protocols pointing for NULL
	m_pInstalledProtocols = NULL;
	
	// Pointers for UDP and TCP protocol info
	m_pUdpProtocolInfo = NULL;
	m_pTcpProtocolInfo = NULL;

	// Num of protocols = 0
	m_NumProtocols = 0;

	// Reset the session number counter 
	m_SessionNumber = 0;

	pHeadListOfSessionsElement = NULL;
	pHeadListOfClientTransMuxElements = NULL;
	pHeadListOfServerTransMuxElements = NULL;

	// Initialize the critical sections
	InitializeCriticalSection(&mSessionIdCriticalSection);
	InitializeCriticalSection(&mClientTransMuxListCriticalSection);
	InitializeCriticalSection(&mServerTransMuxListCriticalSection);
	InitializeCriticalSection(&mDeliverInfoCriticalSection);

	// Event to control the end of the thread
	m_hEventWaitForClientThreadEnd = (WSAEVENT)CreateEvent(NULL, FALSE, FALSE, NULL);

}


/****************************************************************************************

****************************************************************************************/
DLLExport Dai::Dai(RemoteService* pRemoteService)
{
	// Put the pointer for installed protocols pointing for NULL
	m_pInstalledProtocols = NULL;
	
	// Pointers for UDP and TCP protocol info
	m_pUdpProtocolInfo = NULL;
	m_pTcpProtocolInfo = NULL;

	// Num of protocols = 0
	m_NumProtocols = 0;

	// Reset the session number counter 
	m_SessionNumber = 0;

	pHeadListOfSessionsElement = NULL;
	pHeadListOfClientTransMuxElements = NULL;
	pHeadListOfServerTransMuxElements = NULL;

	// Initialize the critical sections
	InitializeCriticalSection(&mSessionIdCriticalSection);
	InitializeCriticalSection(&mClientTransMuxListCriticalSection);
	InitializeCriticalSection(&mServerTransMuxListCriticalSection);
	InitializeCriticalSection(&mDeliverInfoCriticalSection);

	// For the client we don't need to create the event
	m_hEventWaitForClientThreadEnd = 0;

	m_pRemService = pRemoteService;
}





/****************************************************************************************

****************************************************************************************/
DLLExport Dai::~Dai()
{
	// Destroy the event if it exists
	if(m_hEventWaitForClientThreadEnd != 0)
		CloseHandle((HANDLE)m_hEventWaitForClientThreadEnd);

	// Free all existing DaiSessions
	DestroyAndRemoveAllSessionsFromList();

	// Destroy the TransMuxChannels os TransMuxChannels
	RemoveAndDestroyAllTransMuxChannels(SERVER_SIDE);
	RemoveAndDestroyAllTransMuxChannels(CLIENT_SIDE);

	// Delete critical section
	DeleteCriticalSection(&mSessionIdCriticalSection);
	DeleteCriticalSection(&mClientTransMuxListCriticalSection);
	DeleteCriticalSection(&mServerTransMuxListCriticalSection);
	DeleteCriticalSection(&mDeliverInfoCriticalSection);

}




/****************************************************************************************

****************************************************************************************/
BOOL Dai::InitWS2()
{

    int           Error;              // catches return value of WSAStartup
    WORD          VersionRequested;   // passed to WSAStartup
    WSADATA       WsaData;            // receives data from WSAStartup
    BOOL          ReturnValue = TRUE; // return value
    
    // Start WinSock 2.  If it fails, we don't need to call WSACleanup().
    VersionRequested = MAKEWORD(VERSION_MAJOR, VERSION_MINOR); 
    Error = WSAStartup(VersionRequested, &WsaData);
    if (Error)
		ReturnValue = FALSE;
    else
	{
        // Now confirm that the WinSock 2 DLL supports the exact version
        // we want. If not, make sure to call WSACleanup().
        if (LOBYTE(WsaData.wVersion) != VERSION_MAJOR ||
            HIBYTE(WsaData.wVersion) != VERSION_MINOR) 
		{
            WSACleanup();
            ReturnValue = FALSE;
        }
    }
    return(ReturnValue);
}



/****************************************************************************************

  This method tests the Winsock version (InitWS2) and if this is t
  he correct one the protocol structure is filled (FindProtocols)

****************************************************************************************/
BOOL Dai::Initialization()
{
	// Initialize Winsock2
	if(!InitWS2())
	{
		MessageBox(GetActiveWindow(),"Error initializing WinSock library","",MB_OK);
		return FALSE;
	}

	//Find existing protocols
	if(!FillProtocolsStruct())
	{
		MessageBox(GetActiveWindow(),"Error finding protocol structure","",MB_OK);
		return FALSE;
	}


	// Verify if TCP and UDP protocol exist
	if(!IsTcpProtocolAvailable())
	{
		MessageBox(GetActiveWindow(),"Tcp protocol is not available","",MB_OK);
		return FALSE;
	}

	if(!IsUdpProtocolAvailable())
	{
		MessageBox(GetActiveWindow(),"Udp protocol is not available","",MB_OK);
		return FALSE;
	}

	return TRUE;
}
		




/****************************************************************************************

	Dai::FreeWS2

	This method just calls WSACleanup() to free WS2.dll resources

****************************************************************************************/
void Dai::FreeWS2()
{
	WSACleanup();
}





/****************************************************************************************

Dai::FillProtocolsStruct()

 This method uses WSAEnumProtocols to find out about all
installed protocols on the local machine.  It stores this
information in two variables of this class;
InstalledProtocols is a pointer to a buffer of WSAPROTOCOL_INFO
structs, while NumProtocols is the number of protocols in that
buffer.  This function is the only function in the file allowed to
touch these variables. 
Returns:
	TRUE - Successfully initialized the protocol buffer.
	FALSE - Some problem exist.  The user is informed of the 
	        error.

****************************************************************************************/
BOOL Dai::FillProtocolsStruct()
{

    DWORD BufferSize = 0;       // size of InstalledProtocols buffer
    
    // Call WSAEnumProtocols to figure out how big of a buffer we need.
    m_NumProtocols = WSAEnumProtocols(NULL, NULL, &BufferSize); 
    if ((m_NumProtocols != SOCKET_ERROR) && (WSAGetLastError() != WSAENOBUFS))
        // We're in trouble!!
        return(FALSE);


    m_pInstalledProtocols = (LPWSAPROTOCOL_INFO)new char[BufferSize];
    if (m_pInstalledProtocols == NULL)
        return(FALSE);
    

	m_NumProtocols = WSAEnumProtocols(NULL, (LPWSAPROTOCOL_INFO)m_pInstalledProtocols, 
		                                                                  &BufferSize);
    if (m_NumProtocols == SOCKET_ERROR)        // uh-oh
        return(FALSE);
    

	return(TRUE);
}




/****************************************************************************************

  Dai::IsTcpProtocolAvailable

  This method tests if the TCP protocol is available in the
  installed protocol list. If so, returns TRUE, and a pointer
  for the protocol info is returned as an output parameter.
  Otherwise will return FALSE

****************************************************************************************/
BOOL Dai::IsTcpProtocolAvailable()
{
	LPWSAPROTOCOL_INFO pAux;

	for(int i = 0; i < m_NumProtocols; i++)
	{
		pAux = &m_pInstalledProtocols[i];

		//If it is from AF_INET family
		if(pAux -> iAddressFamily == AF_INET)
			// If it is Connection Oriented
			if (!(pAux->dwServiceFlags1 & XP1_CONNECTIONLESS))
				// If guarantees delivery order
				if(pAux->dwServiceFlags1 & XP1_GUARANTEED_ORDER)
				{	
					m_pTcpProtocolInfo = pAux;
					//*pProtocolInfo = pAux;
					return TRUE;
				}
	}
	
	// The TCP Protocol is not available
	m_pTcpProtocolInfo = NULL;

	return FALSE;
}



/****************************************************************************************

  Dai::IsUdpProtocolAvailable

  This method tests if the Udp protocol is available in the
  installed protocol list. If so, returns TRUE, and a pointer
  for the protocol info is returned as an output parameter.
  Otherwise will return FALSE

****************************************************************************************/
BOOL Dai::IsUdpProtocolAvailable()
{
	LPWSAPROTOCOL_INFO pAux;

	for(int i = 0; i < m_NumProtocols; i++)
	{
		pAux = &m_pInstalledProtocols[i];

		//If it is from AF_INET family
		if(pAux -> iAddressFamily == AF_INET)
			// If it is Connectionless 
			if (pAux->dwServiceFlags1 & XP1_CONNECTIONLESS)
				// If doen't guarantees delivery order
				if(pAux->dwServiceFlags1 & XP1_MESSAGE_ORIENTED)
				{	
					m_pUdpProtocolInfo = pAux;
					//*pProtocolInfo = pAux;
					return TRUE;
				}
	}
	
	// The Udp Protocol is not available
	m_pUdpProtocolInfo = NULL;

	return FALSE;
}


/***************************************************************************************

	ReleaseResources

****************************************************************************************/
void Dai::ReleaseResources()
{
	//UnregistService
	DA_ServiceUnregister();

	// Free the space allocated to the installed protocols
	if(m_pInstalledProtocols != NULL)
		delete (LPWSAPROTOCOL_INFO)m_pInstalledProtocols;

	// Free Winsock2
	FreeWS2();

}



/***************************************************************************************

	DA_SessionCreate

	Creates a new session and adds it to the linked list of sessions

***************************************************************************************/
void Dai::DA_SessionCreate(unsigned int& NewSessionId, int& response)
{
	// Initialize response with error value
	response = ERROR;

	
	// Create new DaiSession object
	DaiSession* pNewSession;
	pNewSession = new DaiSession(this);
	pNewSession->m_SessionID = m_SessionNumber;

	
	// Add it to the linked list
	AddSessionToList(pNewSession);

	// Fill the output parameters
	NewSessionId = m_SessionNumber;
	response = OK;
		
	// Increment the session number parameter
	m_SessionNumber++;

}



/***************************************************************************************

	DA_SessionDestroy

	Destroy a session removing it form the linked list of sessions

***************************************************************************************/
void Dai::DA_SessionDestroy(unsigned int SessionId, int /*reason*/, int& response)
{
	if(DestroyAndRemoveSessionFromList(SessionId))
		response = TRUE;
	else
		response = FALSE;
}



/***************************************************************************************

	DA_SessionDestroyResponse

	Destroy a session removing it form the linked list of sessions

***************************************************************************************/
void Dai::DA_SessionDestroyResponse(unsigned int SessionId, int reason, int& response)
{
	// Set event to close the server thread that waits for clients
	SetEvent(m_hEventWaitForClientThreadEnd);

	// Call the method DA_SessionDestroy
	DA_SessionDestroy(SessionId, reason, response);
}



/***************************************************************************************

	DA_SessionAttachRequest

	Attach a session to a specific service in a specifc server. The Dmif control channel
	to that server is established and is created a thread that will be "waiting" for the
	control information send by the server

***************************************************************************************/
BOOL Dai::DA_SessionAttachRequest(unsigned int SessionId,char* PeerAddress, 
					              unsigned int ServiceId, unsigned int ClientDescriptor) 
{
	DaiSession* pSession;
	SOCKET newSocket;
	int AddrLength;
	struct sockaddr_in SockAddr;
	CDmifPeer* pNewDmifPeer;
	unsigned short RemotePortNumber;
	Conversion_Union conv;
	char aux[CONTROL_MSG_SIZE];
	DWORD BytesSent, BytesReceived;


	// First lets find the session wiyh this ID
	if(!FindSessionInList(SessionId, &pSession)) // There is no session with this number
		return FALSE;

	// Lets see if UDP protocol stack is available
	if(m_pUdpProtocolInfo == NULL)
		return FALSE;

	// Alocate space for the address structures
	AddrLength = m_pUdpProtocolInfo->iMaxSockAddr;
	memset((char*)&SockAddr,0,AddrLength);

	// Create socket
	newSocket = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO,
		                  m_pUdpProtocolInfo,0,WSA_FLAG_OVERLAPPED);
	
	// Verify if the socket was correctly created
	if(newSocket == INVALID_SOCKET)
	{
		IdentifyError("DA_SessioAttachRequest - WSASocket: ",WSAGetLastError());
		return FALSE;
	}

	// Fill the address structure
	FillLocalSockAddrStruct(&SockAddr, WELL_KNOWN_UDP_PORT);

	// Lets use the connect on this UDP socket because it will send/receive always to/from
	// the same address
	if(WSAConnect(newSocket,(struct sockaddr*)&SockAddr,
		              AddrLength,NULL,NULL,NULL,NULL) == SOCKET_ERROR)
	{
		IdentifyError("DA_SessioAttachRequest - WSAConnect: ",WSAGetLastError());
		closesocket(newSocket);
		return FALSE;
	}


	if(!ContactLocalNetDaemon(newSocket,PeerAddress, ServiceId, RemotePortNumber ))
	{
		closesocket(newSocket);
		return FALSE;
	}

	//Free the Temporary Udp socket
	closesocket(newSocket);


	// Now lets connect to the remote listening DMIF socket in order to establish
	// the DMIF control channel. At this time, lets create a Peer object and add it to list
	// of peers that exist under this section;

	pNewDmifPeer = (CDmifPeer*) new CDmifPeer(this, pSession);
	strcpy(pNewDmifPeer->m_pDmifPeerAddress,PeerAddress);
	pNewDmifPeer->m_ServiceId = ServiceId;
	if(!EstablishDmifControlChannel(&(pNewDmifPeer->m_hDmifCtrlSock),
		                                                   PeerAddress,RemotePortNumber))
	{
		delete pNewDmifPeer;
		return FALSE;
	}

	// Add the peer to the linked list
	pSession->AddPeerToList(pNewDmifPeer);

	// Lets exchange the ClientDescriptor information
	conv.int_number = ClientDescriptor;
	memcpy(aux,conv.data,4);

	if(!MySend(pNewDmifPeer->m_hDmifCtrlSock,aux,CONTROL_MSG_SIZE,BytesSent))
		return FALSE;


	// The purpose of this receive is to guarantee that the remote side has already
	// issue the DA_SessionAttachIndication with success. In fact the packet does
	//not contain any information
	if(!MyRecv(pNewDmifPeer->m_hDmifCtrlSock,aux,CONTROL_MSG_SIZE,BytesReceived))
		return FALSE;



	if(!pNewDmifPeer->AssociateSocketWithEvent())
		return FALSE;


	// Lets create the Thread that will wait for the event
	DWORD ThreadId;
	ChannelNegotiationThreadStruct* pStruct;

	pStruct = (ChannelNegotiationThreadStruct*) new ChannelNegotiationThreadStruct;
	pStruct -> pDai = this;
	pStruct -> pSession = pSession;
	pStruct -> pDmifPeer = pNewDmifPeer;

	// Lets create the thread
	if(CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)ChannelNegotiationThreadFunction,
		                                                   pStruct,0,&ThreadId) == NULL)
		MessageBox(GetActiveWindow(),"Error on ChannelNegotiationThreadFunction thread creation","",MB_OK);

	return TRUE;
}



/***************************************************************************************

	DA_SessionDetachRequest

	Destroys th session and removes all peers attached to it

***************************************************************************************/
BOOL Dai::DA_SessionDetachRequest(unsigned int SessionId, char* PeerAddress, 
								  unsigned int /*ServiceId*/, unsigned int /*reason*/) 
{
	DaiSession* pSession;
	CDmifPeer* pPeer;
	char BuffAux[CONTROL_MSG_SIZE];
	DWORD SentBytes;
	Conversion_Union conv;


	// First lets find the session wiyh this ID
	if(!FindSessionInList(SessionId, &pSession)) // There is no session with this number
		return FALSE;

	// Lets find if the peer exists and in this case, get the DmifCtrlSock Id
	if(!pSession->FindPeerInList(PeerAddress,&pPeer))
		return FALSE;

	// Inform the opposite side
	conv.int_number = SESSION_DETACH_REQUEST;
	memcpy(BuffAux,conv.data,4);
	MySend(pPeer->m_hDmifCtrlSock,BuffAux,CONTROL_MSG_SIZE,SentBytes);


	//Remove all peers connected to this session
	pSession->RemoveAndDestroyPeerFromList(PeerAddress);

	return TRUE;
}

/***************************************************************************************

	DA_ChannelAddRequest

	Sends to the Dmif server the request for channel to transmit the Elementary Streams
	specified as input parameters

***************************************************************************************/
void Dai::DA_ChannelAddRequest(unsigned int SessionId, char* PeerAddress, DATA* pUuData)
{
	DaiSession* pSession;
	CDmifPeer* pDmifPeer;


	// First lets find the session wiyh this ID
	if(!FindSessionInList(SessionId, &pSession)) // There is no session with this number
		return;

	// Lets find if the peer exists and in this case, get the DmifCtrlSock Id
	if(!pSession->FindPeerInList(PeerAddress,&pDmifPeer))
		return;

	// Send to the server, through the Dmif control channel the Es_Ids
	if(!SendEsIds(pDmifPeer->m_hDmifCtrlSock,pUuData))
		return;
}



/***************************************************************************************


	FillLocalSockAddrStruct
	
	This method fills the local sockaddr_in structure with the local address
	of the host and with the port number requested


***************************************************************************************/
BOOL Dai::FillLocalSockAddrStruct(struct sockaddr_in* pSockAddr, unsigned short port)
{

	hostent* pHostEnt;
	char HostName[128];

	// Fill the address structure
	if(gethostname(HostName,128) == SOCKET_ERROR)
		return FALSE;	

	pHostEnt = gethostbyname(HostName);
	if(pHostEnt == NULL)
		return FALSE;
	


	(pSockAddr->sin_addr).S_un.S_addr = *((u_long *)pHostEnt->h_addr_list[0]);
	pSockAddr->sin_family = AF_INET;
	pSockAddr->sin_port = port;

	return TRUE;
}




/***************************************************************************************

	DA_ServiceRegister
	
	This method is called by the Dmif server side and regists a new service on the 
	NetDaemon. Then creates a UDP socket waiting for receiving packets at the port that 
	the NetDaemon specified.

***************************************************************************************/
BOOL Dai::DA_ServiceRegister(unsigned int NewService)
{
	SOCKET TempSocket;
	int TempSockAddrLength;
	struct sockaddr_in TempSockAddr;
	unsigned short NewPortNbr;


	// Lets see if UDP protocol stack is available
	if(m_pUdpProtocolInfo == NULL)
		return FALSE;

	// Alocate space for the address structures
	TempSockAddrLength = m_pUdpProtocolInfo->iMaxSockAddr;
	memset((char*)&TempSockAddr,0,TempSockAddrLength);

	// Create socket
	TempSocket = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO,
		                  m_pUdpProtocolInfo,0,WSA_FLAG_OVERLAPPED);
	if(TempSocket == INVALID_SOCKET)
	{
		IdentifyError("WSASocket: ",WSAGetLastError());
		return FALSE;
	}

	// Fill the address of the local NetDaemon at the well knwon UDP port
	FillLocalSockAddrStruct(&TempSockAddr, WELL_KNOWN_UDP_PORT);

	
	// I will use this connect on the UDP socket because the destination of all my
	// UDP packets is the same for all them
	if(WSAConnect(TempSocket,(struct sockaddr*)&TempSockAddr,
		              TempSockAddrLength,NULL,NULL,NULL,NULL) == SOCKET_ERROR)
	{
		IdentifyError("WSAConnect: ",WSAGetLastError());
		closesocket(TempSocket);
		return FALSE;
	}

	if(!RegistService(TempSocket,NewService,NewPortNbr))
	{
		closesocket(TempSocket);
		return FALSE;
	}


	// Lets fill the local member variable with the service number
	m_ServiceRegistered = NewService;

	// At this point we already know the port number that the NetDaemon reserved
	// for this service, so lets create one thread and inside this
	// thread lets create  the UDP socket for this service
	DWORD ThreadId;
	ServiceClientThreadStruct* pStruct;

	pStruct = new ServiceClientThreadStruct;
	pStruct->pDai = this;
	pStruct->ServiceId = NewService;
	pStruct->ServicePort = NewPortNbr;
	pStruct->hControlEndEvent = m_hEventWaitForClientThreadEnd;

	if(CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)WaitForServiceClientThreadFunction,
		                                                          pStruct,0,&ThreadId) == NULL)
	{
		MessageBox(GetActiveWindow(),"Error on WaitForServiceClientThreadFunction thread creation","",MB_OK);
		return FALSE;
	}

	return TRUE;;
}




/***************************************************************************************

	DA_ServiceUnregister
	
	This method unregists the service on the NetDaemon. 

***************************************************************************************/
void Dai::DA_ServiceUnregister()
{
	SOCKET TempSocket;
	int TempSockAddrLength;
	struct sockaddr_in TempSockAddr;
	char aux[CONTROL_MSG_SIZE];
	WSABUF WsaBuf;	
	Conversion_Union conv;
	DWORD Flags =0;
	DWORD BytesSent;

	// Lets see if UDP protocol stack is available
	if(m_pUdpProtocolInfo == NULL)
		return;

	// Alocate space for the address structures
	TempSockAddrLength = m_pUdpProtocolInfo->iMaxSockAddr;
	memset((char*)&TempSockAddr,0,TempSockAddrLength);

	// Create socket
	TempSocket = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO,
		                  m_pUdpProtocolInfo,0,WSA_FLAG_OVERLAPPED);
	if(TempSocket == INVALID_SOCKET)
	{
		IdentifyError("UnregistService - WSASocket: ",WSAGetLastError());
		return;
	}

	// Fill the address of the local NetDaemon at the well knwon UDP port
	FillLocalSockAddrStruct(&TempSockAddr, WELL_KNOWN_UDP_PORT);

	//Lets put the information in a packet and then, send it to the NetDaemon for registering
	// the service
	WsaBuf.buf = aux;
	WsaBuf.len = CONTROL_MSG_SIZE;

	conv.int_number = UNREGIST_SERVICE;
	memcpy(aux,conv.data,4);
	conv.int_number = m_ServiceRegistered;
	memcpy(aux+4,conv.data,4);

	if(WSASendTo(TempSocket,&WsaBuf,1,&BytesSent,0,(struct sockaddr*)&TempSockAddr,
						TempSockAddrLength,NULL,NULL) == SOCKET_ERROR)
	{
		IdentifyError("UnregistService - WSASendTo ",WSAGetLastError());
		return;
	}

}



/***************************************************************************************

	Dai::CreateUdpServiceSocket

	Creates the Udp service socket that waits for a packet from the NetDaemon

***************************************************************************************/
BOOL Dai::CreateUdpServiceSocket(unsigned short Port)
{

/*	struct sockaddr_in MySockAddr;
	int SockAddrLength; 

	// Lets see if UDP protocol stack is available
	if(m_pUdpProtocolInfo == NULL)
		return FALSE;

	// Alocate space for the address structures
	SockAddrLength = m_pUdpProtocolInfo->iMaxSockAddr;
	memset((char*)&MySockAddr,0,SockAddrLength);


	// We can use the information we get before in this socket
	m_hUdpServiceSocket = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO,
		                  m_pUdpProtocolInfo,0,WSA_FLAG_OVERLAPPED);
	if(m_hUdpServiceSocket  == INVALID_SOCKET)
	{
		IdentifyError("CreateUdpServiceSocket - WSASocket: ",WSAGetLastError());
		return FALSE;
	}


	// Fill the address of the local NetDaemon at the well knwon UDP port
	FillLocalSockAddrStruct(&MySockAddr, Port);

	// We have to make the binding to this socket
	if(bind(m_hUdpServiceSocket,(struct sockaddr*)&MySockAddr,SockAddrLength) == SOCKET_ERROR)
	{
		IdentifyError("CreateUdpServiceSocket - bind: ",WSAGetLastError());
		return FALSE;
	}  */



	struct sockaddr_in MySockAddr;
	MySockAddr.sin_family = AF_INET;
	MySockAddr.sin_port = Port;
	MySockAddr.sin_addr.s_addr = INADDR_ANY;

	m_hUdpServiceSocket = socket(AF_INET,SOCK_DGRAM,0);
	if(m_hUdpServiceSocket == SOCKET_ERROR)
	{
		IdentifyError("CreateUdpServiceSocket - socket: ",WSAGetLastError());
		return FALSE;
	}

	if(bind(m_hUdpServiceSocket,(struct sockaddr*)&MySockAddr,
		                                      sizeof(struct sockaddr)) == SOCKET_ERROR)
	{
		IdentifyError("CreateUdpServiceSocket - bind: ",WSAGetLastError());
		return FALSE;
	}  


	return TRUE;
}



/***************************************************************************************

	CreateAndBindTcpSocket

	This method creates and binds a TCP socket to a specific port. First, it tries to
	use the suggested por. If this port is already in use, it tries another port 
	and returns the port number used

***************************************************************************************/
BOOL Dai::CreateAndBindTcpSocket(SOCKET& hSock,unsigned short PortSuggested, 
								                unsigned short& PortAttributed)
{
	BOOL ReturnValue;
	int AddrLength;
	struct sockaddr_in* pSockAddr;
	unsigned short Port;

	// Lets see if TCP protocol stack is available
	if(m_pTcpProtocolInfo == NULL)
		return FALSE;

	// Alocate space for the address structures
	AddrLength = m_pTcpProtocolInfo->iMaxSockAddr;
	pSockAddr = (struct sockaddr_in*) new char[AddrLength];
	memset((char*)pSockAddr,0,AddrLength);


	// Create socket
	hSock = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO,
		                  m_pTcpProtocolInfo,0,WSA_FLAG_OVERLAPPED);
	
	// Verify if the socket was correctly created
	if(hSock == INVALID_SOCKET)
	{
		IdentifyError("WSASocket: ",WSAGetLastError());
		ReturnValue = FALSE;
		goto end;
	}

	Port = PortSuggested;

	do
	{
		// Fill local address structure 
		FillLocalSockAddrStruct(pSockAddr, Port);

		// Bind the socket
		if(bind(hSock, (struct sockaddr*)pSockAddr, AddrLength) == SOCKET_ERROR)
		{
			int Error;
			Error = WSAGetLastError();
			if(Error == WSAEADDRINUSE)
			{
				Port++;
				continue;
			}
			else
			{
				IdentifyError("Bind: ",WSAGetLastError());
				ReturnValue = FALSE;
				goto end;	
			}
		}
		else  // The bind was successfull
		{
			PortAttributed = Port;
			break;
		}

	}while(TRUE);
	

	ReturnValue = TRUE;

end:
	delete pSockAddr;
	return (ReturnValue);
}



/***************************************************************************************

	CreateAndBindUdpSocket

  	This method creates and binds a UDP socket to a specific port. First, it tries to
	use the suggested por. If this port is already in use, it tries another port 
	and returns the port number used

***************************************************************************************/
BOOL Dai::CreateAndBindUdpSocket(SOCKET& hSock,unsigned short PortSuggested, 
								              unsigned short& PortAttributed)
{
	unsigned short Port;
	BOOL ReturnValue;
	int AddrLength;
	struct sockaddr_in* pSockAddr;

	// Lets see if UDP protocol stack is available
	if(m_pUdpProtocolInfo == NULL)
		return FALSE;

	// Alocate space for the address structures
	AddrLength = m_pUdpProtocolInfo->iMaxSockAddr;
	pSockAddr = (struct sockaddr_in*) new char[AddrLength];
	memset((char*)pSockAddr,0,AddrLength);


	// We can use the information we get before in this socket
	hSock = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO,
		                  m_pUdpProtocolInfo,0,WSA_FLAG_OVERLAPPED);
	if(hSock  == INVALID_SOCKET)
	{
		IdentifyError("CreateAndBindUdpServiceSocket - WSASocket: ",WSAGetLastError());
		return FALSE;
	}


	Port = PortSuggested;

	do
	{
		// Fill local address structure 
		FillLocalSockAddrStruct(pSockAddr, Port);

		// Bind the socket
		if(bind(hSock, (struct sockaddr*)pSockAddr, AddrLength) == SOCKET_ERROR)
		{
			int Error;
			Error = WSAGetLastError();
			if(Error == WSAEADDRINUSE)
			{
				Port++;
				continue;
			}
			else
			{
				IdentifyError("Bind: ",WSAGetLastError());
				ReturnValue = FALSE;
				goto end;	
			}
		}
		else  // The bind was successfull
		{
			PortAttributed = Port;
			break;
		}

	}while(TRUE);

	ReturnValue = TRUE;

end:
	delete pSockAddr;
	return (ReturnValue);

}



/***************************************************************************************

	MySend()

	Method used to send information through a socket

***************************************************************************************/
BOOL Dai::MySend(SOCKET hSock,char* pInfo,unsigned long BytesToSend,DWORD& SentBytes)
{
	DWORD BytesSent;
	WSABUF WsaBuf;

	WsaBuf.buf = pInfo;
	WsaBuf.len = BytesToSend;

	if(WSASend(hSock, &WsaBuf, 1, &BytesSent , 0, NULL, NULL)== SOCKET_ERROR)
	{
		IdentifyError("MySend  - WSASend: ",WSAGetLastError());
		return FALSE;
	}

	SentBytes = BytesSent;

	return TRUE;

}



/***************************************************************************************

	MyRecv

	Method used to recv information in a socket. This method will be called to receive
	just small ammounts of info (around 256 bytes == CONTROL_MSG_SIZE) so, it is assumed
	that this info is received atomically

***************************************************************************************/
BOOL Dai::MyRecv(SOCKET hSock,char* pInfo,unsigned long RecvBuffSize,DWORD& ReceivedBytes)
{
	WSABUF WsaBuf;
	DWORD Flags = 0;
	DWORD BytesReceived;
	int Error;

	WsaBuf.buf = pInfo;
	WsaBuf.len = RecvBuffSize;

	// This function receives info from the network. If the WSAEWOULDBLOCK is received
	// the function loops until receiving all
	do
	{
		if(WSARecv(hSock, &WsaBuf, 1, &BytesReceived, &Flags, NULL, NULL)== SOCKET_ERROR)
		{
			Error = WSAGetLastError();
			if(Error != WSAEWOULDBLOCK)
			{
				IdentifyError("MyRecv  - WSARecv: ",Error);
				return FALSE;
			}
		}	
		else
		{
			ReceivedBytes = BytesReceived;
			break;  // exit while
		}

	}while(TRUE);

	return TRUE;
}




/***************************************************************************************

	EstablishDmifControlChannel

	Method called by the client part of Dmif to establish the DMIF control channel 
	to the server

***************************************************************************************/
BOOL Dai::EstablishDmifControlChannel(SOCKET* pSock,char* IpAddress,unsigned short Port)
{
	int RemoteSockAddrLength;
	struct sockaddr_in RemoteSockAddr;
	unsigned long internet_address;
	struct hostent* server_pointer;


	// Lets see if TCP protocol stack is available
	if(m_pTcpProtocolInfo == NULL)
		return FALSE;

	// Alocate space for the address structures
	RemoteSockAddrLength = m_pTcpProtocolInfo->iMaxSockAddr;
	memset((char*)&RemoteSockAddr,0,RemoteSockAddrLength);


	// Create socket
	*pSock = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO,
		                  m_pTcpProtocolInfo,0,WSA_FLAG_OVERLAPPED);
	
	// Verify if the socket was correctly created
	if(*pSock == INVALID_SOCKET)
	{
		IdentifyError("EstablishDmifControlChannel: ",WSAGetLastError());
		return FALSE;
	}

	// Fill the remote address structure
	RemoteSockAddr.sin_family = AF_INET;
	RemoteSockAddr.sin_port = Port;

	// Lets see if the address is in the dot decimal format
	if((internet_address=inet_addr(IpAddress)) == INADDR_NONE) 
	{
		// Lets convert the string
		if((server_pointer=gethostbyname(IpAddress))==NULL)
			return FALSE;
		else
			RemoteSockAddr.sin_addr.s_addr=
		              (((struct in_addr *)(server_pointer->h_addr))->s_addr);
	}
	else
		RemoteSockAddr.sin_addr.s_addr=internet_address;

	
	if(WSAConnect(*pSock,(struct sockaddr*)&RemoteSockAddr,
		              RemoteSockAddrLength,NULL,NULL,NULL,NULL) == SOCKET_ERROR)
		return FALSE;


#ifndef BUILD_CSELT_APP
	MessageBox(GetActiveWindow(),"DMIF CHANNEL ESTABLISHED!!!","DmifLayer",MB_OK);
#endif

	return TRUE;
}



/***************************************************************************************

  	SendEsIds

	Fill the packet to send the EsIds to the server

***************************************************************************************/
BOOL Dai::SendEsIds(SOCKET hSock,DATA* pUuData)
{
	unsigned int NumOfEs;
	DWORD BytesSent;
	Conversion_Union conv;
	int Msg;
	char aux[CONTROL_MSG_SIZE];

	Msg = CHANNEL_ADD_REQUEST;
	NumOfEs = pUuData->NumOfElements;

	// Fill the packet that will go to the server side
	conv.int_number = Msg;
	memcpy(aux,conv.data,4);
	conv.int_number = NumOfEs;
	memcpy(aux+4,conv.data,4);
	memcpy(aux+8, pUuData->pBuffer, (NumOfEs*4));

	if(!MySend(hSock,aux,CONTROL_MSG_SIZE,BytesSent))
		return FALSE;

	return TRUE;
}




/***************************************************************************************

	RcvMsgInDmifChannel

	This method receives and identifies the message that arrived at the Dmif control 
	channel. Then, after identifying the message received calls the apropriate method
	to handle it.

***************************************************************************************/
void Dai::RcvMsgInDmifChannel(CDmifPeer* pPeer, DaiSession* pSession)
{
	char TempBuff[CONTROL_MSG_SIZE];
	DWORD BytesReceived;
	Conversion_Union conv;
	int MsgReceived;
	int NumElements;
	DATA* pUuData;
	int* pElements;
	SMT smt;
	int count;

	if(!MyRecv(pPeer->m_hDmifCtrlSock, TempBuff, CONTROL_MSG_SIZE, BytesReceived))
	{
		MessageBox(GetActiveWindow(),"Error reading from DMIF channel","DmifLayer",MB_OK);
		return;
	}

	
	memcpy(conv.data,TempBuff,4);
	MsgReceived = conv.int_number;
	memcpy(conv.data,TempBuff+4,4);
	NumElements = conv.int_number;
	pUuData = (DATA*) new DATA;


	switch(MsgReceived)
	{
		case UDP_PORT_CONFIRMATION:
			// Lets extract the information of the TransMuxTag and the remote UDP Port number
			int TransTag;
			unsigned short RemotePort;

			memcpy(conv.data,TempBuff+8,4);
			TransTag = conv.int_number;
			memcpy(conv.data,TempBuff+12,2);
			RemotePort = conv.ushort_number;

			// So, lets connect this socket
			ConnectUdpSocket(TransTag, RemotePort, pPeer);

			break;
			

		case CHANNEL_ADD_REQUEST:
			// Lets extract the information of the ES_Ids that the client requested
	
			pElements = new int[NumElements];
			memcpy(pElements,TempBuff+8,(NumElements)*4);
			pUuData->NumOfElements = NumElements;
			pUuData->pBuffer = pElements;

			// Call the callback function to inform the upper layer of the elemntary streams 
			//requested
			DA_ChannelAddIndication(pSession->m_SessionID,pPeer->m_pDmifPeerAddress,pUuData);
		
			delete pElements;
			break;

		case CHANNEL_ADD_RESPONSE:

			unsigned int TransMuxTag;
			unsigned int FlexChannelNumber;
			TransMuxChannel* pTransChannel;
			StreamMapElement newElement;

			pElements = new int [NumElements];

			// Now, pElements has a loop of (ChannelAssocTag + EsIds)
			memcpy(pElements,TempBuff+8,(NumElements)*4);
			pUuData->NumOfElements = NumElements;
			pUuData->pBuffer = pElements;


			// I will fill the SMT table of each TransMuxChannel Object
			for(count = 0; count < NumElements/2; count ++)
			{
				if(pPeer->FindElementInFmcTable(*(pElements + 2*count), TransMuxTag, 
					                                              FlexChannelNumber))
				{
					if(FindTransMuxChannelInList(&pTransChannel,TransMuxTag,CLIENT_SIDE))
					{
						newElement.m_nAssociationTag = *(pElements + 2*count);
						newElement.m_nESid = *(pElements + 2*count + 1);
						newElement.m_nFmcNo = FlexChannelNumber;
						(pTransChannel->mSmt)+=(newElement);
					}

				}
				else
					MessageBox(GetActiveWindow(),"CHANNEL_ADD_RESPONSE - Error unexpected",
																		"DmifLayer",MB_OK);
			}

			// Confirm the creation of the channels
			//DA_ChannelAddConfirmation();
			ConfirmChannels(pSession,pPeer);

			delete pElements;

			break;

		case CHANNEL_READY_REQUEST:

			pElements = new int[NumElements];
			memcpy(pElements,TempBuff+8,(NumElements)*4);
			pUuData->NumOfElements = NumElements;
			pUuData->pBuffer = pElements;

			// Call the callback function to inform the upper layer of the elemntary streams
			// requested
			DA_ChannelReadyIndication(pSession->m_SessionID,pPeer->m_pDmifPeerAddress,pUuData);
			delete pElements;

			break;


		case LIST_OF_CHANNELS:
			pElements = new int[NumElements];
			memcpy(pElements,TempBuff+8,(NumElements)*4);
			pUuData->NumOfElements = NumElements;
			pUuData->pBuffer = pElements;

			// I should establish the connections
			EstablishConnectionsClientSide(pSession, pPeer, pUuData);

			delete pElements;
			break;

		case SESSION_DETACH_REQUEST:
			// by now its not being used
			break;

		case SESSION_DESTROY_REQUEST:
			DA_SessionDestroyIndication(pSession->m_SessionID,pPeer->m_pDmifPeerAddress);
			break;

		case FMC_DESCRIPTOR:
			// The FMC contains a loop of (Channel Assoc Tag + FlexMuxChannel Number +
			// + TransMuxAssociationTag
			int* pAux;

			pAux = pElements = new int[NumElements];
			memcpy(pElements,TempBuff+8,(NumElements)*4);
			for(count = 0; count < NumElements/3; count++)
			{
				pPeer->AddElementToFmcTable(*pAux,*(pAux+1),*(pAux+2));
				pAux+=3;
			}
			
			delete pElements;
	
			break;

	}  // End of switch

	delete pUuData;

}


/***************************************************************************************

	DA_ChannelSetupRequest

	Method used by the server part of Dmif asking the Dmif layer for new channels.
	Creates the thread that will handle the channel establishment

***************************************************************************************/
void Dai::DA_ChannelSetupRequest(unsigned int SessionId, char* PeerAddress, DATA* pUData)
{
	DaiSession* pSession;
	CDmifPeer* pDmifPeer;
	AuxiliarThreadStruct* pStruct;
	DWORD ThreadId;
	DATA* pAuxData;

	// First lets find the session wiyh this ID
	if(!FindSessionInList(SessionId, &pSession))// There is no session with this number
		return;

	// Lets find if the peer exists and in this case, get the DmifCtrlSock Id
	if(!pSession->FindPeerInList(PeerAddress,&pDmifPeer))
		return;

	// I will create a structure of type DATA and pass it to thread.
	pAuxData = (DATA*) new DATA;
	pAuxData -> NumOfElements = pUData -> NumOfElements;
	pAuxData-> pBuffer = new int[(pAuxData -> NumOfElements)*4];
	memcpy(pAuxData-> pBuffer, pUData-> pBuffer, (pAuxData -> NumOfElements)*4);

	// Fill the struct for the thread
	pStruct = (AuxiliarThreadStruct*) new AuxiliarThreadStruct;
	pStruct -> pDai = this;
	pStruct -> pSession = pSession;
	pStruct -> pDmifPeer = pDmifPeer;
	pStruct -> pData = pAuxData;
	// Create a Thread to do establish with the other peer the FlexMux and DataMux channel
	if(CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)ChannelEstablishmentThreadFunction,
		                                                   pStruct,0,&ThreadId) == NULL)
		MessageBox(GetActiveWindow(),"Error on ChannelEstablishmentThreadFunction",
																		"DmifLayer",MB_OK);

}




/***************************************************************************************

	DA_ChannelAddResponse

	Method that will be called by the server Dmif part and will send to the client
	side the pairs <EsId , ChannelAssociationTag>

***************************************************************************************/
void Dai::DA_ChannelAddResponse(unsigned int SessionId,char* PeerAddress, DATA* pData)
{

	DaiSession* pSession;
	CDmifPeer* pDmifPeer;
	char BuffAux[CONTROL_MSG_SIZE];
	DWORD SentBytes;
	Conversion_Union conv;


	// First lets find the session wiyh this ID
	if(!FindSessionInList(SessionId, &pSession)) // There is no session with this number
		return;

	// Lets find if the peer exists and in this case, get the DmifCtrlSock Id
	if(!pSession->FindPeerInList(PeerAddress,&pDmifPeer))
		return;

	conv.int_number = CHANNEL_ADD_RESPONSE;
	memcpy(BuffAux,conv.data,4);
	conv.int_number = pData->NumOfElements;
	memcpy(BuffAux+4,conv.data,4);
	memcpy(BuffAux+8,pData->pBuffer,(pData->NumOfElements)*4);

	MySend(pDmifPeer->m_hDmifCtrlSock,BuffAux,CONTROL_MSG_SIZE,SentBytes);

}



/***************************************************************************************

  	DA_ChannelReadyRequest

	Method used by the client side of Dmif to inform the server that is ready to receive
	information at the channels specified in pData structure

***************************************************************************************/
void Dai::DA_ChannelReadyRequest(unsigned int SessionId, char* PeerAddress, DATA* pData)
{
	DaiSession* pSession;
	CDmifPeer* pDmifPeer;
	char BuffAux[CONTROL_MSG_SIZE];
	DWORD SentBytes;
	Conversion_Union conv;


	// First lets find the session wiyh this ID
	if(!FindSessionInList(SessionId, &pSession)) // There is no session with this number
		return;

	// Lets find if the peer exists and in this case, get the DmifCtrlSock Id
	if(!pSession->FindPeerInList(PeerAddress,&pDmifPeer))
		return;

	conv.int_number = CHANNEL_READY_REQUEST;
	memcpy(BuffAux,conv.data,4);
	conv.int_number = pData->NumOfElements;
	memcpy(BuffAux+4,conv.data,4);
	memcpy(BuffAux+8,pData->pBuffer,(pData->NumOfElements)*4);

	MySend(pDmifPeer->m_hDmifCtrlSock,BuffAux,CONTROL_MSG_SIZE,SentBytes);
}



/***************************************************************************************

	EstablishConnectionsServerSide

	Method called inside the server thread to establish the TransMux channels with the
	client. All TCP connections wiil wait for the connection

***************************************************************************************/
BOOL Dai::EstablishConnectionsServerSide(int(*Table)[6], int NumElements, 
                                         DaiSession* pSession, CDmifPeer* pDmifPeer)
{
	int count1, count2;
	unsigned short  AttributedPort;
	BOOL FlagNewSocketNeeded;

	// First, I will create a temporary table to write the TransMuxChannel Pointers;
	// The table will have a maximum size of NumElements
	TransMuxChannel** pTempTable;

	pTempTable = new TransMuxChannel* [NumElements];
	int NumTransMuxInTable = 0;	


	for(count1=0; count1 < NumElements; count1 ++)
	{
		FlagNewSocketNeeded = TRUE;

		for(count2=0; count2 < count1; count2 ++)
		{
			if(Table[count1][0] == Table[count2][0])   // No need for new socket
			{
				// Give the same socket port and socket handle
				Table[count1][4] = Table[count2][4];   
				Table[count1][5] = Table[count2][5];
				FlagNewSocketNeeded = FALSE;
				break;
			}
		}


		if(FlagNewSocketNeeded)  // Lets creat an object of class TransMuxChannel
		{
			TransMuxChannel* pNewChannel;


			// Ask for new socket. If QoS<5 the socket should be TCP, otherwise, UDP
			if(Table[count1][0] < MAX_QOS_VALUE_FOR_TCP_CHANNEL)    // TcpSocket
			{

				pNewChannel = (TransMuxChannel*) new TransMuxChannel(this, pSession, 
			                  pDmifPeer, DATA_BUFFER_SIZE,Table[count1][1], TCP_CHANNEL); 

				CreateAndBindTcpSocket(pNewChannel->m_hListenSocket, 25000, AttributedPort);
				if(listen(pNewChannel -> m_hListenSocket,SOMAXCONN) == SOCKET_ERROR)
					IdentifyError("EstablishConnectionsServerSide - Listen: ",WSAGetLastError());

				Table[count1][5] = pNewChannel->m_hListenSocket;
			}
			else
			{

				pNewChannel = (TransMuxChannel*) new TransMuxChannel(this, pSession, 
	                       pDmifPeer, DATA_BUFFER_SIZE, Table[count1][1], UDP_CHANNEL); 

				CreateAndBindUdpSocket(pNewChannel->m_hSocket, 25000, AttributedPort);

				Table[count1][5] = pNewChannel->m_hSocket;
			}


			//Add Element to linked list of TransMuxChannels
			AddTransMuxChannelToList(pNewChannel,SERVER_SIDE);


			Table[count1][4]= AttributedPort;
			pNewChannel ->UpdatePortNumber(AttributedPort);

			// Add to the temporary table
			*(pTempTable + NumTransMuxInTable) = pNewChannel;
			NumTransMuxInTable++;
		}
	}


	// Send to client the port numbers of the TransMuxChannels
	char BufferAux[CONTROL_MSG_SIZE];
	Conversion_Union conv;
	TransMuxChannel* aux_pointer;
	DWORD BytesSent;
	char* pAux;

	conv.int_number = LIST_OF_CHANNELS;
	memcpy(BufferAux,conv.data,4);
	conv.int_number = NumTransMuxInTable*3;
	memcpy(BufferAux+4,conv.data,4);
	pAux = BufferAux+8;

	// Fill the packet that will be sent to the client
	for(count1 = 0; count1 <  NumTransMuxInTable; count1++)
	{
		aux_pointer = *(pTempTable + count1);
		aux_pointer -> GetChannelProperties(pAux);
		pAux+=12;
	}

	MySend(pDmifPeer->m_hDmifCtrlSock,BufferAux,CONTROL_MSG_SIZE, BytesSent);
	
	// Lets wait for connection, channel by channel, skipping UDP channels
	for(count1 = 0; count1 <  NumTransMuxInTable; count1++)
	{
		aux_pointer = *(pTempTable + count1);
		aux_pointer->AcceptConnection(Table, NumElements);
	}

	delete pTempTable;

	return TRUE;
}





/***************************************************************************************

	EstablishConnectionsClientSide

	Method called by the client part of Dmif to establish the connections with the server.
	The thread that handles that transfer is created or, if it was already created, is 
	updated.

****************************************************************************************/

BOOL Dai::EstablishConnectionsClientSide(DaiSession* pSession,CDmifPeer* pDmifPeer, 
										                                DATA* pData)
{
	int NumOfConnections = (pData->NumOfElements)/3;
	Conversion_Union conv;
	struct sockaddr_in RemoteSockAddr;
	int RemoteSockAddrLength = sizeof(struct sockaddr);
	unsigned long internet_address;
	struct hostent* server_pointer;


	memset((char*)&RemoteSockAddr,0,RemoteSockAddrLength);
	RemoteSockAddr.sin_family = AF_INET;
	
	// Lets see if the address is in the dot decimal format
	if((internet_address=inet_addr(pDmifPeer->m_pDmifPeerAddress)) == INADDR_NONE) 
	{
		// Lets convert the string
		if((server_pointer=gethostbyname(pDmifPeer->m_pDmifPeerAddress))==NULL)
			return FALSE;
		else
			RemoteSockAddr.sin_addr.s_addr=
		              (((struct in_addr *)(server_pointer->h_addr))->s_addr);
	}
	else
		RemoteSockAddr.sin_addr.s_addr=internet_address;


	for(int j = 0; j < NumOfConnections; j++)
	{
		TransMuxChannel* pNewChannel;
		unsigned int AuxTag;
		int AuxTypeOfChannel;
		unsigned short AuxPort;
		
		memcpy(conv.data, (pData->pBuffer + j*3), 4);
		AuxTag = conv.int_number;
		memcpy(conv.data, (pData->pBuffer + j*3 + 1), 4);
		AuxTypeOfChannel = conv.int_number;
		memcpy(conv.data, (pData->pBuffer + j*3 + 2), 4);
		AuxPort = conv.int_number;

		RemoteSockAddr.sin_port = AuxPort;

		if(AuxTypeOfChannel == TCP_CHANNEL)
		{
			pNewChannel = (TransMuxChannel*) new TransMuxChannel(this, pSession,
			              pDmifPeer, DATA_BUFFER_SIZE, AuxTag, AuxTypeOfChannel);

			if(!pNewChannel->ConnectTransChannelToServer((struct sockaddr*)&RemoteSockAddr, 
				               RemoteSockAddrLength,m_pTcpProtocolInfo))
			{
				delete pNewChannel;
				continue;
			}
		}
		else
		{
			pNewChannel = (TransMuxChannel*) new TransMuxChannel(this, pSession,
			              pDmifPeer, DATA_BUFFER_SIZE,  AuxTag, AuxTypeOfChannel);
			
			if(!pNewChannel->ConnectTransChannelToServer((struct sockaddr*)&RemoteSockAddr, 
				               RemoteSockAddrLength,m_pUdpProtocolInfo))
			{
				delete pNewChannel;
				continue;
			}
		}

		// Add TransMuxChannel to the linked list of channels
		AddTransMuxChannelToList(pNewChannel, CLIENT_SIDE);
	
		// Add the new Event to the ArrayEvent. I need a critical section because I can have
		// several peers trying to update this value on the same Session
		pSession->FillEventArray(pNewChannel->GetTransMuxEvent());

	}  // for


	// Actualize or create the TransMuxes in the thread
	pSession->EnterDataChannelsCriticalSection();
	
	if(!pSession->m_ThreadCreated)
	{
		// Create the Thread
		DWORD ThreadId;
		ExchangeDataThreadStruct* pNewThreadStruct;
		pNewThreadStruct = (ExchangeDataThreadStruct*) new (ExchangeDataThreadStruct);
		pNewThreadStruct -> pSession = pSession;
		pNewThreadStruct -> pDai = this;

		if(CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)ExchangeDataThreadFunction,
		                                     pNewThreadStruct,0,&ThreadId) == NULL)
			MessageBox(GetActiveWindow(),"ExchangeDataThreadFunction","DmifLayer",MB_OK);

		pSession->m_ThreadCreated = TRUE;
	}
	else
	{
		// This event will wake-up the thread to read the new table of events
		SetEvent(pSession->m_hRefreshTableEvent);
	}

	pSession->LeaveDataChannelsCriticalSection();

	return TRUE;
}



/***************************************************************************************

	SendFmcDescriptors

	Method called by the server side of Dmif to send the FMC descriptor to the client

****************************************************************************************/
BOOL Dai::SendFmcDescriptors(int(*Table)[6], int NumElements, DaiSession* /*pSession*/,
							                                  CDmifPeer* pDmifPeer)
{
	DWORD BytesSent;
	Conversion_Union conv;
	char Buffer[CONTROL_MSG_SIZE];
	char* AuxPointer;

	conv.int_number = FMC_DESCRIPTOR;
	memcpy(Buffer, conv.data, 4);
	conv.int_number = NumElements*3;  //ChannelAssocTag+FlexMuxChannelNumber+TransMuxAssocTag
	memcpy(Buffer+4, conv.data,4);

	AuxPointer = Buffer+8;

	for(int j = 0; j < NumElements; j++)
	{
		conv.int_number = Table[j][3];     // Channel Association Tag
		memcpy(AuxPointer, conv.data,4);
		AuxPointer+=4;

		conv.int_number = Table[j][2];     // FlexMuxChannelNumber
		memcpy(AuxPointer, conv.data,4);
		AuxPointer+=4;

		conv.int_number = Table[j][1];     // TransMux Association Tag
		memcpy(AuxPointer, conv.data,4);
		AuxPointer+=4;

	}

	// Send the FMC Descriptor
	MySend(pDmifPeer->m_hDmifCtrlSock,Buffer,CONTROL_MSG_SIZE,BytesSent);

	return TRUE;
}



/***************************************************************************************

	AddSessionToList

	Adds the session pointer to the linked list of session pointers

****************************************************************************************/
void Dai::AddSessionToList(DaiSession* pNewSession)
{

	ListOfSessionsElement* pAux;
	ListOfSessionsElement* pNewElement;


	// First, fill the new element fields
	pNewElement = (ListOfSessionsElement*) new ListOfSessionsElement;
	pNewElement -> pDaiSession = pNewSession;

	// Enter the critical section
	EnterCriticalSection(&mSessionIdCriticalSection);

	if(pHeadListOfSessionsElement == NULL) // List its empty
		pHeadListOfSessionsElement = pNewElement;
	else    // Add at the tail of the list
	{
		pAux = pHeadListOfSessionsElement;
		while(pAux -> pNextElement != NULL)
			pAux = pAux->pNextElement;

		pAux->pNextElement = pNewElement;
	}

	// Leave the critical section
	LeaveCriticalSection(&mSessionIdCriticalSection);
}





/***************************************************************************************

	AddTransMuxChannelToList

	Adds a TransMux pointers to the linked list of TransMux pointers

****************************************************************************************/
void Dai::AddTransMuxChannelToList(TransMuxChannel* pNewTransMux, int SideOfTransMux)
{

	ListOfTransMuxElement* pAux;
	ListOfTransMuxElement* pNewElement;

	// First, fill the new element fields
	pNewElement = (ListOfTransMuxElement*) new ListOfTransMuxElement;
	pNewElement -> pTransMuxChannel = pNewTransMux;

	if(SideOfTransMux == SERVER_SIDE)
	{
		// Enter the critical section
		EnterCriticalSection(&mServerTransMuxListCriticalSection);

		if(pHeadListOfServerTransMuxElements == NULL) // List its empty
			pHeadListOfServerTransMuxElements = pNewElement;
		else    // Add at the tail of the list
		{
			pAux = pHeadListOfServerTransMuxElements;
			while(pAux -> pNextElement != NULL)
				pAux = pAux->pNextElement;

			pAux->pNextElement = pNewElement;
		}

		// Leave the critical section
		LeaveCriticalSection(&mServerTransMuxListCriticalSection);
	}
	else    //CLIENT SIDE
	{
		EnterCriticalSection(&mClientTransMuxListCriticalSection);

		if(pHeadListOfClientTransMuxElements == NULL) // List its empty
			pHeadListOfClientTransMuxElements = pNewElement;
		else    // Add at the tail of the list
		{
			pAux = pHeadListOfClientTransMuxElements;
			while(pAux -> pNextElement != NULL)
				pAux = pAux->pNextElement;

			pAux->pNextElement = pNewElement;
		}

		LeaveCriticalSection(&mClientTransMuxListCriticalSection);
	}
}



/***************************************************************************************

	DestroyAndRemoveSessionFromList

	Removes the session specified from the linked list of session pointers and destroys
	the session object

****************************************************************************************/
BOOL Dai::DestroyAndRemoveSessionFromList(unsigned int SessionId)
{
	BOOL ReturnValue = FALSE;
	ListOfSessionsElement* pActual;
	ListOfSessionsElement* pPrevious;


	EnterCriticalSection(&mSessionIdCriticalSection);

	if(pHeadListOfSessionsElement == NULL)			// The list is empty
		ReturnValue = FALSE;
	else
	{
		// The first element is a special case
		if((pHeadListOfSessionsElement->pDaiSession)->m_SessionID == SessionId)
		{
			pActual = pHeadListOfSessionsElement;
			pHeadListOfSessionsElement = pActual->pNextElement;
			delete (pActual->pDaiSession);       // Delete the session element
			delete pActual;						// Delete the linked list element
			ReturnValue = TRUE;
		}
		else 		// Its not the first element
 		{
			pActual = pHeadListOfSessionsElement;
			pPrevious = pHeadListOfSessionsElement;
			do
			{
				if((pActual->pDaiSession)->m_SessionID == SessionId)
				{
					pPrevious->pNextElement = pActual->pNextElement;
					delete (pActual->pDaiSession);       // Delete the session element
					delete pActual;						// Delete the linked list element
					ReturnValue = TRUE;
					break;
				}
				else
				{
					if(pActual->pNextElement != NULL)
					{
						pPrevious = pActual;
						pActual = pActual->pNextElement;
					}
					else
					{
						ReturnValue = FALSE;  // The element doesn't exist
						break;
					}

				}

			}while(TRUE);
		}
	}

	LeaveCriticalSection(&mSessionIdCriticalSection);

	return ReturnValue;
}





/***************************************************************************************

	FindSessionInList

	Search for a session in the linked list and if the session is found a pointer for
	that session object is returned

****************************************************************************************/
BOOL Dai::FindSessionInList(unsigned int SessionID, DaiSession** pDaiSession)
{
	BOOL ReturnValue = FALSE;
	ListOfSessionsElement* pActual;

	EnterCriticalSection(&mSessionIdCriticalSection);

	if(pHeadListOfSessionsElement == NULL) // List is empty
		ReturnValue = FALSE;
	else
	{
		pActual = pHeadListOfSessionsElement;
		do
		{
			if((pActual->pDaiSession)->m_SessionID == SessionID)
			{
				*pDaiSession = (pActual->pDaiSession);
				ReturnValue = TRUE;
				break;
			}
			else
			{
				if(pActual->pNextElement != NULL)
					pActual = pActual->pNextElement;
				else
				{
					ReturnValue = FALSE;
					break;
				}
			}

		}while(TRUE);

	}

	LeaveCriticalSection(&mSessionIdCriticalSection);

	return ReturnValue;
}





/****************************************************************************************

	FindTransMuxChannelInList

  	Search, based on the TransMuxTag, for a TransMux in the linked list,. 
	If the TransMux is found a pointer for that TransMux object is returned

****************************************************************************************/
BOOL Dai::FindTransMuxChannelInList(TransMuxChannel** pTransMux,unsigned int TransMuxTag,
									                                    int type_of_list)
{
	BOOL ReturnValue;
	ListOfTransMuxElement* pActual;

	if(type_of_list == SERVER_SIDE)
	{
		// Enter the critical section
		EnterCriticalSection(&mServerTransMuxListCriticalSection);

		if(pHeadListOfServerTransMuxElements == NULL) // List its empty
			ReturnValue = FALSE;
		else
		{
			pActual = pHeadListOfServerTransMuxElements;
			do
			{
				if(*(pActual->pTransMuxChannel) == TransMuxTag)
				{
					*pTransMux = (pActual->pTransMuxChannel);
					ReturnValue = TRUE;
					break;
				}
				else
				{
					if(pActual->pNextElement != NULL)
						pActual = pActual->pNextElement;
					else
					{
						ReturnValue = FALSE;
						break;
					}
				}

			}while(TRUE);
		}

		// Leave the critical section
		LeaveCriticalSection(&mServerTransMuxListCriticalSection);
	}
	else    //CLIENT SIDE
	{
		EnterCriticalSection(&mClientTransMuxListCriticalSection);

		if(pHeadListOfClientTransMuxElements == NULL) // List its empty
			ReturnValue = FALSE;
		else
		{
			pActual = pHeadListOfClientTransMuxElements;
			do
			{
				if(*(pActual->pTransMuxChannel) == TransMuxTag)
				{
					*pTransMux = (pActual->pTransMuxChannel);
					ReturnValue = TRUE;
					break;
				}
				else
				{
					if(pActual->pNextElement != NULL)
						pActual = pActual->pNextElement;
					else
					{
						ReturnValue = FALSE;
						break;
					}
				}

			}while(TRUE);

		}

		LeaveCriticalSection(&mClientTransMuxListCriticalSection);
	}

	return ReturnValue;
}


/***************************************************************************************

	FindTransMuxChannelInList

   	Search, based on the hEvent, for a TransMux in the linked list. 
	If the TransMux is found a pointer for that TransMux object is returned


****************************************************************************************/
BOOL Dai::FindTransMuxChannelInList(TransMuxChannel** pTransMux, WSAEVENT hEvent, 
									                                    int type_of_list)
{
	BOOL ReturnValue;
	ListOfTransMuxElement* pActual;

	if(type_of_list == SERVER_SIDE)
	{
		// Enter the critical section
		EnterCriticalSection(&mServerTransMuxListCriticalSection);

		if(pHeadListOfServerTransMuxElements == NULL) // List its empty
			ReturnValue = FALSE;
		else
		{
			pActual = pHeadListOfServerTransMuxElements;
			do
			{
				if(*(pActual->pTransMuxChannel) == hEvent)
				{
					*pTransMux = (pActual->pTransMuxChannel);
					ReturnValue = TRUE;
					break;
				}
				else
				{
					if(pActual->pNextElement != NULL)
						pActual = pActual->pNextElement;
					else
					{
						ReturnValue = FALSE;
						break;
					}
				}

			}while(TRUE);
		}

		// Leave the critical section
		LeaveCriticalSection(&mServerTransMuxListCriticalSection);
	}
	else    //CLIENT SIDE
	{
		EnterCriticalSection(&mClientTransMuxListCriticalSection);

		if(pHeadListOfClientTransMuxElements == NULL) // List its empty
			ReturnValue = FALSE;
		else
		{
			pActual = pHeadListOfClientTransMuxElements;
			do
			{
				if(*(pActual->pTransMuxChannel) == hEvent)
				{
					*pTransMux = (pActual->pTransMuxChannel);
					ReturnValue = TRUE;
					break;
				}
				else
				{
					if(pActual->pNextElement != NULL)
						pActual = pActual->pNextElement;
					else
					{
						ReturnValue = FALSE;
						break;
					}
				}

			}while(TRUE);

		}

		LeaveCriticalSection(&mClientTransMuxListCriticalSection);
	}

	return ReturnValue;
}



/***************************************************************************************

  Dai::DestroyAndRemoveAllSessionsFromList

****************************************************************************************/
void Dai::DestroyAndRemoveAllSessionsFromList()
{
	ListOfSessionsElement* pActual;
	ListOfSessionsElement* pNext;

	EnterCriticalSection(&mSessionIdCriticalSection);
	
	if(pHeadListOfSessionsElement != NULL)
	{
		pNext = pActual = pHeadListOfSessionsElement;
		do
		{
			pActual = pNext;
			pNext = pActual->pNextElement;
			
			//Delete the Session object
			delete(pActual-> pDaiSession);
			// remove the linked list element
			delete pActual;

		}while(pNext != NULL);
	}

	LeaveCriticalSection(&mSessionIdCriticalSection);
}




/***************************************************************************************

	ConfirmChannels

	This method is called by the client part of DMIF to send the confirmation, TransMux
	by TransMux to the RemoteService

****************************************************************************************/
void Dai::ConfirmChannels(DaiSession* pSession, CDmifPeer* pPeer)
{
	ListOfTransMuxElement* pActual;

	EnterCriticalSection(&mClientTransMuxListCriticalSection);

	if(pHeadListOfClientTransMuxElements != NULL) // List not empty
	{
		pActual = pHeadListOfClientTransMuxElements;
		do
		{
			if(!(pActual->pTransMuxChannel)->TransTagAlreadySentToClient)
			{

				if((pActual->pTransMuxChannel)->VerifySessionAndPeer(pSession,pPeer))
				{
			
					m_pRemService->OnChannelOpen(
					           (HANDLE)(pActual->pTransMuxChannel)->m_hSocket,
							              ((pActual->pTransMuxChannel)->mSmt));

					(pActual->pTransMuxChannel)->TransTagAlreadySentToClient = TRUE;
				}
			}
				
			if(pActual->pNextElement != NULL)
				pActual = pActual->pNextElement;
			else
				break;

		}while(TRUE);

	}

	LeaveCriticalSection(&mClientTransMuxListCriticalSection);
}



/***************************************************************************************

	HandleEventsOnDataSocket

	This method is called by the client side of Dmif when info arrived on a data channel.
	Then, the method should identify which is the right TransMux and should call it to 
	get the info that has just arrived.

****************************************************************************************/
void Dai::HandleEventsOnDataSocket(WSAEVENT hEvent)
{

	TransMuxChannel* pTransMux;
	WSANETWORKEVENTS NetworkEvents;
	int Result;


	if(FindTransMuxChannelInList(&pTransMux,hEvent,CLIENT_SIDE))
	{
		Result = WSAEnumNetworkEvents(pTransMux->m_hSocket,pTransMux->GetTransMuxEvent(),
			                                                             &NetworkEvents);
		if (Result == SOCKET_ERROR) 
			IdentifyError("WSAEnumNetworkEvents2",WSAGetLastError());
		else
			if(	NetworkEvents.lNetworkEvents  & FD_READ)
				pTransMux->ReceiveData();
	}
	else
		MessageBox(GetActiveWindow(),"HandleEvenOnDataSocket - TransMuxNotFoundInList ",
		                                                              "DmifLAyer",MB_OK);
}




/***************************************************************************************

	RegistService

	Called by the server part of Dmif to regist the service on the NetDaemon

****************************************************************************************/
BOOL Dai::RegistService(SOCKET hSocket,unsigned int NewServiceNumber, 
						                  unsigned short& NewPortNbr)
{
	WSAEVENT hEvent;
	WSAEVENT EventArray[1];
	Conversion_Union conv;
	char aux[CONTROL_MSG_SIZE];
	DWORD BytesSent, BytesReceived, WaitStatus;
	BOOL ReturnValue = FALSE;
	int Result, MsgReceived;
	WSANETWORKEVENTS NetworkEvents;


	// Lets create an event and associate it with the socket
	hEvent = (WSAEVENT)CreateEvent(NULL, FALSE, FALSE, NULL);
	if(WSAEventSelect(hSocket,(WSAEVENT)hEvent, FD_READ) == SOCKET_ERROR)
	{
		IdentifyError("RegistService - WSAEventSelect",WSAGetLastError());
		goto end;
	}
	
	//Lets put the information in a packet and then, send it to the NetDaemon for registering
	// the service
	conv.int_number = NEW_SERVICE_REGISTRATION;
	memcpy(aux,conv.data,4);
	conv.int_number = NewServiceNumber;
	memcpy(aux+4,conv.data,4);

	// Send the request for service registration
	if(!MySend(hSocket,aux,CONTROL_MSG_SIZE,BytesSent))
		goto end;

	// Put the socket waiting for an answer at least during 5 seconds
	// Receive the answer from the NetDaemon		
	EventArray[0] = hEvent;

	WaitStatus = WSAWaitForMultipleEvents(1,EventArray,FALSE,5000,TRUE);
	switch(WaitStatus)
	{
		case WSA_WAIT_FAILED:
			IdentifyError("RegistService - WSAWaitForMultipleEvents ",WSAGetLastError());
			break;

		case WSA_WAIT_EVENT_0:
			Result = WSAEnumNetworkEvents(hSocket, EventArray[0], &NetworkEvents);
			if (Result == SOCKET_ERROR) 
				IdentifyError("RegistService - WSAEnumNetworkEvents3",WSAGetLastError());
			else
			{
				if(	NetworkEvents.lNetworkEvents  & FD_READ)
				{
					if(MyRecv(hSocket,aux,CONTROL_MSG_SIZE,BytesReceived))
					{
						//	 Lets see the result
						memcpy(conv.data,aux,4);
						MsgReceived = conv.int_number;
						if(MsgReceived == MSG_ERROR)
							goto end;
						else
						{
							memcpy(conv.data,aux+4,2);
							NewPortNbr = conv.ushort_number;
							ReturnValue = TRUE;
						}
					}
				}
			}	
			
			break;

		case WSA_WAIT_TIMEOUT:   // A time out occured without registering the service
			MessageBox(GetActiveWindow(),"Service not registered. No answer from NetDaemon",
				                       "DmifLayer", MB_OK);
			break;
	}

end:
	CloseHandle((HANDLE)hEvent);
	return ReturnValue;

}



/***************************************************************************************

	ContactLocalNetDaemon

	Method called by the client part of Dmif. This method contacts the local daemon
	and expects to get the port number of the remote Dmif control socket

****************************************************************************************/
BOOL Dai::ContactLocalNetDaemon(SOCKET hSocket, char* PeerAddress, unsigned int ServiceId,
								                               unsigned short& PortNumber)
{
	WSAEVENT hEvent;
	WSAEVENT EventArray[1];
	Conversion_Union conv;
	char aux[CONTROL_MSG_SIZE];
	DWORD BytesSent, BytesReceived, WaitStatus;
	BOOL ReturnValue = FALSE;
	int Result, MsgReceived;
	WSANETWORKEVENTS NetworkEvents;


	// Lets create an event and associate it with the socket
	hEvent = (WSAEVENT)CreateEvent(NULL, FALSE, FALSE, NULL);
	if(WSAEventSelect(hSocket,(WSAEVENT)hEvent, FD_READ) == SOCKET_ERROR)
	{
		IdentifyError("RegistService - WSAEventSelect",WSAGetLastError());
		goto end;
	}
	
	//Lets put the information in a packet and then, send it to the NetDaemon for registering
	// the service
	conv.int_number = REQUEST_CONNECT_TO_PEER;
	memcpy(aux,conv.data,4);
	conv.int_number = ServiceId;
	memcpy(aux+4,conv.data,4);
	strcpy(aux+8,PeerAddress);

	// Send the request for service registration
	if(!MySend(hSocket,aux,CONTROL_MSG_SIZE,BytesSent))
		goto end;

	// Put the socket waiting for an answer at least during 5 seconds
	// Receive the answer from the NetDaemon		
	EventArray[0] = hEvent;

	WaitStatus = WSAWaitForMultipleEvents(1,EventArray,FALSE,5000,TRUE);
	switch(WaitStatus)
	{
		case WSA_WAIT_FAILED:
			IdentifyError("RegistService - WSAWaitForMultipleEvents ",WSAGetLastError());
			break;

		case WSA_WAIT_EVENT_0:
			Result = WSAEnumNetworkEvents(hSocket, EventArray[0], &NetworkEvents);
			if (Result == SOCKET_ERROR) 
				IdentifyError("RegistService - WSAEnumNetworkEvents5",WSAGetLastError());
			else
			{
				if(	NetworkEvents.lNetworkEvents  & FD_READ)
				{
					if(MyRecv(hSocket,aux,CONTROL_MSG_SIZE,BytesReceived))
					{
						//	 Lets see the result
						memcpy(conv.data,aux,4);
						MsgReceived = conv.int_number;
						if(MsgReceived != DMIF_PORT_NUMBER)
							goto end;
						else
						{
							// Now I know the TCP port number of the listening DMIF 
							//socket on the remote peer
							memcpy(conv.data,aux+4,2);
							PortNumber = conv.ushort_number;
							ReturnValue = TRUE;
						}
					}
				}
			}	
			
			break;

		case WSA_WAIT_TIMEOUT:   // A time out occured without registering the service
			break;
	}

end:
	CloseHandle((HANDLE)hEvent);
	return ReturnValue;
}







/***************************************************************************************

	ConnectUdpSocket

	Called by the server to "connect" the TransMux with the specified tag to a remote
	UDP socket.

****************************************************************************************/
void Dai::ConnectUdpSocket(unsigned int TransTag, unsigned short RemotePort,CDmifPeer* pPeer)
{
	TransMuxChannel* pTransChannel;
	struct sockaddr_in RemoteSockAddr;
	int RemoteSockAddrLength;
	unsigned long internet_address;
	struct hostent* server_pointer;



	// Fill the remote address structure
	RemoteSockAddrLength = m_pTcpProtocolInfo->iMaxSockAddr;
	memset((char*)&RemoteSockAddr,0,RemoteSockAddrLength);

	RemoteSockAddr.sin_family = AF_INET;
	RemoteSockAddr.sin_port = RemotePort;

	// Lets see if the address is in the dot decimal format
	if((internet_address=inet_addr(pPeer->m_pDmifPeerAddress)) == INADDR_NONE) 
	{
		// Lets convert the string
		if((server_pointer=gethostbyname(pPeer->m_pDmifPeerAddress))==NULL)
			return;
		else
			RemoteSockAddr.sin_addr.s_addr=
		              (((struct in_addr *)(server_pointer->h_addr))->s_addr);
	}
	else
		RemoteSockAddr.sin_addr.s_addr=internet_address;


	if(FindTransMuxChannelInList(&pTransChannel,TransTag,SERVER_SIDE))
	{
		if(WSAConnect(pTransChannel->m_hSocket,(struct sockaddr*)&RemoteSockAddr,
	             RemoteSockAddrLength,NULL,NULL,NULL,NULL) == SOCKET_ERROR)
				IdentifyError("ConnectUdpSocket -WSAConnect: ",WSAGetLastError());
	}

}


/***************************************************************************************

	DataAvailableForClient

	This method is called by the client side of Dmif and will inform the client of this
	Dmif layer that data has arrived at the TransMux specified by its handle.

****************************************************************************************/
void Dai::DataAvailableForClient(SOCKET hSock, char* pBuff, int NumBytes)
{	
	m_pRemService -> OnDataReceived((HANDLE)hSock,(unsigned char*)pBuff,NumBytes);
}



/***************************************************************************************

	RemoveAndDestroyAllTransMuxChannels

	Removes all TransMux pointers from the linked list and destroy the TransMux object

****************************************************************************************/
void Dai::RemoveAndDestroyAllTransMuxChannels(int side)
{

	ListOfTransMuxElement* pActual;
	ListOfTransMuxElement* pNext;

	if(side == SERVER_SIDE)
	{
		EnterCriticalSection(&mServerTransMuxListCriticalSection);
	
		if(pHeadListOfServerTransMuxElements != NULL)
		{
			pNext = pActual = pHeadListOfServerTransMuxElements;
			do
			{
				pActual = pNext;
				pNext = pActual->pNextElement;
			
				//Delete the Session object
				delete(pActual-> pTransMuxChannel);
				// remove the linked list element
				delete pActual;

			}while(pNext != NULL);
		}

		LeaveCriticalSection(&mServerTransMuxListCriticalSection);
	
	}
	else  // CLIENT_SIDE
	{
		EnterCriticalSection(&mClientTransMuxListCriticalSection);
	
		if(pHeadListOfClientTransMuxElements != NULL)
		{
			pNext = pActual = pHeadListOfClientTransMuxElements;
			do
			{
				pActual = pNext;
				pNext = pActual->pNextElement;
			
				//Delete the Session object
				delete(pActual-> pTransMuxChannel);
				// remove the linked list element
				delete pActual;

			}while(pNext != NULL);
		}

		LeaveCriticalSection(&mClientTransMuxListCriticalSection);
	}
}




/***************************************************************************************

	IdentifyError

	Identifies the error that occured in a socket

***************************************************************************************/
void Dai::IdentifyError(LPCSTR method, int ErrorNumber)
{
	char FinalMessage[MSG_LEN];
	char ErrorMsg[MSG_LEN];

	switch(ErrorNumber)
	{

	case WSAEINTR:
		strcpy(ErrorMsg,"WSAEINTR");
		break;
	case WSAEBADF:
		strcpy(ErrorMsg,"WSAEBADF");
		break;
	case WSAEACCES:
		strcpy(ErrorMsg,"WSAEACCES");
		break;
	case WSAEFAULT:
		strcpy(ErrorMsg,"WSAEFAULT");
		break;
	case WSAEINVAL:
		strcpy(ErrorMsg,"WSAEINVAL");
		break;
	case WSAEMFILE:
		strcpy(ErrorMsg,"WSAEMFILE");
		break;
	case WSAEWOULDBLOCK:
		strcpy(ErrorMsg,"WSAEWOULDBLOCK");
		break;
	case WSAEINPROGRESS:
		strcpy(ErrorMsg,"WSAEINPROGRESS");
		break;
	case WSAEALREADY:
		strcpy(ErrorMsg,"WSAEALREADY");
		break;
	case WSAENOTSOCK:
		strcpy(ErrorMsg,"WSAENOTSOCK");
		break;
	case WSAEDESTADDRREQ:
		strcpy(ErrorMsg,"WSAEDESTADDRREQ");
		break;
	case WSAEMSGSIZE:
		strcpy(ErrorMsg,"WSAEMSGSIZE");
		break;
	case WSAEPROTOTYPE:
		strcpy(ErrorMsg,"WSAEPROTOTYPE");
		break;
	case WSAENOPROTOOPT:
		strcpy(ErrorMsg,"WSAENOPROTOOPT");
		break;
	case WSAEPROTONOSUPPORT:
		strcpy(ErrorMsg,"WSAEPROTONOSUPPORT");
		break;
	case WSAESOCKTNOSUPPORT:
		strcpy(ErrorMsg,"WSAESOCKTNOSUPPORT");
		break;
	case WSAEOPNOTSUPP:
		strcpy(ErrorMsg,"WSAEOPNOTSUPP");
		break;
	case WSAEPFNOSUPPORT:
		strcpy(ErrorMsg,"WSAEPFNOSUPPORT");
		break;
	case WSAEAFNOSUPPORT:
		strcpy(ErrorMsg,"WSAEAFNOSUPPORT");
		break;
	case WSAEADDRINUSE:
		strcpy(ErrorMsg,"WSAEADDRINUSE");
		break;
	case WSAEADDRNOTAVAIL:
		strcpy(ErrorMsg,"WSAEADDRNOTAVAIL");
		break;
	case WSAENETDOWN:
		strcpy(ErrorMsg,"WSAENETDOWN");
		break;
	case WSAENETUNREACH:
		strcpy(ErrorMsg,"WSAENETUNREACH");
		break;
	case WSAENETRESET:
		strcpy(ErrorMsg,"WSAENETRESET");
		break;
	case WSAECONNABORTED:
		strcpy(ErrorMsg,"WSAECONNABORTED");
		break;
	case WSAECONNRESET:
		strcpy(ErrorMsg,"WSAECONNRESET");
		break;
	case WSAENOBUFS:
		strcpy(ErrorMsg,"WSAENOBUFS");
		break;
	case WSAEISCONN:
		strcpy(ErrorMsg,"WSAEISCONN");
		break;
	case WSAENOTCONN:
		strcpy(ErrorMsg,"WSAENOTCONN");
		break;
	case WSAESHUTDOWN:
		strcpy(ErrorMsg,"WSAESHUTDOWN");
		break;
	case WSAETOOMANYREFS:
		strcpy(ErrorMsg,"WSAETOOMANYREFS");
		break;
	case WSAETIMEDOUT:
		strcpy(ErrorMsg,"WSAETIMEDOUT");
		break;
	case WSAECONNREFUSED:
		strcpy(ErrorMsg,"WSAECONNREFUSED");
		break;
	case WSAELOOP:
		strcpy(ErrorMsg,"WSAELOOP");
		break;
	case WSAENAMETOOLONG:
		strcpy(ErrorMsg,"WSAENAMETOOLONG");
		break;
	case WSAEHOSTDOWN:
		strcpy(ErrorMsg,"WSAEHOSTDOWN");
		break;
	case WSAEHOSTUNREACH:
		strcpy(ErrorMsg,"WSAEHOSTUNREACH");
		break;
	case WSAENOTEMPTY:
		strcpy(ErrorMsg,"WSAENOTEMPTY");
		break;
	case WSAEPROCLIM:
		strcpy(ErrorMsg,"WSAEPROCLIM");
		break;
	case WSAEUSERS:
		strcpy(ErrorMsg,"WSAEUSERS");
		break;
	case WSAEDQUOT:
		strcpy(ErrorMsg,"WSAEDQUOT");
		break;
	case WSAESTALE:
		strcpy(ErrorMsg,"WSAESTALE");
		break;
	case WSAEREMOTE:
		strcpy(ErrorMsg,"WSAEREMOTE");
		break;
	case WSASYSNOTREADY:
		strcpy(ErrorMsg,"WSASYSNOTREADY");
		break;
	case WSAVERNOTSUPPORTED:
		strcpy(ErrorMsg,"WSAVERNOTSUPPORTED");
		break;
	case WSANOTINITIALISED:
		strcpy(ErrorMsg,"WSANOTINITIALISED");
		break;
	case WSAEDISCON:
		strcpy(ErrorMsg,"WSAEDISCON");
		break;
	case WSAENOMORE:
		strcpy(ErrorMsg,"WSAENOMORE");
		break;
	case WSAECANCELLED:
		strcpy(ErrorMsg,"WSAECANCELLED");
		break;
	case WSAEINVALIDPROCTABLE:
		strcpy(ErrorMsg,"WSAEINVALIDPROCTABLE");
		break;
	case WSAEINVALIDPROVIDER:
		strcpy(ErrorMsg,"WSAEINVALIDPROVIDER");
		break;
	case WSAEPROVIDERFAILEDINIT:
		strcpy(ErrorMsg,"WSAEPROVIDERFAILEDINIT");
		break;
	case WSASYSCALLFAILURE:
		strcpy(ErrorMsg,"WSASYSCALLFAILURE");
		break;
	case WSASERVICE_NOT_FOUND:
		strcpy(ErrorMsg,"WSASERVICE_NOT_FOUND");
		break;
	case WSATYPE_NOT_FOUND:
		strcpy(ErrorMsg,"WSATYPE_NOT_FOUND");
		break;
	case WSA_E_NO_MORE:
		strcpy(ErrorMsg,"WSA_E_NO_MORE");
		break;
	case WSA_E_CANCELLED:
		strcpy(ErrorMsg,"WSA_E_CANCELLED");
		break;
/*	case WSAEREFUSED:
		strcpy(ErrorMsg,"WSAEREFUSED");
		break;*/
	case WSAHOST_NOT_FOUND:
		strcpy(ErrorMsg,"WSAHOST_NOT_FOUND");
		break;
	case WSATRY_AGAIN:
		strcpy(ErrorMsg,"WSATRY_AGAIN");
		break;
	case WSANO_RECOVERY:
		strcpy(ErrorMsg,"WSANO_RECOVERY");
		break;
	case WSANO_DATA:
		strcpy(ErrorMsg,"WSANO_DATA");
		break;
	case WSA_IO_PENDING:
		strcpy(ErrorMsg,"WSA_IO_PENDING");
		break;
	case WSA_IO_INCOMPLETE:
		strcpy(ErrorMsg,"WSA_IO_INCOMPLETE");
		break;
	case WSA_INVALID_HANDLE:
		strcpy(ErrorMsg,"WSA_INVALID_HANDLE");
		break;
	case WSA_INVALID_PARAMETER:
		strcpy(ErrorMsg,"WSA_INVALID_PARAMETER");
		break;
	case WSA_NOT_ENOUGH_MEMORY:
		strcpy(ErrorMsg,"WSA_NOT_ENOUGH_MEMORY");
		break;
	case WSA_OPERATION_ABORTED:
		strcpy(ErrorMsg,"WSA_OPERATION_ABORTED");
		break;
	case WSA_MAXIMUM_WAIT_EVENTS:
		strcpy(ErrorMsg,"WSA_MAXIMUM_WAIT_EVENTS");
		break;
	case WSA_WAIT_FAILED:
		strcpy(ErrorMsg,"WSA_WAIT_FAILED");
		break;
	case WSA_WAIT_EVENT_0:
		strcpy(ErrorMsg,"WSA_WAIT_EVENT_0");
		break;
	case WSA_WAIT_IO_COMPLETION:
		strcpy(ErrorMsg,"WSA_WAIT_IO_COMPLETION");
		break;
	case WSA_WAIT_TIMEOUT:
		strcpy(ErrorMsg,"WSA_WAIT_TIMEOUT");
		break;

	default:
		MessageBox(GetActiveWindow(),"Undefined Socket Error","NetManager",MB_OK);
		return;
	} // Switch(ErrorNumber)
	
	
	strcpy(FinalMessage, method);
	strcat(FinalMessage,ErrorMsg);
	MessageBox(GetActiveWindow(),FinalMessage,"TransMuxLayer",MB_OK);

}
