///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: Dai.h
//
// Authors: Nuno Delgado, CSELT
//
// Description:
//
///////////////////////////////////////////////////////////////////////////


#ifndef _Dai_h
#define _Dai_h


// Prevent inclusion of winsock.h in windows.h
#ifndef _WINSOCKAPI_
#define _WINSOCKAPI_
#endif

#include "winsock2.h"
#include "defines.h"
#include "DaiSession.h"
#include "TransMuxChannel.h"
#include "fmt.h"


// Version of Winsock that we need. At least version 2.0
#define VERSION_MAJOR			2
#define VERSION_MINOR			0



// Thread Functions 
BOOL WaitForServiceClientThreadFunction(IN LPVOID);
BOOL AcceptDmifThreadFunction(IN LPVOID);
BOOL ChannelNegotiationThreadFunction(IN LPVOID);
BOOL ChannelEstablishmentThreadFunction(IN LPVOID);



// Union auxiliar for converting data
union Conversion_Union
{
	int int_number;
	unsigned short ushort_number;
	char data[sizeof(int)];
};




// Structure used for passing data in some methods
struct DATA
{	
	int NumOfElements;   // Number of integer elements pointed by pBuffer
	int* pBuffer;
};


class TransMuxChannel;
class RemoteService;



//  Each object of this class will be an element of a linked list of
//  pointers to DaiSessions
class ListOfSessionsElement
{
	public:
		ListOfSessionsElement* pNextElement;
		DaiSession*	pDaiSession;

		ListOfSessionsElement(){pNextElement = NULL;}
		~ListOfSessionsElement() {}
};


//  Each object of this class will be an element of a linked list of
//  pointers to TransMuxChannels
class ListOfTransMuxElement
{
	public:
		ListOfTransMuxElement* pNextElement;
		TransMuxChannel*	pTransMuxChannel;

		ListOfTransMuxElement(){pNextElement = NULL;}
		~ListOfTransMuxElement() {}
};




class DLLExport Dai
{


	private:
		LPWSAPROTOCOL_INFO m_pInstalledProtocols;
		LPWSAPROTOCOL_INFO m_pUdpProtocolInfo;
		LPWSAPROTOCOL_INFO m_pTcpProtocolInfo;
		WSAEVENT m_hEventWaitForClientThreadEnd;
		int m_NumProtocols;
		unsigned int m_SessionNumber;
		unsigned int m_ServiceRegistered;
		RemoteService* m_pRemService;
		ListOfSessionsElement* pHeadListOfSessionsElement;
		ListOfTransMuxElement* pHeadListOfClientTransMuxElements;
		ListOfTransMuxElement* pHeadListOfServerTransMuxElements;

		BOOL InitWS2();
		BOOL FillProtocolsStruct();
		void FreeWS2();
		BOOL IsTcpProtocolAvailable();
		BOOL IsUdpProtocolAvailable();
		BOOL FillLocalSockAddrStruct(struct sockaddr_in*, unsigned short);
		BOOL EstablishDmifControlChannel(SOCKET*,char*,unsigned short);
		BOOL SendEsIds(SOCKET,DATA*);
		void AddSessionToList(DaiSession*);
		BOOL DestroyAndRemoveSessionFromList(unsigned int);
		void DestroyAndRemoveAllSessionsFromList();
		void AddTransMuxChannelToList(TransMuxChannel*, int);
		BOOL FindTransMuxChannelInList(TransMuxChannel**,unsigned int, int);
		BOOL FindTransMuxChannelInList(TransMuxChannel**, WSAEVENT, int);
		BOOL RegistService(SOCKET,unsigned int, unsigned short&);
		BOOL ContactLocalNetDaemon(SOCKET,char*, unsigned int, unsigned short&);
		void ConnectUdpSocket(unsigned int, unsigned short, CDmifPeer*);
		void RemoveAndDestroyAllTransMuxChannels(int);
		void ConfirmChannels(DaiSession*, CDmifPeer*);
		BOOL EstablishConnectionsClientSide(DaiSession*,CDmifPeer*, DATA*);


		CRITICAL_SECTION mSessionIdCriticalSection;
		CRITICAL_SECTION mClientTransMuxListCriticalSection;
		CRITICAL_SECTION mServerTransMuxListCriticalSection;
		CRITICAL_SECTION mDeliverInfoCriticalSection;

	public:

		SOCKET m_hUdpServiceSocket;


		Dai();
		Dai(RemoteService*);
		~Dai();
		BOOL Initialization();
		void ReleaseResources();
		void DA_SessionCreate(unsigned int&, int&);
		void DA_SessionDestroy(unsigned int, int, int&);
		void DA_SessionDestroyResponse(unsigned int, int, int&);
		BOOL DA_ServiceRegister(unsigned int);
		void DA_ServiceUnregister();
		BOOL DA_SessionAttachRequest(unsigned int, char*, unsigned int,unsigned int); 
		BOOL DA_SessionDetachRequest(unsigned int, char*, unsigned int, unsigned int);
		void DA_ChannelAddRequest(unsigned int, char*, DATA*);
		void DA_ChannelSetupRequest(unsigned int, char*, DATA*);
		void DA_ChannelAddResponse(unsigned int, char*, DATA*);
		void DA_ChannelReadyRequest(unsigned int, char*, DATA*);
		void IdentifyError(LPCSTR, int);
		BOOL CreateUdpServiceSocket(unsigned short Port);
		BOOL CreateAndBindTcpSocket(SOCKET&,unsigned short, unsigned short&);
		BOOL CreateAndBindUdpSocket(SOCKET&,unsigned short, unsigned short&);
		BOOL FindSessionInList(unsigned int, DaiSession**);
		BOOL MySend(SOCKET,char*,unsigned long,DWORD&);
		BOOL MyRecv(SOCKET,char*,unsigned long,DWORD&);
		void RcvMsgInDmifChannel(CDmifPeer*,DaiSession*);
		BOOL EstablishConnectionsServerSide(int(*)[6], int, DaiSession*,CDmifPeer*);
		BOOL SendFmcDescriptors(int(*)[6], int, DaiSession*,CDmifPeer*);
		void HandleEventsOnDataSocket(WSAEVENT);
		void DataAvailableForClient(SOCKET, char*, int);


		//Pointer to callback functions
		BOOL (*DA_SessionAttachIndication)(unsigned int, char*, unsigned int, unsigned int);
		void (*DA_SessionDestroyIndication)(unsigned int, char*);
		void (*DA_ChannelAddIndication)(unsigned int, char*, DATA*);
		void (*DA_ChannelSetupConfirmation)(DATA*);
		void (*DA_ChannelReadyIndication)(unsigned int, char*, DATA*);
};



// The following structures are used for passing parameters to threads

struct ServiceClientThreadStruct
{

	Dai* pDai;
	unsigned int ServiceId;		// The service Id
	unsigned short ServicePort; // The port number of the UDP socket
	WSAEVENT hControlEndEvent;	// Controls the end of thread
};


struct AcceptDmifThreadStruct
{
	SOCKET sock;
	unsigned int ServiceId;
	Dai* pDai;
};


struct ChannelNegotiationThreadStruct
{
	CDmifPeer* pDmifPeer;
	Dai* pDai;
	DaiSession* pSession;
};


struct AuxiliarThreadStruct
{
	Dai* pDai;
	CDmifPeer* pDmifPeer;
	DaiSession* pSession;
	DATA* pData;
};



struct ExchangeDataThreadStruct
{
	Dai* pDai;
	DaiSession* pSession;
};


#endif

