///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: DirectDraw.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of a class which encapsulates all of the
//   DirectDraw related functions.
//
// Created: April, 1997
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef DIRECT_DRAW_H
#define DIRECT_DRAW_H


#include <ddraw.h>
#include "BRect.h"


// The different types of RGB displays that are supported
enum RGBpixelType { RGB565, RGB555, RGB24bpp, RGB32bpp, BGR32bpp };


class DirectDraw
{
	//
	// Public functions
	//
public:

	// Constructor - set all pointers to NULL.
	DirectDraw::DirectDraw();

	// This function initialises the direct draw routines, and creates
	// all of the surfaces that are required.  Return NULL if everything
	// went OK, otherwise return a string describing the error.
	char* Init (HWND hwnd, unsigned int compositeWidth,
				unsigned int compositeHeight);

	// Call this function to resize the rendering surfaces.  Return NULL
	// if everything went OK, otherwise return a string describing the error.
	char* Resize (int newWidth, int newHeight);

	// Call this function to shut down the direct draw routines.  Currently
	// this function deallocates the alternate YUV frame store and releases
	// the main DirectDraw object.
	void Shutdown();

	// Destructor - Calls shutdown.
	DirectDraw::~DirectDraw();

	// This function renders the specifed region of the offscreen RGB surface
	// into the destination rectangle provided.  The destination rectangle is
	// given in screen coordinates.
	void RenderSurface(RECT* pSrcRect, RECT* pDestRect);

	// Lock the offscreen RGB surface for writing.  The details of the
	// surface are written into the given surface descriptor.  If an
	// error occurs, a dialog box is displayed which shows the error.
	// Once writing to this surface has finished, call UnlockSurface().
	void DirectDraw::LockSurface(DDSURFACEDESC& surfaceDesc);

	// Unlock the offscreen RGB surface, now that it has been finished
	// writing to.  If an error occurs, a dialog box is displayed which
	// shows the error.
	void DirectDraw::UnlockSurface();

	// Erase the specifed region of the offscreen RGB surface, filling
	// it with black.  If the rectangle is NULL, erase the entire
	// surface.  The coordinates used in the rectangle are in window
	// coordinates, with the origin at the top-left of the window.
	void EraseSurface(RECT const* pUpdateRect = NULL);

	// Get a handle to the device context to use for the offscreen RGB
	// surface.
	void GetHDC(HDC& hDC);

	// Release the handle to the device context for the offscreen RGB
	// surface.
	void ReleaseHDC(HDC& hDC);

	// Return the RGB pixel format
	RGBpixelType GetRGBpixelFormat() const { return rgbPixelFormat; }


	//
	// Private implementation details
	//
private:

	// The DirectDraw object
	LPDIRECTDRAW lpDD;

	// The surfaces which are created
	LPDIRECTDRAWSURFACE	surfacePrimary;	// DirectDraw primary surface
	LPDIRECTDRAWSURFACE	surfaceRGB;		// RGB back buffer surface

	// DirectDraw clipper object
	LPDIRECTDRAWCLIPPER	lpClipper;

	// The RGB pixel type of the current display
	RGBpixelType rgbPixelFormat;

	// This memory is used instead of surfaceYUVall is hardware YUV to RGB is not used
//	unsigned char* compositeYUVframe;
};


#endif // DIRECT_DRAW_H
