///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: DmifPeer.cpp
//
// Authors: Nuno Delgado, CSELT
//
// Description:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "DmifPeer.h"



/***************************************************************************************

***************************************************************************************/
CDmifPeer::CDmifPeer()
{

	// Allocate space for the peer address
	m_pDmifPeerAddress = new char [256];

	// Create the event tha will be associated with this socket
	m_hEvent = (WSAEVENT)CreateEvent(NULL, FALSE, FALSE, NULL);
	// Create the event that will finish the thread that is "listen" at Dmif socket
	m_hControlEvent = (WSAEVENT)CreateEvent(NULL, FALSE, FALSE, NULL);

	m_EventArray[0] = m_hEvent;
	m_EventArray[1] = m_hControlEvent;

	m_NumEvents = 2;
}


/***************************************************************************************

***************************************************************************************/
CDmifPeer::CDmifPeer(Dai* pDai, DaiSession* pSession)
{
	m_pDai = pDai;
	m_pSession = pSession;

		// Allocate space for the peer address
	m_pDmifPeerAddress = new char [256];

	// Create the event tha will be associated with this socket
	m_hEvent = (WSAEVENT)CreateEvent(NULL, FALSE, FALSE, NULL);
	// Create the event that will finish the thread that is "listen" at Dmif socket
	m_hControlEvent = (WSAEVENT)CreateEvent(NULL, FALSE, FALSE, NULL);

	m_EventArray[0] = m_hEvent;
	m_EventArray[1] = m_hControlEvent;

	m_NumEvents = 2;
}



/***************************************************************************************

***************************************************************************************/
CDmifPeer::~CDmifPeer()
{
	// Set event to finish ChannelNegotiationThread
	SetEvent(m_hControlEvent);   

	delete m_pDmifPeerAddress;

	CloseHandle((HANDLE)m_hEvent);
	CloseHandle((HANDLE)m_hControlEvent);
}



/***************************************************************************************

	AddElementToFmcTable	

	This method creates FmC elements and adds them to the FMC table of elements

***************************************************************************************/
void CDmifPeer::AddElementToFmcTable(unsigned int ChannelTag,unsigned int FlexChannelNumber,
								unsigned int TransMuxTag)
{
	
	struct FmcStruct newElement;

	newElement.mChannelAssocTag = ChannelTag;
	newElement.mFlexMuxChannelNumber = FlexChannelNumber;
	newElement.mTransMuxAssocTag = TransMuxTag;
	
	mFmcTable+=(newElement);
}



/***************************************************************************************

	FindElementInFmcTable

	Search for elements in the FMC table based on its Channel Tag and returns the Trans
	Mux Tag and FlexChannel Number of that element

***************************************************************************************/
BOOL CDmifPeer::FindElementInFmcTable(unsigned int ChannelTag, unsigned int& TransMuxTag, 
									                     unsigned int& FlexChannelNumber)
{
	for(int i =0; i < mFmcTable.GetLength(); i++)
	{
		if(mFmcTable[i].mChannelAssocTag == ChannelTag)
		{
			TransMuxTag = mFmcTable[i].mTransMuxAssocTag;
			FlexChannelNumber = mFmcTable[i].mFlexMuxChannelNumber;
			return TRUE;
		}
	}

	return FALSE;
}


/***************************************************************************************

  AssociateSocketWithEvent

  Associates the Dmif control socket with an event that is set when there is information
  available for reading or when the connection is closed. It is also set if we try to 
  send information at a rate higher than the buffer can support
  
***************************************************************************************/
BOOL CDmifPeer::AssociateSocketWithEvent()
{
	// Associate the member variable event with this socket	
	if(WSAEventSelect(m_hDmifCtrlSock, (WSAEVENT)m_hEvent, 
		             FD_READ | FD_WRITE) == SOCKET_ERROR)
	{
		m_pDai->IdentifyError("AssociateSocketWithEvent - WSAEventSelect ",WSAGetLastError());
		return FALSE;
	}

	return TRUE;
}



/***************************************************************************************

	FillDaiAndDaiSessionInfo

	Fills the member variables related with Dai and the DaiSession

***************************************************************************************/
void CDmifPeer::FillDaiAndDaiSessionInfo(Dai* pDai, DaiSession* pDaiSession)
{
	m_pDai = pDai;
	m_pSession = pDaiSession;
}



