///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  DrawableImage.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of a class which implements functionality common to
//   nodes which need to display images, such as the Image2DRenderer,
//   Movie2DRenderer and Background2DRenderer nodes.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef DRAWABLE_IMAGE_H
#define DRAWABLE_IMAGE_H


#include "DrawableNode.h"
#include "StreamCo.h"


class DrawableImage : public DrawableNode
{
public:
	// Class constructor & destructor
	DrawableImage(MediaObject* pMO);
	~DrawableImage();

	// Return a pointer to the StreamConsumer node that this node is
	// the proxy for.
	StreamConsumer* GetStreamConsumer() const { return (StreamConsumer*)GetObject(); }

	// Return a pointer to the stream that this node reads from.
	MediaStream* GetStream() const { return GetStreamConsumer()->GetStream(); }

	// Functions inherited from the base classes
	virtual void PreRender(Effects const& effects);
	virtual BOOL NeedsDrawing() const { return m_updateRequired; }
	virtual BOOL IsTransparent() const
	{ return m_isTransparent || DrawableNode::IsTransparent(); }
	virtual void Draw(BRect const* pClipBRect);
	BOOL IsFinished() const { return m_isFinished; }


	// Protected implementation details
protected:
	BOOL m_updateRequired;
	BOOL m_isTransparent;
	long m_startUpDelay;
	BOOL m_recurrentInDraw;

	// This method is called when the end of a stream is detected in either
	// the PreRender() method or the Draw() method.
	virtual void OnEndOfStream();


	// Private implementation details
private:
	BOOL m_isFinished;
	int m_preRenderOffsetX;
	int m_preRenderOffsetY;
};


#endif // DRAWABLE_IMAGE_H
