///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Text2DProxy.h
//
// Authors: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of a class which implements functionality common to
//   nodes which need to display text, namely the Text2D and
//   StreamingText nodes.
//
// Created: October, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef DRAWABLE_TEXT_H
#define DRAWABLE_TEXT_H


#include "GeometryNode.h"


class LineOfText
{
public:
	int start;			// Offset into m_data array for the start of the line
	int numChars;		// Number of characters in the line
	unsigned int width;	// Dimension, in pixels, of the line
	unsigned int height;
};


class DrawableText : public GeometryNode
{
public:
	DrawableText(MediaObject* pMO);
	~DrawableText();

	// Return a pointer to the FontStyle2D object belonging to the
	// Text2D node that this node is the proxy for.  To be overridden
	// by deriving classes.
	virtual FontStyle2D* GetFontStyle2D() const = 0;

	// Return the value of the length field of a Text2D node.  Nodes
	// which do not have this field must return -1.
	virtual int GetLength() const { return -1; }

	// Functions inherited from the base class
	virtual void PreRender(Effects const& effects);
	virtual void Draw(BRect const* pClipBRect);

	//
	// Protected implementation details
	//
protected:

	// Functions to get the value from the FontStyle2D node, which
	// use the default values if this node is missing
	const char* GetFStype() const;
	int GetFSstyle() const;
	int GetFSsize() const;
	int GetFShSpacing() const;
	int GetFSvSpacing() const;
	const char* GetFSlanguage() const;

	LPCSTR m_fontName;		// Font and formatting of the string
	int m_fontSize;
	BOOL m_bold;
	BOOL m_italic;
	BOOL m_underline;

	const char* m_data;		// The string of text to display
	int m_numLines;			// Number of lines in the string
	LineOfText* m_lines;	// An array of the lines of text
	int m_numLinesAllocated;// Allocated size of array
};


#endif // DRAWABLE_TEXT_H
