///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  FontStyle2D.cpp
//
// Authors: Antonio Arena, CSELT, arena@kim.cselt.it
//			Franco Casalino, CSELT, Franco.Casalino@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the FontStyle2D node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "FontStyle2DProxy.h"


// =====================================================================

//
// PresenterCSELT::InitFontStyle2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitFontStyle2D(FontStyle2D* pObj)
{
	pObj->SetPresenterData(new FontStyle2DProxy(pObj));
}


// =====================================================================

//
// FontStyle2DProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void FontStyle2DProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "FontStyle2D {\n");
	const char* indentStr = IndentPrefix(depth + 1);

	fprintf(outfile, "%stype \"%s\"\n", indentStr,
			(const char*)(ZString)GetFontStyle2D()->type);
	fprintf(outfile, "%sstyle %d (%s%s%s)\n", indentStr,
			(int)GetFontStyle2D()->style,
			((int)GetFontStyle2D()->style & 1) ? "Bold" : "",
			((int)GetFontStyle2D()->style & 2) ? "Italic" : "",
			((int)GetFontStyle2D()->style & 4) ? "Underline" : "");
	fprintf(outfile, "%ssize %d\n", indentStr,
			(int)GetFontStyle2D()->size);
	fprintf(outfile, "%shSpacing %d\n", indentStr,
			(int)GetFontStyle2D()->hSpacing);
	fprintf(outfile, "%svSpacing %d\n", indentStr,
			(int)GetFontStyle2D()->vSpacing);
	fprintf(outfile, "%sdata \"%s\"\n", indentStr,
			GetFontStyle2D()->language);

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* FontStyle2DProxy::Print */


// =====================================================================
