///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Group2DProxy.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Group2D node.
//
// Created: August, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Group2DProxy.h"


// =====================================================================

//
// PresenterCSELT::InitGroup2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitGroup2D(Group2D* pObj)
{
	pObj->SetPresenterData(new Group2DProxy(pObj));
}


// =====================================================================

//
// Group2DProxy::PreRender
//
// Apply the PreRender method to all of the children of this node.
//

void Group2DProxy::PreRender(Effects const& effects)
{
	PreRenderChildren(effects);
} /* Group2DProxy::PreRender */


// =====================================================================

//
// Group2DProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void Group2DProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sGroup2D {\n", IndentPrefix(depth));

	// Print out the children of this node
	char const* indentStr = IndentPrefix(depth + 1);
	fprintf(outfile, "%schildren", indentStr);
	PrintChildren(outfile, depth + 1);

	// Print out the fields of this node
	fprintf(outfile, "%sbboxCenter %.2f %.2f\n", indentStr,
			((Vec2f)GetGroup2D()->bboxCenter).x,
			((Vec2f)GetGroup2D()->bboxCenter).y);
	fprintf(outfile, "%sbboxSize %.2f %.2f\n", indentStr,
			((Vec2f)GetGroup2D()->bboxSize).x,
			((Vec2f)GetGroup2D()->bboxSize).y);

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* Group2DProxy::Print */


// =====================================================================
