///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  GroupNodeProxy.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of a class which encapsulates all of the features
//   of a node which are common to nodes which have children.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "GroupNodeProxy.h"


// =====================================================================

//
// GroupNodeProxy::UpdateTimeSensor
//
// UpdateTimeSensor nodes which are the children of this node.
//

void GroupNodeProxy::UpdateTimeSensor()
{
	GroupNode* pGroupNode = GetGroupNode();
	for (int i = 0; i < GetGroupNode()->GetChildren().GetLength(); i++)
	{
		Traversable* trav = GetChild(i);
		trav->UpdateTimeSensor();
	}
} /* GroupNodeProxy::UpdateTimeSensor */


// =====================================================================

//
// GroupNodeProxy::PreRenderChildren
//
// Apply the PreRender method to all of the children of this node.
//

void GroupNodeProxy::PreRenderChildren(Effects const& effects)
{
	for (int i = 0; i < GetGroupNode()->GetChildren().GetLength(); i++)
		GetChild(i)->PreRender(effects);
} /* GroupNodeProxy::PreRender */


// =====================================================================

//
// GroupNodeProxy::PrintChildren
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void GroupNodeProxy::PrintChildren(FILE* outfile, int depth) const
{
	fprintf(outfile, " [\n");
	for (int i = 0; i < GetGroupNode()->GetChildren().GetLength(); i++)
		GetChild(i)->Print(outfile, depth + 1);
	fprintf(outfile, "%s]\n", IndentPrefix(depth));
} /* GroupNodeProxy::PrintChildren */


// =====================================================================

//
// GroupNodeProxy::FindTouchSensor
//
// Call OnUserEvent() for all of the children nodes in this group.
//
// If none of the children return TRUE, call FindTouchSensor() of the
// enclosing grouping node, using the base class's function.
//

void GroupNodeProxy::FindTouchSensor(EventInfo const* pEvent)
{
	BOOL trueReturned = FALSE;

	for (int i = 0; i < GetGroupNode()->GetChildren().GetLength(); i++)
		trueReturned |= GetChild(i)->OnUserEvent(pEvent);

	// If none of the children returned TRUE, look in the enclosing group
	if (!trueReturned)
		Traversable::FindTouchSensor(pEvent);
} /* GroupNodeProxy::FindTouchSensor */


// =====================================================================

//
// GroupNodeProxy::IsFinished
//
// A grouping node is only considered as finished when all of its
// children have finished.
//

BOOL GroupNodeProxy::IsFinished() const
{
	for (int i = 0; i < GetGroupNode()->GetChildren().GetLength(); i++)
		if (!GetChild(i)->IsFinished())
			return FALSE;

	return TRUE;
} /* GroupNodeProxy::IsFinished */


// ========================================================================
