///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  IndexedLineSet2DProxy.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of the proxy object used by the presenter
//   to render the IndexedLineSet2D node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef INDEXEDLINESET_PROXY_H
#define INDEXEDLINESET_PROXY_H


#include "GeometryNode.h"
#include "VisRend.h"


class IndexedLineSet2DProxy : public GeometryNode
{
public:
	PointF* m_lpPoints;
	PointF* m_shadows;
	int m_nLength;
	int m_nAllocatedLength;

public:
	IndexedLineSet2DProxy(MediaObject* pMO) : GeometryNode(pMO) 
	{
		m_nAllocatedLength = 0;
		m_nPrevColorLength = 0;
		m_nPrevCoordLength = 0;
		m_lpPoints = NULL;
		m_shadows = NULL;
		m_pColorIndex = NULL;
		m_pCoordIndex = NULL;
	}

	~IndexedLineSet2DProxy() 
	{ 
		if (m_lpPoints)
			delete[] m_lpPoints;
		if (m_shadows)
			delete[] m_shadows;
		if (m_pColorIndex)
			delete[] m_pColorIndex; 
		if (m_pCoordIndex)
			delete[] m_pCoordIndex;
	}

	// Return a pointer to the IndexedLineSet2D node that this node is the proxy for
	IndexedLineSet2D* GetIndexLineSet() const { return (IndexedLineSet2D*)GetObject(); }
	BOOL Rotated() { return m_transform.ContainsRotation(); }

	// Return a pointer to the Color object belonging to the
	// node for which this node is the proxy
	Color* GetColor() const
	{ return (Color*)(GetIndexLineSet()->color); }

	// Return a pointer to the Properties2D object belonging to the
	// Appearance2D node that this node is the proxy for.
	Coordinate2D* GetCoordinate2D() const
	{ return (Coordinate2D*)(GetIndexLineSet()->coord); }

	// Return the colorPerVertex value
	BOOL GetVertex() const
	{
		return (BOOL)(Bool)(GetIndexLineSet()->colorPerVertex);
	}

	// Functions inherited from the base class
	virtual void PreRender(Effects const& effects);
	virtual void Print(FILE* outfile, int depth) const;
	virtual void Draw(BRect const* pClipBRect);

	// Private implementation details
private:
	int* m_pColorIndex;
	int  m_nColorLength;
	int  m_nPrevColorLength;

	int* m_pCoordIndex;
	int  m_nCoordLength;
	int  m_nPrevCoordLength;
};

#endif // INDEXEDLINESET_PROXY_H
