///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  LayoutProxy.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Layout node.
//
// Created: October, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "LayoutProxy.h"


// =====================================================================

//
// PresenterCSELT::InitLayout
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitLayout(Layout* pObj)
{
	pObj->SetPresenterData(new LayoutProxy(pObj));
}


// =====================================================================

//
// LayoutProxy::PreRender
//
// Apply the PreRender method to all of the children of this node, and
// then, using their bounding boxes, change their position according to
// the layout parameters.
//

void LayoutProxy::PreRender(Effects const& effects)
{
	PreRenderChildren(effects);


} /* LayoutProxy::PreRender */


// =====================================================================

//
// LayoutProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void LayoutProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sLayout {\n", IndentPrefix(depth));

	// Print out the children of this node
	fprintf(outfile, "%schildren", IndentPrefix(depth));
	PrintChildren(outfile, depth + 1);

	// Print out the fields of this node
	const char* indentStr = IndentPrefix(depth + 1);
	fprintf(outfile, "%smode %d\n", indentStr,
			GetLayout()->mode);
	fprintf(outfile, "%swidth %.2f\n", indentStr,
			GetLayout()->width);
	fprintf(outfile, "%sheight %.2f\n", indentStr,
			GetLayout()->height);
	fprintf(outfile, "%slineSpacing %.2f\n", indentStr,
			GetLayout()->lineSpacing);

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* LayoutProxy::Print */


// =====================================================================
