// MPEG4Player.h : main header file for the MPEG4PLAYER application
//

#ifndef __AFXWIN_H__
	#error include 'defs.h' before including this file for PCH
#endif

#include "resource.h"       // main symbols
#include "App.h"
#include "Executiv.h"


class CThreadInfoDlg;


// User defined messages
#define WM_SCENE_FINISHED (WM_USER + 1)


/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp:
// See MPEG4Player.cpp for the implementation of this class
//

class CMPEG4PlayerApp : public CWinApp, public Application
{
public:
	CMPEG4PlayerApp();

	// Functions defined in the Application class
	void OnConnect(CONNECTION_RESULT result);
	void OnFinish(SERVICE_TERMINATION reason);

	// Handles the message generated when the presenter has
	// finished rendering the scene.
	void OnSceneFinished();

	// Return a pointer to the executive
	Executive* GetExecutive() const { return m_pExecutive; }

	// Called when the view is first initialised
	void CreateExecutive(HWND hWnd);

	// Read and write floating point values to the INI file
	float GetProfileFloat(LPCTSTR lpszSection, LPCTSTR lpszEntry,
						  float fDefault);
	BOOL WriteProfileFloat(LPCTSTR lpszSection, LPCTSTR lpszEntry,
						   float value);

	// Read and write the list of network address to the INI file
	void GetProfileNetworkAddresses();
	void WriteProfileNetworkAddresses();

	// Return a reference to the array of network addresses
	ZArray<LPSTR>& GetAddressList() { return m_networkAddresses; }


public:
	// The mutex is set while the file is open
	ZSemaphore m_smpFileOpen;

private:
	// The executive object
	Executive* m_pExecutive;

	// This is TRUE if a sequence should loop continuously
	BOOL m_loopContinuously;

	// This is TRUE if the screen should be cleared when the end
	// of a file is reached.
	BOOL m_clearOnEnd;

	// The timer used to update the playing time
	UINT m_timerID;

	// An array of network address that have been entered
	ZArray<LPSTR> m_networkAddresses;

	// The reason the session finished
	SERVICE_TERMINATION m_reason;


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMPEG4PlayerApp)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Implementation

	//{{AFX_MSG(CMPEG4PlayerApp)
	afx_msg void OnAppAbout();
	afx_msg void OnOptions25fps();
	afx_msg void OnUpdateOptions25fps(CCmdUI* pCmdUI);
	afx_msg void OnOptions15fps();
	afx_msg void OnUpdateOptions15fps(CCmdUI* pCmdUI);
	afx_msg void OnOptions10fps();
	afx_msg void OnUpdateOptions10fps(CCmdUI* pCmdUI);
	afx_msg void OnOptions5fps();
	afx_msg void OnUpdateOptions5fps(CCmdUI* pCmdUI);
	afx_msg void OnOptionsLoopContinuously();
	afx_msg void OnUpdateOptionsLoopContinuously(CCmdUI* pCmdUI);
	afx_msg void OnViewThreadInfo();
	afx_msg void OnViewWorldInfo();
	afx_msg void OnOptionsClearOnEnd();
	afx_msg void OnUpdateOptionsClearOnEnd(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


// Return a pointer to the current application object
inline CMPEG4PlayerApp* GetApp() { return (CMPEG4PlayerApp*)AfxGetApp(); }

// Return a pointer to the main frame window
class CMainFrame;
inline CMainFrame* GetMainFrame() { return (CMainFrame*)GetApp()->m_pMainWnd; }

// Return a pointer to the current document.
class CMPEG4PlayerDoc;
CMPEG4PlayerDoc* GetDocument();

/////////////////////////////////////////////////////////////////////////////
