///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Nodes-CSELT.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   This file contains the node table definitions.
//
// Created: October, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Im1defs.h"
#include "Nodes-CSELT.h"


const int SF3DNode :: m_nodeTable [] =
{
	-1, //QuantizationParameter :: nodeCode,
	Switch :: nodeCode,
	-1, //AnimationStream :: nodeCode,
	ColorInterpolator :: nodeCode,
	-1, //Conditional :: nodeCode,
	ScalarInterpolator :: nodeCode,
	TimeSensor :: nodeCode,
	-1, //TouchSensor :: nodeCode,
	-1, //Billboard :: nodeCode,
	-1, //Group :: nodeCode,
	-1, //Inline :: nodeCode,
	-1, //LOD :: nodeCode,
	-1, //Transform :: nodeCode,
	-1, //AmbientLight :: nodeCode,
	-1, //Background :: nodeCode,
	-1, //DirectionalLight :: nodeCode,
	-1, //ListeningPoint :: nodeCode,
	-1, //PointLight :: nodeCode,
	-1, //Shape :: nodeCode,
	-1, //Sound :: nodeCode,
	-1, //SpotLight :: nodeCode,
	-1, //ViewPoint :: nodeCode,
	-1, //Collision :: nodeCode,
	-1, //CollisionSensor :: nodeCode,
 	 0, //CoordinateInterpolator :: nodeCode,
	 0, //NormalInterpolator :: nodeCode,
	 0, //OrientationInterpolator :: nodeCode,
	 0, //PositionInterpolator :: nodeCode
	-1, //ProximitySensor :: nodeCode,
    -1, //FBA :: nodeCode,
    -1, //CompositeMap :: nodeCode,
    //official list stops here ???
	-1, //NavigationInfo :: nodeCode,
	-1, //Text :: nodeCode,
	-1, //AudioClip :: nodeCode,
	-1, //CylinderSensor :: nodeCode,
	-1, //PlaneSensor:: nodeCode,
	-1, //StringInterpolator :: nodeCode,
	-1 //ToggleInterpolator :: nodeCode,
};

const int SF2DNode :: m_nodeTable [] =
{
	-1, //QuantizationParameter :: nodeCode,
	 0, //Switch2D :: nodeCode,
	ColorInterpolator :: nodeCode,
	-1, //Conditional :: nodeCode,
	ScalarInterpolator :: nodeCode,
	TimeSensor :: nodeCode,
	TouchSensor2D :: nodeCode,
	Group2D :: nodeCode,
	-1, //Inline2D :: nodeCode,
	Layout :: nodeCode,
	-1, //LOD2D :: nodeCode,
	Scroll :: nodeCode,
	Transform2D :: nodeCode,
	XLayout :: nodeCode,
	Background2D :: nodeCode,
	Image2D :: nodeCode,
	Movie2D :: nodeCode,
	Shape2D :: nodeCode,
	Sound2D :: nodeCode,
	Position2DInterpolator :: nodeCode,
	-1, //Proximity2DSensor :: nodeCode,
        // the next is missing in the official list
	-1, //AnimationStream :: nodeCode,
};

const int SFGeometry2DNode :: m_nodeTable [] =
{
	Circle :: nodeCode,
	Curve2D :: nodeCode,
	0, //IndexedFaceSet2D :: nodeCode,
	IndexedLineSet2D :: nodeCode,
	PointSet2D :: nodeCode,
	Rectangle :: nodeCode,
	-1, //StreamingText :: nodeCode,
	Text2D :: nodeCode,
};

const int SFGeometryNode :: m_nodeTable [] =
{
	-1, //Box :: nodeCode,
	-1, //Cone :: nodeCode,
	-1, //Cylinder :: nodeCode,
	-1, //ElevationGrid :: nodeCode,
	-1, //Extrusion :: nodeCode,
	-1, //IndexedFaceSet :: nodeCode,
	-1, //IndexedLineSet :: nodeCode,
	-1, //PointSet :: nodeCode,
	-1, //Sphere :: nodeCode,
};

const int SFAudioNode :: m_nodeTable [] =
{
	-1, //AudioDelay :: nodeCode,
	-1, //AudioMix :: nodeCode,
	AudioSource :: nodeCode,
	-1, //AudioTransform :: nodeCode,
	-1, //AudioSwitch :: nodeCode,
};

const int SFTextureNode :: m_nodeTable [] =
{
	-1, //ImageTexture :: nodeCode,
	-1, //MovieTexture :: nodeCode,
	-1, //Composite2DTexture :: nodeCode,
	-1 //Composite3DTexture :: nodeCode
};

const int SFRootNode :: m_nodeTable [] =
{
	Group2D :: nodeCode,
	-1, //Group :: nodeCode,
	-1, //Layer2D :: nodeCode,
	-1, //Layer3D :: nodeCode
};

const int SFBehaviorNode :: m_nodeTable [] =
{
	-1, //AnimationStream :: nodeCode,
	ColorInterpolator :: nodeCode,
	-1, //Conditional :: nodeCode,
	ScalarInterpolator :: nodeCode,
	TimeSensor :: nodeCode,
	0, //CoordinateInterpolator :: nodeCode,
	0, //NormalInterpolator :: nodeCode,
	0, //OrientationInterpolator :: nodeCode,
	0 //PositionInterpolator :: nodeCode
};

const int SFBehavior2DNode :: m_nodeTable [] =
{
	ColorInterpolator :: nodeCode,
	-1, //Conditional :: nodeCode,
	ScalarInterpolator :: nodeCode,
	TimeSensor :: nodeCode,
	Position2DInterpolator :: nodeCode,
};

template <NodeType type> int SFNodeType<type> :: GetNodeTableSize ()
{
	return sizeof m_nodeTable / sizeof (int);
}

/*-----------------------------------------------------------------
Make sure each necessary SFxxxNode class is instantiated.
(This code is never called, it is here to make sure the linker gets
everything intantiated).
-----------------------------------------------------------------*/
void IntantiateTemplateInstances ()
{
	SF2DNode sf2D;
	SF3DNode sf3D;
	SFGeometry2DNode sfGeometry2D;
	SFGeometryNode sfGeometry;
	SFRootNode sfRoot;
	SFAudioNode sfAudio;
	SFBehaviorNode sfBehavior;
	SFBehavior2DNode sfBehavior2D;
	SFTextureNode sfTexture;
}
