///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  PointSet2DProxy.h
//
// Author: Antonio Arena, CSELT, arena@kim.cselt.it
//
// Description:
//   The definition of the proxy object used by the presenter
//   to handle the PointSet2DProxy node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef POINTSET2D_PROXY_H
#define POINTSET2D_PROXY_H


#include "GeometryNode.h"


class PointSet2DProxy : public GeometryNode
{
public:
	PointSet2DProxy(MediaObject* pMO);
	~PointSet2DProxy();

	// Return a pointer to the PointSet2D node that this node is the proxy for
	PointSet2D* GetPointSet2D() const
	{ return (PointSet2D*)GetObject(); }
	// Return a pointer to the Coordinate2D node contained in this node
	Coordinate2D* GetCoordinate2D() const
	{ return (Coordinate2D*)(GetPointSet2D()->coord); }
	// Return a pointer to the Color node contained in this node
	Color* GetColorNode() const
	{ return (Color*)(GetPointSet2D()->color); }

	// Functions inherited from the base classes
	virtual void PreRender(Effects const&effects);
	virtual void Print(FILE* outfile, int depth) const;
	virtual void Draw(BRect const* pClipBRect);


	// Private implementation details
private:
	PointF* m_lpPoints;
	PointF* m_shadowPoints;
	int m_nLength;
	int m_nAllocatedLength;
};


#endif // POINTSET2D_PROXY_H
