///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Position2DInterpolatorProxy.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Position2DInterpolator node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Position2DInterpolatorProxy.h"


// =====================================================================

//
// PresenterCSELT::InitPosition2DInterpolator
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitPosition2DInterpolator(Position2DInterpolator* pObj)
{
	pObj->SetPresenterData(new Position2DInterpolatorProxy(pObj));
}


// =====================================================================

//
// Position2DInterpolatorProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void Position2DInterpolatorProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sPosition2DInterpolator {\n", IndentPrefix(depth));
	const char* indentStr = IndentPrefix(depth + 1);

	fprintf(outfile, "%skey [", indentStr);
	int i;
	for (i = 0; i < GetPosition2DInterpolator()->key.GetLength(); i++)
		fprintf(outfile, " %.4f", (float)GetPosition2DInterpolator()->key[i]);
	fprintf(outfile, " ]\n");

	fprintf(outfile, "%skeyValue [", indentStr);
	for (i = 0; i < GetPosition2DInterpolator()->keyValue.GetLength(); i++)
		fprintf(outfile, " %.4f %.4f",
				(float)GetPosition2DInterpolator()->keyValue[i].x,
				(float)GetPosition2DInterpolator()->keyValue[i].y);
	fprintf(outfile, " ]\n");

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* Position2DInterpolatorProxy::Print */


// =====================================================================
