///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Properties2DProxy.cpp
//
// Authors: Antonio Arena, CSELT, arena@kim.cselt.it
//			Franco Casalino, CSELT, Franco.Casalino@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Properties2D node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Properties2DProxy.h"


// =====================================================================

//
// PresenterCSELT::InitProperties2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitProperties2D(Properties2D* pObj)
{
	pObj->SetPresenterData(new Properties2DProxy(pObj));
}


// =====================================================================

//
// Properties2DProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void Properties2DProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "Properties2D {\n");
	const char* indentStr = IndentPrefix(depth + 1);

	fprintf(outfile, "%sfilled %s\n", indentStr,
			BoolToStr(GetProperties2D()->filled));

	fprintf(outfile, "%sshadow %s\n", indentStr,
			BoolToStr(GetProperties2D()->shadow));

	fprintf(outfile, "%sshadpos %.2f %.2f\n", indentStr,
			(float)((Vec2f)GetProperties2D()->shadPos).x,
			(float)((Vec2f)GetProperties2D()->shadPos).y);

	fprintf(outfile, "%sdotted", indentStr);
	for (int i = 0; i < GetProperties2D()->dotted.GetLength(); i++)
		fprintf(outfile, " %d", indentStr, (int)(GetProperties2D()->dotted[i]));
	fprintf(outfile, "\n");

	fprintf(outfile, "%swidth %d\n", indentStr,
			(int)GetProperties2D()->width);

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* Properties2DProxy::Print */


// =====================================================================
