///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  RectangleProxy.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Rectangle node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef RECTANGLE_PROXY_H
#define RECTANGLE_PROXY_H


#include "GeometryNode.h"
#include "VisRend.h"


class RectangleProxy : public GeometryNode
{
public:
	RectangleProxy(MediaObject* pMO) : GeometryNode(pMO) {}

	// Return a pointer to the Rectangle node that this node is the proxy for
	Rectangle* GetRectangle() const { return (Rectangle*)GetObject(); }
	BOOL Rotated() { return m_transform.ContainsRotation(); }

	// Functions inherited from the base class
	virtual void PreRender(Effects const& effects);
	virtual void Print(FILE* outfile, int depth) const;
	virtual void Draw(BRect const* pClipBRect);
	virtual BOOL IsTransparent() const;


	// Private implementation details
private:
	PointF m_corners[4];
	PointF m_shadows[4];
	BRect m_drawRect;
	BRect m_shadowRect;
};

#endif // RECTANGLE_PROXY_H
