///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Transform2DProxy.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Scroll node.
//
// Created: October, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "ScrollProxy.h"
#include "trace.h"


// =====================================================================

//
// PresenterCSELT::InitScroll
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitScroll(Scroll* pObj)
{
	pObj->SetPresenterData(new ScrollProxy(pObj));
}


// =====================================================================

//
// ScrollProxy::PreRender
//
// Add the transform parameters of this node to the given effects, and
// then apply the PreRender method to all of the children of this node.
//

void ScrollProxy::PreRender(Effects const& effects)
{
	PreRenderChildren(effects);
} /* ScrollProxy::PreRender */


// =====================================================================

//
// ScrollProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void ScrollProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sScroll {\n", IndentPrefix(depth));

	// Print out the children of this node
	fprintf(outfile, "%schildren", IndentPrefix(depth));
	PrintChildren(outfile, depth + 1);

	// Print out the fields of this node
	const char* indentStr = IndentPrefix(depth + 1);
	fprintf(outfile, "%ssmoothScroll %s\n", indentStr,
			BoolToStr(GetScroll()->smoothScroll));
	fprintf(outfile, "%sloop %s\n", indentStr,
			BoolToStr(GetScroll()->loop));
	fprintf(outfile, "%sscrollXRate %.2f\n", indentStr,
			(float)GetScroll()->scrollXRate);
	fprintf(outfile, "%sscrollYRate %.2f\n", indentStr,
			(float)GetScroll()->scrollYRate);
	fprintf(outfile, "%swidth %.2f\n", indentStr,
			(float)GetScroll()->width);
	fprintf(outfile, "%sheight %.2f\n", indentStr,
			(float)GetScroll()->height);

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* ScrollProxy::Print */


// =====================================================================

//
// ScrollProxy::IsFinished
//
// A scroll node is only considered as finished when it has stopped
// scrolling its children, and all of its children have also finished.
//

BOOL ScrollProxy::IsFinished() const
{
	// Determine if the scroll node has finished scrolling its children


	// If it has, then check if the children have also finished
	return GroupNodeProxy::IsFinished();
} /* ScrollProxy::IsFinished */


// ========================================================================
