///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Shape2DProxy.cpp
//
// Authors: Antonio Arena, CSELT, arena@kim.cselt.it
//			Franco Casalino, CSELT, Franco.Casalino@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Shape2D node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Shape2DProxy.h"
#include "Appearance2DProxy.h"
#include "GeometryNode.h"
#include "trace.h"


// =====================================================================

//
// PresenterCSELT::InitShape2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitShape2D(Shape2D* pObj)
{
	pObj->SetPresenterData(new Shape2DProxy(pObj));
}


// =====================================================================

//
// Shape2DProxy::PreRender
//
// Add the transform parameters of this node to the given effects, and
// then apply the PreRender method to all of the children of this node.
//

void Shape2DProxy::PreRender(Effects const& effects)
{
	GeometryNode* pGeometryProxy = (GeometryNode*)(GetGeometry()->GetPresenterData());
	pGeometryProxy->SetAppearance(GetShape2D()->appearance);
	pGeometryProxy->SetShape2DProxy(this);
	pGeometryProxy->PreRender(effects);
} /* Shape2DProxy::PreRender */


// =====================================================================

//
// Shape2DProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void Shape2DProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sShape2D {\n", IndentPrefix(depth));

	fprintf(outfile, "%sappearance ", IndentPrefix(depth + 1));
	if (GetAppearance2D() == NULL)
		fprintf(outfile, "NULL\n");
	else
		((Appearance2DProxy*)(GetAppearance2D()->GetPresenterData()))->Print(outfile, depth + 1);

	fprintf(outfile, "%sgeometry ", IndentPrefix(depth + 1));
	if (GetGeometry() == NULL)
		fprintf(outfile, "NULL\n");
	else
		((GeometryNode*)(GetGeometry()->GetPresenterData()))->Print(outfile, depth + 1);

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* Shape2DProxy::Print */


// =====================================================================
