///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Sound2DProxy.cpp
//
// Authors: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Sound2D node.
//
// Created: October, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Sound2DProxy.h"


// ===================================================================

//
// PresenterCSELT::InitSound2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitSound2D(Sound2D* pObj)
{
	pObj->SetPresenterData(new Sound2DProxy(pObj));
}


// ===================================================================

//
// Sound2DProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void Sound2DProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sSound2D {\n", IndentPrefix(depth));
	const char* indentStr = IndentPrefix(depth + 1);

	fprintf(outfile, "%slocation %.2f %.2f\n", indentStr,
			((Vec2f)GetSound2D()->location).x,
			((Vec2f)GetSound2D()->location).y);

	fprintf(outfile, "%sintensity %.3f\n", indentStr,
			(float)GetSound2D()->intensity);

	fprintf(outfile, "%ssound ", indentStr);
	if (GetAudioNode() == NULL)
		fprintf(outfile, "NULL\n");
	else
		((Traversable*)(GetAudioNode()->GetPresenterData()))->Print(outfile, depth + 1);

	fprintf(outfile, "%sspatialize %s\n", indentStr,
			BoolToStr(GetSound2D()->spatialize));

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* Sound2DProxy::Print */


// ===================================================================

//
// Sound2DProxy::PreRender
//
// Call the PreRender() method of the sound node associated with this
// node.
//

void Sound2DProxy::PreRender(Effects const& effects)
{
	if (GetAudioNode() != NULL)
		((Traversable*)(GetAudioNode()->GetPresenterData()))->PreRender(effects);
} /* Sound2DProxy::PreRender */


// ===================================================================

//
// Sound2DProxy::IsFinished
//
// Return TRUE if the sound node associated with this node is finished,
// otherwise return FALSE.
//

BOOL Sound2DProxy::IsFinished() const
{
	if (GetAudioNode() != NULL)
		return ((Traversable*)(GetAudioNode()->GetPresenterData()))->IsFinished();
	else
		return TRUE;
} /* Sound2DProxy::IsFinished */


// ===================================================================
