///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Transform2DProxy.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Switch node.
//
// Created: October, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "SwitchProxy.h"
#include "trace.h"


// =====================================================================

//
// PresenterCSELT::InitTransform2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitSwitch(Switch* pObj)
{
	pObj->SetPresenterData(new SwitchProxy(pObj));
}


// =====================================================================

//
// SwitchProxy::PreRender
//
// Add the transform parameters of this node to the given effects, and
// then apply the PreRender method to all of the children of this node.
//

void SwitchProxy::PreRender(Effects const& effects)
{
	Effects newEffects = effects;

	// Go through the nodes, switching off all nodes except the given
	// numbered child.  If this switch node is already switched off,
	// then no child will be switched on.
	for (int i = 0; i < GetGroupNode()->GetChildren().GetLength(); i++)
	{
		if ((i == GetSwitch()->whichChoice) && (effects.isSwitchedOn))
			newEffects.isSwitchedOn = TRUE;
		else
			newEffects.isSwitchedOn = FALSE;

		GetChild(i)->PreRender(newEffects);
	}
} /* SwitchProxy::PreRender */


// =====================================================================

//
// SwitchProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void SwitchProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sSwitch {\n", IndentPrefix(depth));

	// Print out the children of this node
	fprintf(outfile, "%schildren [\n", IndentPrefix(depth));
	PrintChildren(outfile, depth + 1);

	// Print out the fields of this node
	fprintf(outfile, "%swhichChoice %d\n", IndentPrefix(depth + 1),
			GetSwitch()->whichChoice);

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* SwitchProxy::Print */


// =====================================================================
