///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: main.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The main console application which is used to test the audio
//   renderer class.
//
// Created: August, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "AudRend-CSELT.h"
#include "AudioSourceProxy.h"

#ifdef MAKE_TEST_APP
FILE* traceFile;
#endif


// ============================================================================

#ifndef WIN32

#include <sys/time.h>

//
// Define a GetTickCount function for machines which don't have it.
//

unsigned long GetTickCount()
{
	struct timeval currentTime;
	gettimeofday(&currentTime, NULL);

	unsigned long tickCount = currentTime.tv_sec * 1000 + currentTime.tv_usec / 1000;
	return tickCount;
} /* GetTickCount */

#endif


// ============================================================================

#ifdef sgi

//
// Define a usleep function for the SGI (doesn't seem to have one).
//

int usleep(unsigned int useconds)
{
	struct timespec delay;
	delay.tv_sec = useconds / 1000000;
	delay.tv_nsec = (useconds % 1000000) * 1000;

	return nanosleep(&delay, NULL);
} /* usleep */

#endif

// ============================================================================

//
// Get a handle to the console window that we are running in.
//

#ifdef WIN32

HWND GetConsoleHwnd()
{
	#define MY_BUFSIZE 1024 // buffer size for console window titles
	HWND hwndFound;         // this is what is returned to the caller
	char pszNewWindowTitle[MY_BUFSIZE]; // contains fabricated WindowTitle
	char pszOldWindowTitle[MY_BUFSIZE]; // contains original WindowTitle

	// fetch current window title
	GetConsoleTitle(pszOldWindowTitle, MY_BUFSIZE);

	// format a "unique" NewWindowTitle
	wsprintf(pszNewWindowTitle, "%d/%d",
			 GetTickCount(),
			 GetCurrentProcessId());

	// change current window title
	SetConsoleTitle(pszNewWindowTitle);

	// ensure window title has been updated
	Sleep(40);

	// look for NewWindowTitle
	hwndFound = FindWindow(NULL, pszNewWindowTitle);

	// restore original window title
	SetConsoleTitle(pszOldWindowTitle);

	return(hwndFound);
} /* GetConsoleHwnd */

#endif // WIN32


// ============================================================================

int main(int argc, char* argv[])
{
	// Check the command line
	if (argc < 2)
	{
		fprintf(stderr, "Usage: AudRendTest soundFile(s)\n");
		return -1;
	}

#ifdef MAKE_TEST_APP
	traceFile = fopen("trace.log", "w");
	if (traceFile == NULL)
	{
		fprintf(stderr, "AudRendTest: Error opening 'trace.log' for writing.\n");
		return -1;
	}
#endif

	// Create the audio renderer
#ifdef WIN32
	AudioRenderer* pAudRend = AudioRenderer::Create(NULL, GetConsoleHwnd());
#endif
#ifdef sgi
	AudioRenderer* pAudRend = AudioRenderer::Create(NULL, NULL);
#endif

	// Initialise the audio rendering
	pAudRend->Init();

	// Create an array of audio source nodes for the audio renderer to
	// read from.
	int numNodes = argc - 1;
	AudioSourceProxy** nodes = new AudioSourceProxy* [numNodes];
	for (int i = 0; i < numNodes; i++)
	{
		nodes[i] = new AudioSourceProxy;
		if (!nodes[i]->Init(argv[1 + i], 8000, 16, 1, pAudRend))
		{
			fprintf(stderr, "AudRendTest: Error opening '%s'.\n", argv[1]);
			return -1;
		}
	}

	// Start the audio rendering thread
	pAudRend->Start();

	// Wait for the user to press ENTER before terminating the audio renderer
	printf("Press ENTER to terminate the audio renderer: ");
	fflush(stdout);
	getchar();
	pAudRend->Terminate();

	// Delete all of the audio nodes
	for (int j = 0; j < numNodes; j++)
		delete nodes[j];
	delete [] nodes;

	return 0;
} /* main */
