///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  TouchSensor2DProxy.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of the proxy object used by the presenter
//   to handle the TouchSensor2D node.
//
// Created: October, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef TOUCH_SENSOR2D_PROXY_H
#define TOUCH_SENSOR2D_PROXY_H


#include "UserEventHandlerProxy.h"


class TouchSensor2DProxy : public UserEventHandlerProxy
{
public:
	TouchSensor2DProxy(MediaObject* pMO) : UserEventHandlerProxy(pMO)
	{
		m_mouseDown = FALSE;
		m_registered = FALSE;
		m_prevPass = 0;
	}

	// Return a pointer to the TimeSensor2D node that this node is the proxy for
	TouchSensor2D* GetTouchSensor2D() const
	{ return (TouchSensor2D*)GetObject(); }

	// Functions inherited from the base classes
	BOOL OnUserEvent(EventInfo const* pEvent);
	virtual void Print(FILE* outfile, int depth) const;
	BOOL IsFinished() const;


private:
	BOOL m_mouseDown;	// Remember the previous state of the mouse button
	BOOL m_registered;	// Remember if this is registered with presenter
	int m_prevPass;

	// These static functions and data are used to grab exclusive
	// access to movement events when the mouse button is pressed.
	static BOOL IsMovementGrabbed() { return m_movementGrabbed; }
	static void SetGrabbedOn() { m_movementGrabbed = TRUE; }
	static void SetGrabbedOff() { m_movementGrabbed = FALSE; }
	static BOOL m_movementGrabbed;
};


#endif // TOUCH_SENSOR2D_PROXY_H
