///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  TransformMatrix2D.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of a class which implements a 2-D transform
//   matrix.
//
// Created: August, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef TRANSFORM_MATRIX_2D_H
#define TRANSFORM_MATRIX_2D_H


class TransformMatrix2D
{
	//
	// Public functions
	//
public:
	
	
	// Set the matrix to the identity matrix
	void SetIdentity();

	// Set the matrix to represent the given translation
	void SetTranslation(float dx, float dy);

	// Set the matrix to represent the scale (in X and Y)
	void SetScale(float sx, float sy);

	// Set the matrix to represent the given anticlockwise rotation
	// (in radians)
	void SetRotation(float angle);

	// Return TRUE if the transform matrix specifies only translation.
	BOOL IsOnlyTranslation() const;

	// Return TRUE if the transform matrix contains some rotation.
	BOOL ContainsRotation() const;

	// Apply the transform to the given point.  This assumes that the
	// given point is placed in a column vector, and then the matrix
	// multiplication is performed.
	void Apply(float& x, float& y) const;
	void Apply(long& x, long& y) const;
	void Apply(int& x, int& y) const;

	// Multiple two matricies together, returning a third
	friend TransformMatrix2D operator* (const TransformMatrix2D& mat1,
										const TransformMatrix2D& mat2);

	// Test for equality
	friend int operator== (const TransformMatrix2D& mat1,
						   const TransformMatrix2D& mat2);

	// Test for inequality
	friend int operator!= (const TransformMatrix2D& mat1,
						   const TransformMatrix2D& mat2);

	// Print out the contents of this matrix (only intended for debugging)
	void Print(FILE* outfile) const;


	//
	// Private implementation details
	//
public:
	float m_data[3][3];
};


#endif // TRANSFORM_MATRIX_2D_H

