/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Core
FILE:       AL.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of ALManager
-------------------------------------------------------------------------*/
#include "defs.h"
#include "AL.h"
#include "Channel.h"
#include "Executiv.h"
#include "Service.h"
#include "StreamCo.h"

/*-----------------------------------------------------------------
Create a DataChannel object, configure it, and add it to a list
of channels waiting to be opened. The nESid is added to a list of
channels that we have to request when RequestChannels is called
-----------------------------------------------------------------*/
DataChannel *ALManager :: AttachStream (
	int nESid,
	MediaStream *pStream,
	ESCONFIGPARAMS *pConfigParams,
	int nExpectedAULength
)
{
	DataChannel *pChannel = NULL;
	Service *pService =
		m_pExecutive ? m_pExecutive -> GetService () :
		m_pObject -> GetService ();
	if (pStream)
	{	pChannel = new DataChannel (pService, nESid);
		pChannel -> Config (pStream, pConfigParams, nExpectedAULength);
		m_channels += pChannel;
	}
	if (m_pExecutive)
		m_ids += nESid;
	else
		pService -> RequestChannels (1, &nESid);
	return pChannel;
}

/*-----------------------------------------------------------------
Request the openning of all the channels which AttachStream was called
for
-----------------------------------------------------------------*/
void ALManager :: RequestChannels ()
{
	if (m_pExecutive -> GetService ())
		m_pExecutive -> GetService () -> RequestChannels (
			m_ids.GetLength (),
			m_ids
		);
	m_ids.Clear ();
}

/*-----------------------------------------------------------------
Pass OnConnect message to the Executive
-----------------------------------------------------------------*/
void ALManager :: OnConnect (CONNECTION_RESULT result)
{
	if (m_pExecutive)
		m_pExecutive -> OnConnect (result);
	else
		m_pObject -> OnConnect (result);
}

/*-----------------------------------------------------------------
When channel is open returns its corresponding DataObject and call the
Channel object OnChannelOpen() method.
-----------------------------------------------------------------*/
DataChannel *ALManager :: OnChannelOpen (int nESid, int nTag)
{
	for (int i = m_channels.GetLength (); --i >=0;)
		if (m_channels [i] -> GetESid () == nESid)
		{	DataChannel *pChannel = m_channels [i];
			if (pChannel)
				pChannel -> OnChannelOpen (nTag);
			return pChannel;
		}
	return NULL;
}

/*-----------------------------------------------------------------
When session is terminated delete all channels which were not attached
to Service yet, and pass the notification to Executive
-----------------------------------------------------------------*/
void ALManager :: OnFinish  (SERVICE_TERMINATION reason)
{
	/*-----------------------------------------------------------------
	Wait till initial scene is constructed before terminating the AL
	-----------------------------------------------------------------*/	
	if (m_pExecutive)
		m_pExecutive -> WaitForScene ();
	for (int i = m_channels.GetLength (); --i >= 0;)
	{	delete m_channels [i];
		m_channels.DeleteFrom (i);
	}
	if (m_pExecutive)
		m_pExecutive -> OnFinish (reason);
}
