#if !defined(_presenter_h)	// Sentry, use file only if it's not already included.
#define _presenter_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Presenter
FILE:       Presenter.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Definition of Presenter
-------------------------------------------------------------------------*/
#include "ztl.h"
#include "Im1defs.h"
#include "custom.h"

class Executive;
class VisualRenderer;
class AudioRenderer;
class Capabilities;
class MediaObject;

class Presenter
{
protected:
	VisualRenderer *m_pVisualRenderer;
	AudioRenderer *m_pAudioRenderer;
	Executive *m_pExecutive;
	MediaObject *m_pScene;

public:
	VisualRenderer *GetVisualRenderer () {return m_pVisualRenderer;}
	AudioRenderer *GetAudioRenderer () {return m_pAudioRenderer;}
	Executive* GetExecutive() {return m_pExecutive;}

	virtual void GetCapabilities (Capabilities *pPresentationCapabilities) = 0;
	virtual void Init (MediaObject *pScene) = 0;
	virtual void Terminate () = 0;
	virtual void OnUserEvent (USER_EVENT event, int x, int y, UINT uControlKeysStatus) = 0;
	virtual DWORD GetTime () = 0;
	virtual void SetTargetFrameRate(float newFrameRate) = 0;
	virtual float GetTargetFrameRate() const = 0;
	virtual float GetActualFrameRate() const = 0;

	// Return a pointer to a presenter.  Use this to construct
	// a presenterr appropriate for the current platform.  A pointer
	// to the executive, and a platform specific parameter are given.
	static Presenter* Create(Executive* pExecutive, void* pInfo = NULL);

	// We need a virtual desctructor
	virtual ~Presenter() {}
};

#endif                                      // _presenter_h sentry.
