/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Buffer management
FILE:       Clock.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of ClockReference.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Clock.h"

#ifndef WIN32

#include <sys/time.h>
#include <unistd.h>
DWORD timeGetTime ()
{
	struct timeval time;
	::gettimeofday(&time);
	return (DWORD) (time.tv_sec * 1000 + time.tv_usec / 1000);
}

void Sleep (DWORD dw)
{
	usleep ((long long) dw * 1000);
}

#endif

/*-----------------------------------------------------------------
ClockReference constructor. Save current time and granularity
-----------------------------------------------------------------*/
ClockReference :: ClockReference (int nGranularity)
{
	m_nGranularity = nGranularity;
	Reset ();
}

/*-----------------------------------------------------------------
Reset the clock to zero (but don't start it running)
-----------------------------------------------------------------*/
void ClockReference :: Reset ()
{
	m_bStarted = FALSE;
}

/*-----------------------------------------------------------------
Start the clock running
-----------------------------------------------------------------*/
void ClockReference :: Start ()
{
	m_dwStartTime = timeGetTime ();
	m_bStarted = TRUE;
}

/*-----------------------------------------------------------------
Return current value of the object
-----------------------------------------------------------------*/
DWORD ClockReference :: GetTime ()
{
	return ((m_bStarted) ? (timeGetTime () - m_dwStartTime) : 0);
}

/*-----------------------------------------------------------------
Wait till the object reaches the given time
-----------------------------------------------------------------*/
void ClockReference ::WaitOn (DWORD dwTime)
{
	long lDelta = (long) dwTime - GetTime ();

	if (lDelta > 0)
		Sleep (lDelta);
}

/*-----------------------------------------------------------------
Compare two time stamps
-----------------------------------------------------------------*/
int ClockReference :: CompareTimes (DWORD dwTime, DWORD dwCompareTo)
{
	int nResult;

	if (dwCompareTo == 0)
		dwCompareTo = GetTime ();
	nResult = (long) dwTime - (long) dwCompareTo;
	if (abs (nResult) <= m_nGranularity)
		nResult = 0;
	return nResult;
}