#if !defined(_decoder_h)	// Sentry, use file only if it's not already included.
#define _decoder_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       Decoder.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Definition of Decoder
-------------------------------------------------------------------------*/
#include "Im1defs.h"
#include "ztl.h"

class Executive;
class Capabilities;
class MediaStream;
class Root;

class Decoder
{
	Executive *m_pExecutive;
	FORMAT m_fmtMyFormat;
	MediaStream *m_pInStream, *m_pOutStream;
	ZSemaphore m_smpActive;
	Root *m_pScene;

private:
	void DecoderThread ();

protected:
	BOOL m_bActive;
	BOOL m_streamClosed;
	MediaStream *GetInputStream () {return m_pInStream;}
	MediaStream *GetOutputStream () {return m_pOutStream;}
	Root *GetRoot () {return m_pScene;}

public:
	Decoder (Executive *pExecutive);
	virtual ~Decoder () {};
	virtual FORMAT SetFormat (Capabilities *pPresentationCapabilities) = 0;
	void Start (
		MediaStream *pCodedStream,
		MediaStream *pPresentationStream = NULL
	);
	void Start (MediaStream *pCodedStream, Root *pScene)
	{
		m_pScene = pScene;
		Start (pCodedStream);
	}
	virtual void Run ();
	void Close ();
	virtual FORMAT GetFormat () {return m_fmtMyFormat;}
	virtual int GetOptimalInputSize () {return 256;}
	virtual int GetOptimalOutputSize () {return 1000;}
	virtual int GetRecommendedInputBufferSize () {return 2000;}
	virtual int GetRecommendedOutputBufferSize () {return 4000;}
	virtual BOOL Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
	) = 0;
	void CloseOpenStreams ();
};
#endif                                      // _decoder_h sentry.
