#if !defined(_executiv_h)	// Sentry, use file only if it's not already included.
#define _executiv_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Executive
FILE:       Executiv.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Definition of Executive
-------------------------------------------------------------------------*/
#include "ztl.h"
#include "Im1defs.h"
#include "custom.h"

class Presenter;
class Service;
class ALManager;
class Application;
class MediaStream;
class BIFSDecoder;
class MediaObject;
class ClockReference;
class StreamConsumer;

class Executive
{
	Presenter *m_pPresenter;
	Service *m_pService;
	ALManager *m_pAL;
	Application *m_pApp;
	MediaStream *m_pBifs;
	BIFSDecoder *m_pBifsDecoder;
	ClockReference *m_pClock;
	HWND m_hWnd;
	MediaObject *m_pScene;
	ZSemaphore m_smpExecutiveActive, m_smpWaitToStart;
	ZSemaphore m_smpExecutiveThread, m_smpOpenStreams;
	ZSemaphore m_smpSceneBuilt;
	ZMutex m_mtxScene;
	BOOL m_bDelete, m_bActive;
	int m_nOpenStreams;
	ZArray<StreamConsumer *> m_registeredNodes;
	int m_nUnattachedStreams;
	
private:
	void ExecutiveThread ();

public:
	Executive (Application *pApp, HWND hWnd);
	~Executive ();
	void Start (SERVICE_TYPE type, LPCSTR szAddress);
	void OnConnect (CONNECTION_RESULT result = CONNECTION_SUCCESS);
	void Stop ();
	void OnFinish (SERVICE_TERMINATION reason = END_OF_SERVICE);
	void Paint (void *info);
	Presenter *GetPresenter () {return m_pPresenter;}
	Service *GetService () {return m_pService;}
	ALManager *GetALManager () {return m_pAL;}
	ClockReference *GetClock () {return m_pClock;}
	BOOL LockScene (BOOL bWait = TRUE);
	void UnlockScene () {m_mtxScene.V ();}
	void StreamOpen ();
	void StreamClose ();
	void RegisterStreamConsumer (StreamConsumer *pNode);
	void UnregisterStreamConsumer  (StreamConsumer *pNode);
	void SetStream (
		int nObjectDescriptorId,
		ES_DESCRIPTOR *pES_Descriptor,
		LPCSTR szFileName
	);
	void OnUserEvent  (USER_EVENT event, int x, int y, UINT uControlKeysStatus);
	void WaitForScene () {m_smpSceneBuilt.P ();}

	CUSTOMIZE_EXECUTIVE_DECLARATION
};

#endif                                      // _executiv_h sentry.
