/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  DMIF
FILE:       FlexDmux.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of FlexDmux
-------------------------------------------------------------------------*/
#include "defs.h"
#include "FlexDmux.h"
#include "AL.h"

/*-----------------------------------------------------------------
Constructor. Save the ALManager address and update the channel
table
-----------------------------------------------------------------*/
FlexDemux :: FlexDemux (ALManager *pAL, SMT &rStreamMapTable)
{
	m_pAL = pAL;
	UpdateTable (rStreamMapTable);
}

/*-----------------------------------------------------------------
Update the channel table. Instantiate a DataChannel object for each
new channel
-----------------------------------------------------------------*/
void FlexDemux :: UpdateTable (SMT &rStreamMapTable)
{
	for (int i = rStreamMapTable.GetLength (); --i >= 0;)
	{	int nFmcNo = rStreamMapTable [i].m_nFmcNo;
		int nESid = rStreamMapTable [i].m_nESid;

		/*-----------------------------------------------------------------
		Delete a replaced or deleted entry
		-----------------------------------------------------------------*/
		if (nFmcNo < m_channelTable.GetLength ())
		{	delete m_channelTable [nFmcNo];
			m_channelTable [nFmcNo] = NULL;
		}			

		/*-----------------------------------------------------------------
		If nESid not negative instantiate a new channel
		-----------------------------------------------------------------*/
		if (nESid >= 0)
			m_channelTable.SetAt (
				nFmcNo,
				m_pAL -> OnChannelOpen (
					nESid,
					rStreamMapTable [i].m_nAssociationTag
				)
			);
	}
}

/*-----------------------------------------------------------------
A received data packet is demultiplexed and dispatched to the
proper data channel(s)
-----------------------------------------------------------------*/
void FlexDemux :: OnPacketReceived (LPBYTE pData, int nPacketLength)
{
	while (nPacketLength > 0)
	{
		/*-----------------------------------------------------------------
		Let the channel parse its unit
		-----------------------------------------------------------------*/
		int nChannel = *pData++;
		int nLength = *pData++;
		if (m_channelTable [nChannel] != NULL)
			m_channelTable [nChannel] ->
				OnPacketReceived (pData, nLength);
		nPacketLength -= nLength + 2;
	}
}