/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       Group.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of Group
-------------------------------------------------------------------------*/

#include "defs.h"
#include "Group.h"
#include "bifs.h"
#include "NodeFld.h"
#include "BifsDec.h"

/*-----------------------------------------------------------------
Init searches for the field which is flagged as 'children',
stores a reference to the field, and set 'this' as the parent of the
field
-----------------------------------------------------------------*/
void GroupNode :: Init ()
{
	int i = -1;
	do
		assert (++i < GetFieldCount ());
	while (!GetFieldTable () [i].m_bChildren);
	m_pChildren = (MFGenericNode *) GetField (i);
	m_pChildren -> SetParent (this);

	CUSTOMIZE_GROUPNODE_INITIALIZATION
}

/*-----------------------------------------------------------------
Group :: Parse() first calls Init, then calls the base class Parse
function
-----------------------------------------------------------------*/
BOOL GroupNode :: Parse (BIFSDecoder *pBifs)
{
	Init ();
	return MediaObject :: Parse (pBifs);
}

/*-----------------------------------------------------------------
Similar to Parse(), this function is called by BIFSDecoder to insert
one node into the children field of the node.  The information about 
the new node and its exact position are parsed from the BIFS stream by 
this function.   Returns TRUE if successful.
-----------------------------------------------------------------*/
BOOL GroupNode :: ParseChild (BIFSDecoder *pBifs)
{
	int nPos;
	BOOL bResult = FALSE;

	switch (pBifs -> ParseInt (2)) {
	case 0:				// insertion at a specified position
		nPos = pBifs -> ParseInt (8);
		TRACEF ("[%d] ", nPos);
		bResult = GetChildren ().InsertElement (pBifs, this, nPos);
		break;
	case 1: 			// insertion at the beginning of the field
		TRACEF ("[0] ", 0);
		bResult = GetChildren ().InsertElement (pBifs, this, 0);
		break;
	case 2: 			// insertion at the end of the field
		TRACEF ("[end] ", 0);
		bResult = GetChildren ().AppendElement (pBifs, this);
		break;
	}
	if (bResult)
	{	OnUpdate ();
		GetChildren ().OnFieldUpdate ();
	}
	return bResult;
}
