/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       H263GDec.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of H263Decoder (simulation - just copies
			the units)
-------------------------------------------------------------------------*/
#include "Defs.h"
#include "H263Dec.h"

//static FILE *fOut;

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/
FORMAT H263Decoder :: SetFormat (Capabilities *)
{
//	fOut = fopen ("test.rgb", "wb");

	m_biDst.biSize = sizeof (BITMAPINFOHEADER);
	m_biDst.biWidth = 176; 
	m_biDst.biHeight = 144; 
	m_biDst.biPlanes = 1; //always 
	m_biDst.biBitCount = 24; 
	m_biDst.biCompression = BI_RGB; //RGB_FORMAT  
	m_biDst.biSizeImage = 176 * 144 * m_biDst.biBitCount / 8; 

	m_biSrc.biSize = sizeof (BITMAPINFOHEADER);
	m_biSrc.biWidth = 176; 
	m_biSrc.biHeight = 144; 
	m_biSrc.biPlanes = 1; 
	m_biSrc.biBitCount = 24; 
	m_biSrc.biCompression = FOURCC_H263; 

	m_decodeObject.m_DecodeInst.DecFrameBuffer = NULL;
	m_decodeObject.m_DecodeInst.VDODecBuffer = NULL;
	m_decodeObject.m_RGBLow = NULL;
	m_decodeObject.m_InputFourcc = 0;
	m_decodeObject.m_OutputFourcc = 0;
	m_decodeObject.m_DoLowRes = 0;
	m_decodeObject.m_HurryUp = FALSE;
	m_decodeObject.m_pCompressedData = (LPBYTE) &m_biSrc;	// just to make it
															// non-null.

	if(!InitDecodedObject (&m_biSrc, &m_biDst, &m_decodeObject))
		return FORMAT_UNKNOWN;

	return FORMAT_RGB;
}

/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int H263Decoder :: GetOptimalOutputSize ()
{
	return 176 * 144 * 3 + 1 + 4;
}


BOOL H263Decoder :: Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
	int nError;

	if (rnOutputLength < GetOptimalOutputSize ())
	{	rnOutputLength = GetOptimalOutputSize ();
		return FALSE;
	}

	m_decodeObject.m_pCompressedData = pInput;
	m_biSrc.biSizeImage = nInputLength;
	* (WORD *) pOutput = 176;
	* (WORD *) (pOutput + sizeof (WORD)) = 144;
	nError = DecodeFrame (
		&m_biSrc,
		&m_biDst,
		pOutput + 2 * sizeof (WORD),
		&m_decodeObject,
		FALSE,
		TRUE,
		FALSE,
		1
	);
	rnOutputLength = nError == 0 ? GetOptimalOutputSize () : 0;

/*	if (nError == 0)
	{	fwrite ("\0\0\x81", 1, 3, fOut);
		fwrite (pOutput, 1, rnOutputLength, fOut);
	}
*/

	return TRUE;
}