/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       PCMDec.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of PCMDecoder
-------------------------------------------------------------------------*/
#include "Defs.h"
#include <limits.h>
#include "PCMDec.h"
#include "Caps.h"
#include "Stream.h"

/*-----------------------------------------------------------------
Set the decoder output format and initialize the decoder
-----------------------------------------------------------------*/
FORMAT PCMDecoder :: SetFormat (Capabilities *)
{
	return FORMAT_PCM;
}		

/*-----------------------------------------------------------------
Decompress a unit.
-----------------------------------------------------------------*/
BOOL PCMDecoder :: Decode (
		LPBYTE pInput,
		int /*nInputLength*/,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
	if (rnOutputLength < GetOptimalOutputSize ())
	{	rnOutputLength = GetOptimalOutputSize ();
		return FALSE;
	}
	
	memcpy (pOutput, pInput + 1, 480);
	rnOutputLength = GetOptimalOutputSize ();
	return TRUE;
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, decode them, and dispatch
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void PCMDecoder :: Run ()
{
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;

	SetThreadPriority (GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
	while (
		(pInData =
			(LPBYTE) GetInputStream () ->
				FetchNext (nInSize, dwTime, TRUE)
		) !=
		END_OF_STREAM
	)
	{
		nOutSize = GetOptimalOutputSize (),
		pOutData =
			(LPBYTE) GetOutputStream () -> Allocate (nOutSize, TRUE, FALSE);
		if (pOutData == NULL)
			throw;
		if (pOutData == END_OF_STREAM)
		{	GetInputStream () -> Terminate ();
			return;
		}
		while (!Decode (pInData, nInSize, pOutData, nOutSize))
		{	pOutData =
				(LPBYTE) GetOutputStream () -> Reallocate (nOutSize);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
			{	GetInputStream () -> Terminate ();
				return;
			}
		}
		GetOutputStream () -> Dispatch (nOutSize, m_nSamples++ * 30);
		GetInputStream () -> Release (TRUE);
	}
	GetOutputStream () -> Close ();
}
