/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  DMIF
FILE:       Service.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of Service
-------------------------------------------------------------------------*/
#include "defs.h"
#include "service.h"
#include "AL.h"
#include "FlexDmux.h"

/*-----------------------------------------------------------------
Service constructor
-----------------------------------------------------------------*/
Service :: Service (ALManager *pAL)
{
	m_pAL = pAL;
	m_nPendingChannels = 0;
}

/*-----------------------------------------------------------------
The destructor deletes all FlexDemux objects (DataChannels are deleted
by ALManager)
-----------------------------------------------------------------*/
Service :: ~Service ()
{
	for (int i = m_channelMap.GetLength(); --i >= 0;)
		if (m_channelMap [i].m_bIsFlexMux)
			delete m_channelMap [i].m_pChannel;
}

/*-----------------------------------------------------------------
Request the service to open the specified channels. Set a counter to
monitor when all the channels are open
-----------------------------------------------------------------*/
void Service :: RequestChannels (int nNumberOfChannels, const int *pESidList)
{
	m_nPendingChannels += nNumberOfChannels;
	OpenChannels (nNumberOfChannels, pESidList);
}
	
/*-----------------------------------------------------------------
Called from the other version of OnChannelOpen() when a non FlexMux
channel is opened
-----------------------------------------------------------------*/
void Service :: OnChannelOpen (
	HANDLE hChannel,
	int nAssociationTag,
	int nES_id
)
{
	/*-----------------------------------------------------------------
	Request the AL to instantiate a DataChannel object.
	Saves an association between the handle and the object.
	-----------------------------------------------------------------*/
	m_smpChannelMap.P ();
	DataChannel *pChannel =
		m_pAL -> OnChannelOpen (nES_id, nAssociationTag);
	if (pChannel)
		m_channelMap += ChannelMapEntry (hChannel, pChannel, FALSE);
	m_smpChannelMap.V ();

	/*-----------------------------------------------------------------
	Add the tag to the list of ready channels tags. If all channels
	are ready, call ChannelsReady
	-----------------------------------------------------------------*/
/*	m_readyChannels += nAssociationTag;
	if (--m_nPendingChannels == 0)
	{	ChannelsReady (m_readyChannels.GetLength (), m_readyChannels);
		m_readyChannels.Clear ();
	}*/
}

/*-----------------------------------------------------------------
Called from a derived class when a FlexMux channel is opened
-----------------------------------------------------------------*/
void Service :: OnChannelOpen (HANDLE hChannel, SMT &rStreamMapTable)
{
	/*-----------------------------------------------------------------
	If this is a non-FlexMux channel, the map table contains just one
	entry with m_nFmcNo = -1
	-----------------------------------------------------------------*/
	if (rStreamMapTable.GetLength () == 1 &&
		rStreamMapTable [0].m_nFmcNo == -1
	)
	{
		OnChannelOpen (
			hChannel,
			rStreamMapTable [0].m_nAssociationTag,
			rStreamMapTable [0].m_nESid
		);
		return;
	}

	/*-----------------------------------------------------------------
	If a FlexDemux object for this channel already exists, update
	it FMT, otherwise instantiate a new one
	-----------------------------------------------------------------*/
	m_smpChannelMap.P ();
	int i = m_channelMap.Find (ChannelMapEntry (hChannel));
	if (i >= 0)
		((FlexDemux *) m_channelMap [i].m_pChannel) ->
			UpdateTable (rStreamMapTable);
	else
		m_channelMap += ChannelMapEntry (
							hChannel,
							new FlexDemux (m_pAL, rStreamMapTable),
							TRUE
						 );
	m_smpChannelMap.V ();

	/*-----------------------------------------------------------------
	Add all the tags to the list of ready channels tags. If all channels
	are ready, call ChannelsReady
	-----------------------------------------------------------------*/
/*	for (i = 0; i < rStreamMapTable.GetLength (); i++)
	{			
		m_readyChannels += rStreamMapTable [i].m_nAssociationTag;
		--m_nPendingChannels;
	}
	if (m_nPendingChannels == 0)
	{	ChannelsReady (m_readyChannels.GetLength (), m_readyChannels);
		m_readyChannels.Clear ();
	}*/
}

/*-----------------------------------------------------------------
Pass a received data packet to the proper channel object
-----------------------------------------------------------------*/
void Service :: OnDataReceived (
	HANDLE hChannel,
	LPBYTE pData,
	int nPacketLength
)
{
	m_smpChannelMap.P ();
	for (int i = m_channelMap.GetLength (); --i >= 0;)
		if (m_channelMap [i].m_hChannel == hChannel)
		{	m_channelMap [i].m_pChannel ->
				OnPacketReceived (pData, nPacketLength);
			m_smpChannelMap.V ();
			return;
		}
	m_smpChannelMap.V ();
}
