#if !defined(_stream_h)	// Sentry, use file only if it's not already included.
#define _stream_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Buffer management
FILE:       Stream.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Definition of MediaStream.
-------------------------------------------------------------------------*/
#include <stdio.h>
#include "ztl.h"

class ClockReference;

class MediaStream
{
	int m_nBufferSize;
	LPBYTE m_pBuffer;
	int m_nAllocatedLength, m_nLockedLength;
	int m_nFreeStart, m_nTakenStart;
	ZMutex m_mtxLockStream;
	ZSemaphore m_smpLockOutBlock, m_smpWaitForMemory, m_smpWaitForData;
	ZSemaphore m_smpSendClosed, m_smpReceiveClosed;
	BOOL m_bWaitingForMemory, m_bReallocInProgress, m_bWaitingForData;
	BOOL m_bWait, m_bShove, m_bEndOfStream, m_bFirstRetrieved;
	BOOL m_bTrace;
	ClockReference *m_pClock;

	static int m_nStreamCount;
	static FILE *m_fTraceFile;
	static ZMutex m_mtxLockTraceFile;
	int m_nSN;

private:
	void Discard ();
	int GetBlockLength (int nOffset)
	{
		return LOADINT (m_pBuffer + nOffset);
	}
	DWORD GetTimeStamp (int nOffset)
		{return LOADDWORD (m_pBuffer + nOffset + sizeof (int));}
	int GetNext (int nOffset);
	void Trace (int nOffset, LPCSTR szAction);

public:
	void Terminate ();
	MediaStream (ClockReference *pClock, int nBufferSize);
	~MediaStream ();
	void Reset ();
	LPVOID Allocate (int nSize, BOOL bWait = TRUE, BOOL bShove = FALSE);
	LPVOID Reallocate (int nSize);
	BOOL Dispatch (int nSize, DWORD dwTime);
	LPVOID FetchNext (int &rnSize, DWORD &rdwTime, BOOL bWait = TRUE);
	LPVOID Fetch (
		int &rnSize,
		DWORD &rdwTime,
		BOOL &rbRecurrent,
		BOOL &rbEndOfStream,
		BOOL bSkip = TRUE,
		BOOL bWait = FALSE,
		long deDelta = 0		//ML changed from DWORD to long
	);
	void Release (BOOL bDiscard = TRUE);
	void Close ();
	void SetTraceFlag (BOOL bFlag) {m_bTrace = bFlag;}
};

#endif                                      // _stream_h sentry.
