#if !defined(_bifs_h)	// Sentry, use file only if it's not already included.
#define _bifs_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       bifs.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Declarations of macros and objects for BIFS parsing
-------------------------------------------------------------------------*/

#include "MediaObj.h"

/*-----------------------------------------------------------------
Each node declaration includes a static table with a row for each field.
This table is used to parse the contents of the field.
The table is an array of type BifsFieldTable.
The node includes virtual functions that return the table address and
its length.
-----------------------------------------------------------------*/
typedef void (MediaObject :: *EventHandler) ();

#define BIFS_FIELD_OFFSET(className, fieldName) \
	(WORD) ((size_t) ((NodeField *) & ((className *)0) -> fieldName) - \
	(size_t) (MediaObject *)((className *)0))

#define BIFS_NODE_DECLARATION \
	public: static int nodeCode; \
	private: \
	static const BifsFieldTable m_fieldTable []; \
	static const BifsBounds m_bifsBounds; \
	virtual int GetFieldCount () const; \
	virtual const BifsFieldTable *GetFieldTable () const \
		{return m_fieldTable;} \
	virtual const BifsBounds *GetBifsBounds () const \
		{return &m_bifsBounds;}

#define BIFS_NODE_START_IMPLEMENTATION(className) \
	int className :: nodeCode = CODE_##className; \
	static BifsNodeFactory<class className> bifsFactory##className; \
	const BifsFieldTable className :: m_fieldTable [] = {

#define BIFS_NULL_ENTRY {0},

#define BIFS_FIELD_ROOT(className, fieldName, event, \
	defID, inID, outID, dynID, \
	animationType, quantizationType \
) \
	BIFS_FIELD_OFFSET (className, fieldName), \
	event, \
	FALSE, \
	defID, \
	inID, \
	outID, \
	dynID, \
	animationType, \
	quantizationType

#ifdef _DEBUG

#ifdef BIFSENC
#define BIFS_DECLARE_NODE \
	BIFS_NODE_DECLARATION \
	virtual int GetNodeCode (); \
	virtual char *GetName ();
#else	// BIFSENC
#define BIFS_DECLARE_NODE \
	BIFS_NODE_DECLARATION \
	virtual char *GetName ();
#endif	// BISENC

#ifdef BIFSENC
#define BIFS_IMPLEMENT_NODE_START(className) \
	char *className :: GetName () {return #className;} \
	int className :: GetNodeCode () {return nodeCode;} \
	BIFS_NODE_START_IMPLEMENTATION (className)
#else	// BIFSENC
#define BIFS_IMPLEMENT_NODE_START(className) \
	char *className :: GetName () {return #className;} \
	BIFS_NODE_START_IMPLEMENTATION (className)
#endif	// BISENC

/*-----------------------------------------------------------------
BIFS_FIELD inserts a row in the field table.
The row contains the offset of the field (casted to its NodeField base)
from the beginning of the node class (casted to MediaObject base class).
The debug version includes a field fieldName for each node field
-----------------------------------------------------------------*/
#define BIFS_FIELD(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		0, \
		{0}, {0}, \
		#fieldName \
	},

#define BIFS_FIELD1(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, min, max \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		1, \
		{min}, {max}, \
		#fieldName \
	},

#define BIFS_FIELD2(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, min0, min1, max0, max1 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		2, \
		{min0, min1}, {max0, max1}, \
		#fieldName \
	},

#define BIFS_FIELD3(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  max0,  max1,  max2 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		3, \
		{min0, min1, min2}, {max0, max1, max2}, \
		#fieldName \
	},

#define BIFS_FIELD4(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  min3,  max0,  max1,  max2,  max3 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		4, \
		{min0, min1, min2, min3}, {max0, max1, max2, max3}, \
		#fieldName \
	},

#define BIFS_EVENT(className, fieldName, inID, dynID, \
	quantizationType, animationType \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		0, \
		{0}, {0}, \
		#fieldName \
	},

#define BIFS_EVENT1(className, fieldName, inID, dynID, \
	quantizationType, animationType, min, max \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		1, \
		{min}, {max}, \
		#fieldName \
	},

#define BIFS_EVENT2(className, fieldName, inID, dynID, \
	quantizationType, animationType, min0, min1, max0, max1 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		2, \
		{min0, min1}, {max0, max1}, \
		#fieldName \
	},

#define BIFS_EVENT3(className, fieldName, inID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  max0,  max1,  max2 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		3, \
		{min0, min1, min2}, {max0, max1, max2}, \
		#fieldName \
	},

#define BIFS_EVENT4(className, fieldName, inID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  min3,  max0,  max1,  max2,  max3 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		4, \
		{min0, min1, min2, min3}, {max0, max1, max2, max3}, \
		#fieldName \
	},

#define BIFS_CHILDREN(className, fieldName, defID, inID,  outID,  dynID) \
	{ \
		BIFS_FIELD_OFFSET (className, fieldName), \
		NULL, \
		TRUE, \
		defID, inID, outID, dynID, \
		-1, -1, \
		0, \
		{0}, {0}, \
		#fieldName \
	},

#else

#define BIFS_DECLARE_NODE  BIFS_NODE_DECLARATION

#define BIFS_IMPLEMENT_NODE_START(className) \
	BIFS_NODE_START_IMPLEMENTATION (className)

/*-----------------------------------------------------------------
Don't include field fieldName in non-debug version
-----------------------------------------------------------------*/
#define BIFS_FIELD(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		0, \
	},

#define BIFS_FIELD1(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, min, max \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		1, \
		{min}, {max}, \
	},

#define BIFS_FIELD2(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, min0, min1, max0, max1 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		2, \
		{min0, min1}, {max0, max1}, \
	},

#define BIFS_FIELD3(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  max0,  max1,  max2 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		3, \
		{min0, min1, min2}, {max0, max1, max2}, \
	},

#define BIFS_FIELD4(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  min3,  max0,  max1,  max2,  max3 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, animationType \
		), \
		4, \
		{min0, min1, min2, min3}, {max0, max1, max2, max3}, \
	},

#define BIFS_EVENT(className, fieldName, inID, dynID, \
	quantizationType, animationType \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		0, \
	},

#define BIFS_EVENT1(className, fieldName, inID, dynID, \
	quantizationType, animationType, min, max \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		1, \
		{min}, {max}, \
	},

#define BIFS_EVENT2(className, fieldName, inID, dynID, \
	quantizationType, animationType, min0, min1, max0, max1 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		2, \
		{min0, min1}, {max0, max1}, \
	},

#define BIFS_EVENT3(className, fieldName, inID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  max0,  max1,  max2 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		3, \
		{min0, min1, min2}, {max0, max1, max2}, \
	},

#define BIFS_EVENT4(className, fieldName, inID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  min3,  max0,  max1,  max2,  max3 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, (EventHandler) On##fieldName, \
			-1, inID, -1, dynID, \
			quantizationType, animationType \
		), \
		4, \
		{min0, min1, min2, min3}, {max0, max1, max2, max3}, \
	},

#define BIFS_CHILDREN(className, fieldName, defID, inID,  outID,  dynID) \
	{ \
		BIFS_FIELD_OFFSET (className, fieldName), \
		NULL, \
		TRUE, \
		defID, inID, outID, dynID, \
		-1, -1, \
		0 \
	},

#endif

#define BIFS_IMPLEMENT_NODE_END(className) \
	}; \
	int className :: GetFieldCount () const \
		{return sizeof m_fieldTable / sizeof (BifsFieldTable);} \
	const BifsBounds className :: m_bifsBounds ( \
		m_fieldTable, \
		sizeof m_fieldTable / sizeof (BifsFieldTable) \
	);

/*-----------------------------------------------------------------
The field table defines in nodes contains:
1. An offset of the field within the node class
2. Event handling function address for eventIns.
3. (Debug mode only) the field fieldName
-----------------------------------------------------------------*/
#ifdef WIN32
#pragma pack (push, 2)
#endif

struct BifsFieldTable
{
	WORD m_offset;
	EventHandler m_function;
	BYTE m_bChildren;
	char m_defID, m_inID, m_outID, m_dynID;
	char m_AnimationType, m_nQuantizationType;
	BYTE m_nDimension;
	float m_fMin [3], m_fMax [3];
#ifdef _DEBUG
	LPCSTR m_szName;
#endif
};

struct BifsBounds
{
	int m_nTotalDefs, m_nTotalIns, m_nTotalOuts, m_nTotalDyns;
	int m_nBitsPerDef, m_nBitsPerIn, m_nBitsPerOut, m_nBitsPerDyn;
	BifsBounds (const BifsFieldTable *pTable, int nSize);
};

#ifdef WIN32
#pragma pack (pop)
#endif

#endif                                      // _bifs_h sentry.
