/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  all
FILE:       main.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   main() function for testing.
-------------------------------------------------------------------------*/
#include "defs.h"
#include <stdio.h>
#include "Executiv.h"
#include "ztl.h"
#include "App.h"

//#define SECOND_TRY

/*-----------------------------------------------------------------
The application body is derived from Application
-----------------------------------------------------------------*/
class MyApp : public Application
{
	LPCSTR m_szFileName;
	ZSemaphore m_smpDemuxWorking;
	Executive m_myExecutive;

	virtual void OnConnect (
		CONNECTION_RESULT result = CONNECTION_SUCCESS
	);
	virtual void OnFinish (
		SERVICE_TERMINATION reason = END_OF_SERVICE
	);

public:
	MyApp () : m_smpDemuxWorking (1, 0), m_myExecutive (this, NULL) {}
	void Run (LPCSTR szFileName);
};


/*-----------------------------------------------------------------
Instaniates a derivation from Application, run it,
and waits till Application :: Finish() is called.
-----------------------------------------------------------------*/
int main (int argc, char **argv)
{
	MyApp myApp;

	if (argc < 2)
	{	printf ("Missing file name\n");
		return -1;
	}

	myApp.Run (*++argv);

#ifdef SECOND_TRY
	Sleep (100);
	myApp.Run (*argv);
#endif

	return 0;
}

/*-----------------------------------------------------------------
Run a session and wait on a semaphore till it finishes
-----------------------------------------------------------------*/
void MyApp :: Run (LPCSTR szFileName)
{
	m_szFileName = szFileName;
	m_myExecutive.Start (FILE_SERVICE, szFileName);
	m_smpDemuxWorking.P ();
}

/*-----------------------------------------------------------------
If could not start reading the file display a message and release the
semaphore
-----------------------------------------------------------------*/
void MyApp :: OnConnect (CONNECTION_RESULT result)
{
	if (result != CONNECTION_SUCCESS)
	{	printf ("Cannot open %s\n", m_szFileName);
		m_smpDemuxWorking.V ();
	}
}

/*-----------------------------------------------------------------
Release the semaphore when finished
-----------------------------------------------------------------*/
void MyApp :: OnFinish (SERVICE_TERMINATION)
{
	m_smpDemuxWorking.V ();
}
