#if !defined(_ztype_h)	// Sentry, use file only if it's not already included.
#define _ztype_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Compositor
FILE:       ztype.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   class definitions for types of node fields
-------------------------------------------------------------------------*/

/*-----------------------------------------------------------------
Bool
-----------------------------------------------------------------*/
class Bool
{
	BOOL m_b;

public:
	Bool (BOOL b = FALSE) : m_b (b) {}
	operator BOOL () const {return m_b;}
	BOOL operator = (BOOL b) {return m_b = b;}
	BOOL operator += (BOOL b) {return m_b += b;}
	BOOL operator -= (BOOL b) {return m_b -= b;}
	BOOL operator &= (BOOL b) {return m_b &= b;}
	BOOL operator |= (BOOL b) {return m_b |= b;}
	BOOL operator ^= (BOOL b) {return m_b ^= b;}
	BOOL operator ++ () {return ++m_b;}
	BOOL operator ++ (BOOL) {return m_b++;}
};

/*-----------------------------------------------------------------
Time
-----------------------------------------------------------------*/
typedef double Time;

/*-----------------------------------------------------------------
Vec3f
-----------------------------------------------------------------*/
class Vec3f
{
public:
	float x, y, z;

public:
	Vec3f (float xx = 0, float yy = 0, float zz = 0) :
		x (xx), y (yy), z(zz)
	{}
	BOOL operator == (const Vec3f &vec) const
	{
		return x == vec.x && y == vec.y && z == vec.z;
	}
};

/*-----------------------------------------------------------------
Vec3f
-----------------------------------------------------------------*/
class Vec2f
{
public:
	float x, y;

public:
	Vec2f (float xx = 0, float yy = 0) :
		x (xx), y (yy)
	{}
	BOOL operator == (const Vec2f &vec) const
	{
		return x == vec.x && y == vec.y;
	}
};

/*-----------------------------------------------------------------
Rotation
-----------------------------------------------------------------*/
class Rotation
{
public:
	float x, y, z, theta;

public:
	Rotation (float xx = 0, float yy = 0, float zz = 0, float th = 0) :
		x (xx), y (yy), z (zz), theta (th)
	{}
	BOOL operator == (const Rotation &vec) const
	{
		return
			x == vec.x && y == vec.y && z == vec.z && theta == vec.theta;
	}
};

/*-----------------------------------------------------------------
Rotation2D
-----------------------------------------------------------------*/
class Rotation2D
{
public:
	float x, y, theta;

public:
	Rotation2D (float xx = 0, float yy = 0, float th = 0) :
		x (xx), y (yy), theta (th)
	{}
	BOOL operator == (const Rotation2D &vec) const
	{
		return
			x == vec.x && y == vec.y && theta == vec.theta;
	}
};

/*-----------------------------------------------------------------
Color
-----------------------------------------------------------------*/
class Color3f
{
public:
	float r, g, b;

public:
	Color3f (float rr = 0, float gg = 0, float bb = 0) :
		r (rr), g (gg), b(bb)
	{}
	BOOL operator == (const Color3f &color) const
	{
		return r == color.r && g == color.g && b == color.b;
	}
};

#endif                                      // _ztype_h sentry.
