@echo off

REM
REM   This script is used to do the BIFS encoding and multiplexing
REM   in the subdirecties.  It also copies the resulting output files
REM   into the ..\CSELT\Samples
REM
REM   Matthew Leditschke
REM   CSELT
REM

REM  Get the name of the scene (compulsory argument)
if .%1==. goto DisplayUsage
set SCENE=%1

REM  Get the name of the output file (compulsory argument)
if .%2==. goto DisplayUsage
set OUTPUT=%2

REM  Get the base directory name (optional argument)
if .%3==. goto DefaultBaseDir
set BASE=%3
goto GotBaseDir

:DefaultBaseDir
set BASE=..

:GotBaseDir


echo *** Converting the scene description from text to BIFS ***
%BASE%\BifsEnc %SCENE%.txt
%BASE%\perl -w %BASE%\LookForErrors.perl %SCENE%.lst


echo.
echo *** Multiplexing using Zvi's mux ***
%BASE%\MuxZvi %OUTPUT%.scr
copy %OUTPUT%.mp4 %OUTPUT%-z.mpg4
del %OUTPUT%.mp4


echo.
echo *** Multiplexing using Angelika's mux ***
%BASE%\MuxAngelika -o muxed-stream -i mux-info.log -t mux-time.log MuxCommand.txt
REM del mux.log
copy muxed-stream.1 %OUTPUT%-a.mpg4
del muxed-stream.1


echo.
echo.
echo *** Copying the final output stream to the Samples directory ***
copy %OUTPUT%-a.mpg4 %BASE%\..\CSELT\Samples\
copy %OUTPUT%-z.mpg4 %BASE%\..\CSELT\Samples\


REM  Determine if a description file needs to be copied (optional argument)
if .%4==. goto Finished
copy %4 %BASE%\..\CSELT\Samples\


:Finished
echo.
echo *** Finished ***
echo.
@pause
goto end


:DisplayUsage
echo usage: DoMultiplex scene-name output-name [base-dir] [description]
echo.
echo This assumes that the following files exist:
echo   {scene-name}.txt    - the textual description of the scene
echo   {output-name}.scr    - the script file for Zvi'm multiplexer
echo   MuxCommand.txt      - the config file for Angelika's multiplexer
echo.
echo The file {output-name}.txt is copied to the Samples directory, which
echo can be used to describe the scene.
echo.
echo The base directory is the offset from the directory in which this
echo script is run to the directory which contains this script.  The
echo default value is ..
echo.
echo The description argument is the name of a file to copy into the
echo Samples directory which describes what this scene is doing.
echo.

:end
