/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       BifsStream.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of BifsStream, the class that handles BIFS streams
input streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "BifsStream.h"

/*-----------------------------------------------------------------
BifsStream constructor loads first character
-----------------------------------------------------------------*/
BifsStream :: BifsStream (FILE *fStream, const char*) : Stream (fStream)
{
	m_cNext = getc ();
}

/*-----------------------------------------------------------------
BIFS time stamps in this version are all zeros
-----------------------------------------------------------------*/
long BifsStream :: GetTimeStamp ()
{
	return m_cNext == EOF ? -1 : 0;
}

/*-----------------------------------------------------------------
GetChar returns a character and read one char ahead
-----------------------------------------------------------------*/
int BifsStream :: GetChar ()
{
	if (m_cNext == EOF)
		return -1;
	int c = m_cNext;
	m_cNext = getc ();
	return c;
}

static StreamFactory<BifsStream> bifsFactory ("BIFS");