/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       H263.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of H263Stream, the class that handles H263
input streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "H263.h"

/*-----------------------------------------------------------------
H263Stream constructor reads first 3 character
-----------------------------------------------------------------*/
H263Stream :: H263Stream (FILE *fStream, const char* streamData) : Stream (fStream)
{
	m_cNext [0] = getc ();
	m_cNext [1] = getc ();
	m_cNext [2] = getc ();
	m_lFrames = 0;
	m_bStart = TRUE;

	// The stream data specifies the frame rate to use (default to 15)
	if ((streamData == NULL) || (streamData[0] == 0))
		m_frameRate = 15;
	else
	{
		m_frameRate = atof(streamData);
		if (m_frameRate < 0.0001)
		{
			fprintf(stderr, "The H263 frame rate must be greater than 0.\n");
			exit(1);
		}
	}
}

/*-----------------------------------------------------------------
Time stamps are calculated according to the rate given by the
streamData field
-----------------------------------------------------------------*/
long H263Stream :: GetTimeStamp ()
{
	if (m_cNext [0] == EOF)
		return -1;
	long lTime = (long)((float)(m_lFrames * 1000) / m_frameRate);
	++m_lFrames;
	return lTime;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char. We are always
3 chars ahead, which are used to check whether this is end of AU.
-----------------------------------------------------------------*/
int H263Stream :: GetChar ()
{
	if (!m_bStart &&
		(	m_cNext [0] == EOF ||
			m_cNext [0] == 0 && m_cNext [1] == 0 &&
			(m_cNext [2] & 0x8c) == 0x80
		)
	)
	{	m_bStart = TRUE;
		return -1;
	}
	int c = m_cNext [0];
	m_cNext [0] = m_cNext [1];
	m_cNext [1] = m_cNext [2];
	m_cNext [2] = getc ();
	m_bStart = FALSE;
	return c;
}

static StreamFactory<H263Stream> bifsFactory ("visual/H263");