#include "defs.h"
#include "Mpg4Vid.h"

/*-----------------------------------------------------------------
Mpg4VidStream constructor reads first 3 character
-----------------------------------------------------------------*/
Mpg4VidStream :: Mpg4VidStream (FILE *fStream, const char* streamData) : Stream (fStream)
{
	m_cNext [0] = getc ();
	m_cNext [1] = getc ();
	m_cNext [2] = getc ();
	m_cNext [3] = getc ();
	m_lFrames = 0;
	m_firstStartCode = TRUE;
	m_returnedStartCodeLastCall = FALSE;

	// The stream data specifies the frame rate to use (default to 15)
	if ((streamData == NULL) || (streamData[0] == 0))
		m_frameRate = 15;
	else
	{
		m_frameRate = atof(streamData);
		if (m_frameRate < 0.0001)
		{
			fprintf(stderr, "The H263 frame rate must be greater than 0.\n");
			exit(1);
		}
	}
}

/*-----------------------------------------------------------------
Time stamps are calculated according to the rate given by the
streamData field
-----------------------------------------------------------------*/
long Mpg4VidStream :: GetTimeStamp ()
{
	if (m_cNext [0] == EOF)
		return -1;
	long lTime = (long)((float)(m_lFrames * 1000) / m_frameRate);
	++m_lFrames;
	return lTime;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char. We are always
4 chars ahead, which are used to check whether this is end of AU.
-----------------------------------------------------------------*/
int Mpg4VidStream :: GetChar ()
{
	if (m_cNext[0] == EOF)
		return -1;

	if (!m_returnedStartCodeLastCall)
	{
		if ((m_cNext[0] == 0) && (m_cNext[1] == 0) &&
			(m_cNext[2] == 1) && (m_cNext[3] == 0x73))
		{
			// Ignore the first start code
			if (m_firstStartCode)
				m_firstStartCode = FALSE;
			else
			{
				m_returnedStartCodeLastCall = TRUE;
				return -1;
			}
		}
	}

	int c = m_cNext [0];
	m_cNext [0] = m_cNext [1];
	m_cNext [1] = m_cNext [2];
	m_cNext [2] = m_cNext [3];
	m_cNext [3] = getc ();
	m_returnedStartCodeLastCall = FALSE;

	return c;
}

static StreamFactory<Mpg4VidStream> bifsFactory ("visual/MPEG4");