/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       Mux.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of Mux, the object that performs the multiplexing
-------------------------------------------------------------------------*/

#include "defs.h"
#include "Mux.h"
#include "MuxOD.h"
#include "Token.h"

void Mux :: Start ()
{
	/*-----------------------------------------------------------------
	Parse the script file which should contain [special version of]
	object descriptors
	-----------------------------------------------------------------*/
	ZArray<MuxObjectDescriptor> descriptors;
	int nFMC = 0;
			
	while (nextToken != TID_EOF)
		if (!(++descriptors).Parse (m_fMP4, nFMC++))
		{	printf ("Hit Enter to continue\x07\n");
			getchar ();
			return;
		}

	/*-----------------------------------------------------------------
	Write the stream map table
	-----------------------------------------------------------------*/
	fputc (descriptors.GetLength (), m_fMP4);
	for (int i = 0; i < descriptors.GetLength (); i++)
	{	fputc (0, m_fMP4);
		int nESid = (descriptors [i].GetODid () << 6) | 1;
		fputc (HIBYTE (nESid), m_fMP4);
		fputc (LOBYTE (nESid), m_fMP4);
		fputc (i, m_fMP4);
	}
		
	/*-----------------------------------------------------------------
	Multiplexing is done by finding the AU with the lower time stamp and 
	appending it to the muxed file. We iterate till all files reach EOF
	-----------------------------------------------------------------*/
	while (descriptors.GetLength () > 0)
	{
		int index = -1;
		long lLowestTime = INT_MAX;

		for (int i = 0; i < descriptors.GetLength (); i++)
		{	long lTS = descriptors [i].GetTimeStamp ();
			if (lTS == -1)		// means EOF
			{	descriptors.DeleteFrom (i);
				break;
			}
			lTS += descriptors [i].GetStartTime ();
			if (lTS < lLowestTime)
			{	index = i;
				lLowestTime = lTS;
			}
		}

		/*-----------------------------------------------------------------
		Store one AU of the proper stream (unless one of the streams
		reached EOF)
		-----------------------------------------------------------------*/
		if (index >= 0)
			descriptors [index].Store ();
	}
}
